y5.require(['Strings', 'Types'], function () {

var partitiveRegexp = /\%e[^\%]*\%B/,
    stringParseRegexp = /((\d+):(\d+):?(\d+)?)?\s?((\d+)[-.\/,](\d+)[-.,\/,](\d+))\s?((\d+):(\d+):?(\d+)?)?/,
    toFormatStrings = ['Y', 'y', 'm', 'n', 'B', 'b', 'd', 'e', 'H', 'M', 'S'],
    toFormatRegexp = {},
    i, toFormatStringsLen = toFormatStrings.length;

for (i=0; i<toFormatStringsLen; i++) {
    toFormatRegexp[toFormatStrings[i]] = new RegExp('%'+toFormatStrings[i], 'g');
}

/**
 * Функции для работы с датами. 
 * @class Содержит функции для работы с датами.
 * @static
 * @name y5.Dates
 */
y5.Dates = {

    /**
     * Дни недели
     * @name y5.Dates.days
     * @memberOf y5.Dates
     * @type Object
     */
    days: {Sunday: 0, Monday: 1, Tuesday: 2, Wednesday: 3, Thursday: 4, Friday: 5, Saturday: 6},
    
    /**
     * Однобуквенные названия дней недели (П, В, С, Ч, П, С, В)
     * @name y5.Dates.dayNames
     * @memberOf y5.Dates
     * @type Array
     */
    dayNames: ['\u041f', '\u0412', '\u0421', '\u0427', '\u041f', '\u0421', '\u0412'],
    
    /**
     * Двухбуквенные названия дней недели (Пн, Вт, Ср, Чт, Пт, Сб, Вс)
     * @name y5.Dates.dayNames2
     * @memberOf y5.Dates
     * @type Array
     */
    dayNames2: ['\u041F\u043D', '\u0412\u0442', '\u0421\u0440', '\u0427\u0442', '\u041F\u0442', '\u0421\u0431', '\u0412\u0441'],
    
    /**
     * Полные названия месяцев ("Январь" ... "Декабрь")
     * @name y5.Dates.monthNames
     * @memberOf y5.Dates
     * @type Array
     */
    monthNames: ['\u042f\u043d\u0432\u0430\u0440\u044c', '\u0424\u0435\u0432\u0440\u0430\u043b\u044c', '\u041c\u0430\u0440\u0442', '\u0410\u043f\u0440\u0435\u043b\u044c', '\u041c\u0430\u0439', '\u0418\u044e\u043d\u044c', '\u0418\u044e\u043b\u044c', '\u0410\u0432\u0433\u0443\u0441\u0442', '\u0421\u0435\u043d\u0442\u044f\u0431\u0440\u044c', '\u041e\u043a\u0442\u044f\u0431\u0440\u044c', '\u041d\u043e\u044f\u0431\u0440\u044c', '\u0414\u0435\u043a\u0430\u0431\u0440\u044c'],
    
    /**
     * Трехбуквенные названия месяцев ('Янв ... Дек')
     * @name y5.Dates.monthShortNames
     * @memberOf y5.Dates
     * @type Array
     */
    monthShortNames: ['\u042f\u043d\u0432', '\u0424\u0435\u0432', '\u041c\u0430\u0440', '\u0410\u043f\u0440', '\u041c\u0430\u0439', '\u0418\u044e\u043d', '\u0418\u044e\u043b', '\u0410\u0432\u0433', '\u0421\u0435\u043d', '\u041e\u043a\u0442', '\u041d\u043e\u044f', '\u0414\u0435\u043a'],
    
    /**
     * Полные названия месяцев в родительном падеже ("января" ... "декабря")
     * @name y5.Dates.monthNamesPartitive
     * @memberOf y5.Dates
     * @type Array
     */
    monthNamesPartitive: ['\u044f\u043d\u0432\u0430\u0440\u044f', '\u0444\u0435\u0432\u0440\u0430\u043b\u044f', '\u043c\u0430\u0440\u0442\u0430', '\u0430\u043f\u0440\u0435\u043b\u044f', '\u043c\u0430\u044f', '\u0438\u044e\u043d\u044f', '\u0438\u044e\u043b\u044f', '\u0430\u0432\u0433\u0443\u0441\u0442\u0430', '\u0441\u0435\u043d\u0442\u044f\u0431\u0440\u044f', '\u043e\u043a\u0442\u044f\u0431\u0440\u044f', '\u043d\u043e\u044f\u0431\u0440\u044f', '\u0434\u0435\u043a\u0430\u0431\u0440\u044f'],

    /**
     * Форматирует дату согласно заданному формату.
     * Формат передается в виде "%Y.%m.%d %H:%M:%S".
     *   Y - год (4 цифры)
     *   y - год (2 цифры)
     *   m - месяц (2 цифры)
     *   n - месяц без предшествующего нуля
     *   B - месяц (полное имя, например, Январь)
     *   b - месяц (короткое имя, например, Янв)
     *   d - день месяца (2 цифры)
     *   e - день месяца без предшествующего нуля
     *   H - часы (2 цифры от 00 до 23)
     *   M - минуты (2 цифры)
     *   S - секунды (2 цифры)
     * Внимание: шаблон "%e %B" возвращает имя месяца в родительном падеже.
     * @name y5.Dates.toFormat
     * @memberOf y5.Dates
     * @function
     * @param {Date} date Дата
     * @param {String} dateFormat Формат
     * @returns {String} Результат обработки
     */
    toFormat: function (date, dateFormat) {
        var month = (partitiveRegexp.test(dateFormat) ? this.monthNamesPartitive : this.monthNames),
            formats = [
                [toFormatRegexp.Y, '%04d', date.getFullYear()],
                [toFormatRegexp.y, '%02d', date.getFullYear() % 100],
                [toFormatRegexp.m, '%02d', date.getMonth() + 1],
                [toFormatRegexp.n,   '%d', date.getMonth() + 1],
                [toFormatRegexp.d, '%02d', date.getDate()],
                [toFormatRegexp.e,   '%d', date.getDate()],
                [toFormatRegexp.H, '%02d', date.getHours()],
                [toFormatRegexp.M, '%02d', date.getMinutes()],
                [toFormatRegexp.S, '%02d', date.getSeconds()],
                [toFormatRegexp.B,   '%s', month[date.getMonth()]],
                [toFormatRegexp.b,   '%s', this.monthShortNames[date.getMonth()]]
            ],
            i,
            l = formats.length,
            format = null;
        for (i = 0; i < l; i++){
            format = formats[i];
            dateFormat = dateFormat.replace(format[0], y5.Strings.printf(format[1], [format[2]]));
        }
        formats = null;
        return dateFormat;
    },

    /**
     * Меняет месяц.
     * Если месяц имеет 31 день, а новый - 30, то дата будет установлена в 30-е число.
     * @name y5.Dates.changeMonth
     * @memberOf y5.Dates
     * @function
     * @param {Date} date Дата
     * @param {Number} shift Смещение
     * @returns {Date} Измененная дата
     */
    changeMonth: function (date, shift) {
        date = this.clone(date);
        var month = date.getMonth() + shift,
            // приводим month к положительному числу и вычисляем остаток от деления на 12
            expectedMonth = expectedMonth>0 ? month%12 : (month+(12*Math.ceil(Math.abs(month/12))))%12;
        date.setMonth(month);
        if (expectedMonth != date.getMonth()) {
            date.setDate(0);
        }
        return date;
    },

    /**
     * Меняет год.
     * Если месяц имеет 31 день, а новый - 30, то дата будет установлена в 30-е число.
     * @name y5.Dates.changeYear
     * @memberOf y5.Dates
     * @function
     * @param {Date} date Дата
     * @param {Number} shift Смещение
     * @returns {Date} Измененная дата
     */
    changeYear: function (date, shift) {
        date = this.clone(date);
        var month = date.getMonth();
        date.setFullYear(date.getFullYear() + shift);
        if (month != date.getMonth()) {
            date.setDate(0);
        }
        return date;
    },
    
    /**
     * Преобразует текстовое представление даты в объект Date. При ошибке преобразования вернет NaN
     * @param {String} string Текстовое представление даты.
     * @name y5.Dates.strToTime
     * @memberOf y5.Dates
     * @function
     * @returns {Date | NaN} Дата или NaN
     */
    strToTime: function (string) {
    /**
     * Функция успешно обрабатывает ISO-даты и различные записи (m/d/y, m,d,y, m-d-y) вместе с временем.
     * Поэтому можно сделать parse, если не получается, то меняем разделители на / и m<->d
     * Но различные браузеры поддерживают разное количество форматов и по-разному их интерпретируют.
     * 
     * В данной реализации, опять же, можно выделять д,м,г и отправлять их в parse, но снова проблема интерпретации в разных браузерах.
     */
        var _this=this, date = NaN, y=0, m=0, d=0, h=0, i=0, s=0, match = string.match(stringParseRegexp);
        if (match && match[5]) {
            
            // yyyy.mm.dd
            if (match[6].length == 4) {
                y = parseInt(match[6], 10);
                if (1<=match[7] && match[7]<=12) {
                    m = parseInt(match[7], 10);
                    d = parseInt(match[8], 10);
                } else {
                    m = parseInt(match[8], 10);
                    d = parseInt(match[7], 10);
                }
                
            // dd.mm.yyyy
            } else if (match[8].length == 4) {
                y = parseInt(match[8], 10);
                if (1<=match[7] && match[7]<=12) {
                    m = parseInt(match[7], 10);
                    d = parseInt(match[6], 10);
                } else {
                    m = parseInt(match[6], 10);
                    d = parseInt(match[7], 10);
                }
                
            // dd.mm.yy
            } else {
                if (1<=match[6] && match[6]<=31) {
                    d = parseInt(match[6], 10);
                    m = parseInt(match[7], 10);
                    y = parseInt(match[8], 10);
                } else {
                    d = parseInt(match[8], 10);
                    m = parseInt(match[7], 10);
                    y = parseInt(match[6], 10);
                }
                y = y<70?2000+y:1900+y;
            }
            
            if (match[1]) {
                h = parseInt(match[2], 10);
                i = parseInt(match[3], 10);
                s = parseInt(match[4], 10);
                
                
            } else if (match[9]) {
                h = parseInt(match[10], 10);
                i = parseInt(match[11], 10);
                s = parseInt(match[12], 10);
            }
            //если нет секунд, то 0
            s = isNaN(s)?0:s;
            date = new Date(y,m-1,d,h,i,s);
        
        //28 февраля 2008
        //28 фев 2008	
        } else if ( (match = string.match(/(\d+)\s+(.*)\s+(\d+)/)) ) {
            
            d = parseInt(match[1], 10);
            y = parseInt(match[3], 10);
            m = searchMonth(match[2], 10);
            
            if (!isNaN(d) && !isNaN(y) && m != -1) {
                date = new Date();
                date.setDate(d);
                date.setMonth(m);
                date.setYear(y);
            }
            
        //28 февраля
        //01 апр
        } else if ( (match = string.match(/(\d+)\s+(.*)/)) ) {
            
            d = parseInt(match[1], 10);
            m = searchMonth(match[2]);
            if (!isNaN(d) && m != -1) {
                date = new Date();
                date.setDate(d);
                date.setMonth(m);
            }
        }
        
        function searchMonth (monthString) {
            monthString = monthString.toLowerCase();
            var arrays4Search = ['monthNamesPartitive', 'monthShortNames', 'monthNames'], i, n = -1, j;
            for (i=0; i<3; i++) {
                for (j=0; j<12; j++) {
                    if (monthString == _this[arrays4Search[i]][j].toLowerCase()) {
                        n = j;
                        break;
                    }
                }
                if (n != -1) {
                    break;
                }
            }  
            return n;
        } 
        
        return date;
    },
    
    /**
     * Возвращает количество дней в месяце.
     * @name y5.Dates.getMonthDays
     * @memberOf y5.Dates
     * @function
     * @param {Number | Date} month Номер месяца (0..11) или объект Date
     * @param {Number} [year] year Год. Указывается только вместе с номером месяца
     * @returns {Number}
     * 
     * @example
     * y5.Dates.getMonthDays(1, 2008);
     * // -> 29
     */
    getMonthDays: function (month, year) {
        if (y5.Types.date(month)) {
            year = month.getYear();
            month = month.getMonth();
        }
        var days = [31,((year % 4 == 0) ? 29 : 28),31,30,31,30,31,31,30,31,30,31];
        return days[month];
    },
    
    /**
     * Возвращает разницу между firstDay и currentDay.
     * Также может использоваться для вычисления какой день недели currentDay, если первый firstDay.
     * @name y5.Dates.getDayOffset
     * @memberOf y5.Dates
     * @function 
     * @param {Number | String} currentDay Текущий день
     * @param {Number | String} firstDay Первый день недели
     * @returns {Number}
     * 
     * @example
     * y5.Dates.getDayOffset('Sunday', 'Monday')
     * // -> 6
     */
    getDayOffset: function (currentDay, firstDay) {
        if (this.days[firstDay]) {
            firstDay = this.days[firstDay];
        } else {
            firstDay = parseInt(firstDay, 10); 
        }
        if (isNaN(firstDay)) {
            firstDay = 1;
        }
        
        
        if (this.days[currentDay]) {
            currentDay = this.days[currentDay];
        } else {
            currentDay = parseInt(currentDay, 10);; 
        }
        if (isNaN(currentDay)) {
            currentDay = 1;
        }
        
        if (currentDay < firstDay) {
            return 7 - firstDay + currentDay;
        } else {
            return currentDay - firstDay;
        }
    },

    /**
     * Создает копию объекта Date
     * @function
     * @memberOf y5.Dates
     * @name y5.Dates.clone
     * @param {Date} date Объект Date
     * @returns {Date} Копия объекта
     */
    clone: function (date) {
        return new Date(date);
    }
};

var Dates = y5.Dates;

/// aliases

/**
* @name getDay
* @memberOf y5.Dates
* @function
* @deprecated y5.Dates.getDayOffset
*/
Dates.getDay = Dates.getDayOffset;

y5.loaded('Dates');
    
});
