y5.require(['Classes', 'Strings', 'Elements'], function() {

var Types = y5.Types,
    Classes = y5.Classes,
    Strings = y5.Strings,
    Elements = y5.Elements,
    UNDEF = y5.UNDEF,
    Asterix = '*',
    ParentNode = 'parentNode',
    PreviousSibling = 'previousSibling',
    NextSibling = 'nextSibling';

function domGetById(id) {
    return document.getElementById(id);
}

/* Функция преобразования единиц измерения */
function unitConvert(value, element, func, length) {
    element = element || y5.Dom.getBody();

    var elem = Elements.create('span', {style: 'position:absolute;display:block;visibility:hidden;width:100' + length});
    var node = element.appendChild(elem);
    var result = func(value, node.clientWidth, 100);
    element.removeChild(node);

    return result;
}

function toPxConvert(value, width, num) {
    return value * width / num;
}

function fromPxConvert(value, width, num) {
    return value / (width / num);
}

function elements2array(elements, limit) {
    var length = elements.length;

    if (typeof limit != UNDEF) {
        length = Math.min(limit, length);
    }

    var nodes = new Array(length),
        i = 0;

    for (; i < length; i++) {
        nodes[i] = elements[i];
    }

    return nodes;
}

function getElementsByTagName (context, tagName) {
    if (tagName === Asterix) {
        return getAll(context);

    } else if (Types.string(tagName)) {
        return elements2array(context.getElementsByTagName(tagName));

    } else {
        var elements = [],
            i = 0,
            j = tagName.length;
        for (; i<j; i++) {
            if (tagName[i] === Asterix) {
                return getAll(context);
            }
            elements = elements.concat(getElementsByTagName(context, tagName[i]));
        }
        return elements;
    }
}

function getFirstFromSet(elements) {
    if (typeof elements[0] != UNDEF) {
        return elements[0];
    }
    return null;
}

var getAll;
if (y5.is_ie6down) {
    getAll = function(context) {
        return context.all;
    };
} else {
    getAll = function(context) {
        return context.getElementsByTagName(Asterix);
    };
}

/**
 * Функции для работы с DOM.
 * @class Содержит функции для работы с DOM.
 * @static
 * @name y5.Dom
 * @memberOf y5
 */
y5.Dom = {
    /**
     * Браузер поддерживает запросы с помощью XPath.
 	 * @name XPathSupport
 	 * @memberOf y5.Dom
 	 * @field
     * @type Boolean
     */
    XPathSupport: typeof XPathEvaluator != UNDEF,


/// Функции выбора элементов
/////////////////////////////////////////////////////////////////////

    /**
     * Возвращает элемент body документа.
 	 * @name getBody
 	 * @memberOf y5.Dom
 	 * @function
     * @returns {HTMLElement} Элемент body
     */
    getBody: function() {
        return this.body || (this.body = document.body || this.getDescendant(document, 'body'));
    },

    /**
     * Возвращает элемент html документа.
 	 * @name getHtml
 	 * @memberOf y5.Dom
 	 * @function
     * @returns {HTMLElement} Элемент html
     */
    getHtml: function() {
        return this.html || (this.html = document.documentElement || this.getDescendant(document, 'html'));
    },

    /**
     * Возвращает список потомков по имени и классу.
 	 * @name getDescendants
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @param {Number} [limit] Лимит количества найденных элементов
     * @returns {Array} Результат поиска
     */
    getDescendants: function(context, tagName, className, limit) {
        return this.filterElements(getElementsByTagName(context, tagName || Asterix), Asterix, className, limit);
    },

    /**
     * Возвращает список потомков узла и/или сам узел по имени и классу.
 	 * @name getDescendantsOrSelf
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @param {Number} [limit] Лимит количества найденных элементов
     * @returns {Array} Результат поиска
     */
    getDescendantsOrSelf: function(context, tagName, className, limit) {
        return this.filterElements(getElementsByTagName(context, tagName || Asterix), tagName, className, limit, context);
    },

    /**
     * Возвращает первого потомка по имени и классу.
 	 * @name getDescendant
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getDescendant: function(context, tagName, className) {
        return getFirstFromSet(this.getDescendants(context, tagName, className, 1));
    },

    /**
     * Возвращает первого потомка по имени и классу.
 	 * @name getDescendantOrSelf
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getDescendantOrSelf: function(context, tagName, className) {
        return getFirstFromSet(this.getDescendantsOrSelf(context, tagName, className, 1));
    },

    /**
     * Возвращает элементы-предки по имени и классу.
 	 * @name getAncestors
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Array} Результат поиска
     */
    getAncestors: function(context, tagName, className) {
        return this.getElementsByType(context, tagName, className, ParentNode);
    },

    /**
     * Возвращает элементы-предки узла и/или сам узел по имени и классу.
 	 * @name getAncestorsOrSelf
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Array} Результат поиска
     */
    getAncestorsOrSelf: function(context, tagName, className) {
        return this.getElementsByType(context, tagName, className, ParentNode, context);
    },

    /**
     * Возвращает первый элемент-родитель по имени и классу.
 	 * @name getAncestor
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getAncestor: function(context, tagName, className) {
        return this.getElementByType(context, tagName, className, ParentNode);
    },

    /**
     * Возвращает первый элемент-родитель узла и/или сам узел по имени и классу.
 	 * @name getAncestorOrSelf
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getAncestorOrSelf: function(context, tagName, className) {
        return this.getElementByType(context, tagName, className, ParentNode, context);
    },

    /**
     * Возвращает дочерние элементы по имени и классу.
 	 * @name getChildren
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Array} Результат поиска
     */
    getChildren: function(context, tagName, className, limit) {
        return this.filterElements(context.childNodes, tagName, className, limit);
    },

    /**
     * Возвращает первый дочерний элемент по имени и классу.
 	 * @name getChild
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getChild: function(context, tagName, className) {
        return getFirstFromSet(this.getChildren(context, tagName, className, 1));
    },

    /**
     * Возвращает предыдущие элементы по имени и классу.
 	 * @name getPreceding
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getPreceding: function(context, tagName, className) {
        return this.getElementsByType(context, tagName, className, PreviousSibling);
    },

    /**
     * Возвращает первый предыдущий элемент по имени и классу.
 	 * @name getPrev
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getPrev: function(context, tagName, className) {
        return this.getElementByType(context, tagName, className, PreviousSibling);
    },

    /**
     * Возвращает следующие элементы по имени и классу.
 	 * @name getFollowing
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getFollowing: function(context, tagName, className) {
        return this.getElementsByType(context, tagName, className, NextSibling);
    },

    /**
     * Возвращает первый следующий элемент по имени и классу.
 	 * @name getNext
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Element | null} Результат поиска
     */
    getNext: function(context, tagName, className) {
        return this.getElementByType(context, tagName, className, NextSibling);
    },

    /// частные случаи

    /**
     * Возвращает потомков с заданным именем и классом.
     * @param {Array | String} tagName Имя элемента ('*' - любое имя)
     * @param {String} className Имя класса
     * @param {Element} [context] Контекст поиска (если не указан, то используется document)
     * @param {Number} [limit] Ограничить количество элементов
     * @returns {Array of Elements} Набор узлов документа
     * @deprecated y5.Dom.getDescendants
     */
    getElementsByTagNameAndClass: function(tagName, className, context, limit) {
        return this.getDescendants(context || document, tagName, className, limit);
    },

    /**
     * Возвращает первого потомка с заданным именем и классом.
     * Эквивалентно вызову getElementsByTagNameAndClass с параметром limit == 1.
     * @param {Array | String} tagName Имя элемента ('*' - любое имя)
     * @param {String} className Имя класса
     * @param {Element} [context] Контекст поиска (если не указан, то используется document)
     * @returns {Element} Узел документа
     * @deprecated y5.Dom.getDescendant
     */
    getElementByTagNameAndClass: function(tagName, className, context) {
        return getFirstFromSet(this.getElementsByTagNameAndClass(tagName, className, context, 1));
    },

    /**
     * Возвращает первого потомка с заданным именем.
     * @param {Array | String} tagName Имя элемента ('*' - любое имя)
     * @param {Element} [context] Контекст поиска (если не указан, то используется document)
     * @returns {Element} Элемент
     * @deprecated y5.Dom.getDescendants
     */
    getElementsByTagName: function(tagName, context, limit) {
        return this.getDescendants(context || document, tagName, Asterix, limit);
    },

    /**
     * Возвращает первого потомка с заданным именем.
     * @param {Array | String} tagName Имя элемента ('*' - любое имя)
     * @param {Element} [context] Контекст поиска (если не указан, то используется document)
     * @returns {Element} Элемент
     * @deprecated y5.Dom.getDescendant
     */
    getElementByTagName: function(tagName, context) {
        return getFirstFromSet(this.getElementsByTagName(tagName, context, 1));
    },

    /**
     * Возвращает потомков с заданным классом.
 	 * @name getElementsByClass
 	 * @memberOf y5.Dom
 	 * @function
     * @param {String} className Имя класса
     * @param {Element} [context] Контекст поиска (если не указан, то используется document)
     * @param {Number} [limit] Ограничить количество элементов
     * @returns {Array of Elements} Набор узлов документа
     */
    getElementsByClass: function(className, context, limit) {
        return this.getDescendants(context || document, Asterix, className, limit);
    },

    /**
     * Возвращает первого потомка с заданным классом.
     * Эквивалентно вызову getElementsByClass с параметром limit == 1.
 	 * @name getElementByClass
 	 * @memberOf y5.Dom
 	 * @function
     * @param {String} className Имя класса
     * @param {Element} [context] Контекст поиска (если не указан, то используется document)
     * @returns {Element} Узел документа
     */
    getElementByClass: function(className, context) {
        return getFirstFromSet(this.getElementsByClass(className, context, 1));
    },

    /**
     * Возвращает первого предка с заданным именем.
     * @param {Element} context Контекст поиска
     * @param {Array | String} [tagName] Имя элемента ('*' - любое имя)
     * @returns {Element} Узел документа
     * @deprecated y5.Dom.getAncestorOrSelf
     */
    getParentByTagName: function(context, tagName) {
        return this.getAncestorOrSelf(context, tagName, Asterix);
    },

    /**
     * Возвращает первого предка с заданным именем класса.
     * @param {String} className Имя класса
     * @param {Element} context Контекст поиска
     * @returns {Element} Узел документа
     * @deprecated y5.Dom.getAncestorOrSelf
     */
    getParentByClass: function(className, context) {
        return this.getAncestorOrSelf(context, Asterix, className);
    },


/// Тесты и фильтры
/////////////////////////////////////////////////////////////////////

    /**
     * Фильтрует список элементов по имени тега и класса.
 	 * @name filterElements
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Array of Elements} elements Список элементов
     * @param {Array | String} [tagName] Имя (имена) элемента для проверки ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @param {Number} [limit] Лимит количества найденных элементов
     * @returns {Array of Elements} Результат фильтрации
     * @optimize
     */
    filterElements: function(elements, tagName, className, limit, selfNode) {
        var element, result = [], i = 0, j = 0;
        limit = limit || -1;

        if (selfNode) {
            if (this.testElement(selfNode, tagName, className)) {
                result[j++] = selfNode;
            }
        }

        while (j != limit && (element = elements[i++])) {
            if (this.testElement(element, tagName, className)) {
                result[j++] = element;
            }
        }

        return result;
    },

    /*
    filterByClass: function(elements, className, limit) {
        return this.filterElements(elements, Asterix, className, limit);
    },

    filterByTagName: function(elements, tagName, limit) {
        return this.filterElements(elements, tagName, Asterix, limit);
    },
    */

    /**
     * Проверяет элемент на имя и класс.
 	 * @name testElement
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} element Проверяемый элемент
     * @param {Array | String} [tagName] Имя (имена) элемента для проверки ('*' - любое имя)
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Boolean} Результат проверки
     */
    testElement: function(element, tagName, className) {
        return (
            this.testTagName(element, tagName) &&
            this.testClassName(element, className)
        );
    },

    /**
     * Проверяет имя элемента.
 	 * @name testTagName
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} element Элемент для проверки
     * @param {Array | String} [tagName] Имя (имена) элемента для проверки ('*' - любое имя)
     * @returns {Boolean} Результат проверки
     *
     * @example
     * // &lt;div />
     * y5.Dom.testTagName(div, 'div');
     * // -> true
     * y5.Dom.testTagName(div, 'span');
     * // -> false
     * y5.Dom.testTagName(div, ['div', 'span']);
     * // -> true
     */
    testTagName: function(element, tagName) {
        if (!element || !element.tagName) {
            return false;
        }

        if ((tagName || Asterix) == Asterix) {
            return true;
        }

        var elTagName = element.tagName.toLowerCase();
        if (typeof tagName == 'string') {
            return elTagName == tagName.toLowerCase();
        }

        var name,
            i = 0,
            l = tagName.length;
        for (; i < l; i++) {
            name = tagName[i];
            if (name == Asterix || elTagName == name.toLowerCase()) {
                return true;
            }
        }

        return false;
    },

    /**
     * Проверяет имя класса элемента.
 	 * @name testClassName
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} element Элемент для проверки
     * @param {String | RegExp} [className] Имя класса ('*' - любое имя)
     * @returns {Boolean} Результат проверки
     *
     * @example
     * // &lt;div class="foo"/>
     * y5.Dom.testClassName(div, 'foo');
     * // -> true
     * y5.Dom.testClassName(div, 'bar');
     * // -> false
     * y5.Dom.testClassName(div, '*');
     * // -> true
     */
    testClassName: function(element, className) {
        return Classes.test(element, className || Asterix);
    },

    /**
     * Проверяет, что элемент является дочерним.
 	 * @name isChild
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} childNode Проверяемый элемент
     * @param {Element} [parentNode] Предполагаемый родитель (если не указан, то используется document)
     * @returns {Boolean} Результат проверки
     */
    isChild: function(childNode, parentNode) {
        if (parentNode == document) {
            return true;
        }
        while (childNode) {
            if (childNode === parentNode) {
                return true;
            }
            childNode = childNode.parentNode;
        }
        return false;
    },


/// Функции удаления, замены узлов
/////////////////////////////////////////////////////////////////////

    /**
     * Удаляет узел из документа.
 	 * @name removeNode
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Node} node Узел
     */
    removeNode: function(node) {
        if (node && node.parentNode) {
            node.parentNode.removeChild(node);
        }
    },

    /**
     * Удаляет всех потомков узла.
 	 * @name clearNode
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Node} node Узел
     */
    clearNode: function(node) {
        if (!node) {
            return null;
        }
        var child;
        while ((child = node.firstChild)) {
            node.removeChild(child);
        }
        return node;
    },

    /**
     * Удаляет узел из документа и добавляет всех его потомков к его родителю.
 	 * @name cutNode
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Node} node Узел
     */
    cutNode: function(node) {
        var parent = node.parentNode, child;
        while ((child = node.firstChild)) {
            parent.appendChild(child);
        }
        this.removeNode(node);
    },

    /**
     * Заменяет один элемент на другой.
 	 * @name replaceNode
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} element Заменяемый элемент
     * @param {Element | String} [newElement] Новый элемент или строка HTML
     * @todo генерировать уникальный ID
     */
    replaceNode: function(element, newElement) {
        switch (typeof newElement) {
            case 'string':
                if (element.outerHTML) {
                    if (newElement.indexOf('<') == 0) {
                        var spanId = '__outer_span__';
                        element.outerHTML = '<span id="' + spanId + '">&#160;</span>' + newElement;

                        var span = y5.$(spanId);
                        span.parentNode.removeChild(span);
                    } else {
                        element.outerHTML = newElement;
                    }
                } else {
                    var replaced;

                    // для FF 1.0, вставка пустого элемента
                    if (Strings.normalize(newElement) == '') {
                        replaced = document.createTextNode(newElement)
                    } else {
                        var range = element.ownerDocument.createRange();
                        range.selectNodeContents(element);
                        replaced = range.createContextualFragment(newElement);
                    }
                    element.parentNode.replaceChild(replaced, element);
                }
                break;

            default:
                element.parentNode.replaceChild(newElement, element);
                break;
        }
    },

    /**
     * Возвращает текстовое содержимое элемента.
 	 * @name textContent
 	 * @memberOf y5.Dom
 	 * @function
     * @function
     * @param {Element} element Элемент
     * @returns {String} Текст
     */
    textContent: (function() {
        var span = document.createElement('span');
        if (y5.gecko_ver > 1.7 && Types.def(span.textContent)) {
            return function(element) {
                return element.textContent;
            }
        } else if (Types.def(span.innerText)) {
            return function(element) {
                return element.innerText;
            }
        } else {
            return function(element) {
                // using RegExp for IE 5
                return element.innerHTML.replace(new RegExp('<.*?>', 'g'), '');
            }
        }
    })(),


/// Функции вставки узлов
/////////////////////////////////////////////////////////////////////

    /**
     * Вставляет новый элемент перед указанным.
     * Обертка для DOM-функции insertBefore.
 	 * @name insertBefore
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} newElem Элемент для вставки
     * @param {Element} refElem Элемент перед которым будет вставлен новый элемент
     * @returns {Node} Вставленный элемент
     * @example
     * // - parent
     * //    - refElem
     * y5.Dom.insertBefore(newElem, refElem);
     * // - parent
     * //    - newElem*
     * //    - refElem
     */
    insertBefore: function(newElem, refElem) {
        return refElem.parentNode.insertBefore(newElem, refElem);
    },

    /**
     * Вставляет новый элемент после указанного.
 	 * @name insertAfter
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} newElem Элемент для вставки
     * @param {Element} refElem Элемент после которого будет вставлен новый элемент
     * @returns {Node} Вставленный элемент
     * @example
     * // - parent
     * //    - refElem
     * y5.Dom.insertAfter(newElem, refElem);
     * // - parent
     * //    - refElem
     * //    - newElem*
     */
    insertAfter: function(newElem, refElem) {
        var next = refElem.nextSibling;
        if (next) {
            return this.insertBefore(newElem, next);
        }

        return refElem.parentNode.appendChild(newElem);
    },


/// Функции для работы со стилями и свойствами элементов
/////////////////////////////////////////////////////////////////////

    /**
     * Возвращает размеры видимой области документа.
 	 * @name viewPort
 	 * @memberOf y5.Dom
 	 * @function
     * @returns {Array} [ширина, высота]
     */
    viewPort: function() {
        var body = this.getBody();
        return [body.clientWidth, body.clientHeight];
    },

    /**
     * Возвращает левое и верхнее смещение одного элемента относительно другого.
     * Если второй аргумент не указан, то смещение считается от document.
 	 * @name getOffset
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} targetElement Элемент, для которого считается смещение
     * @param {Element} [relativeElement] Элемент, относительно которого считается смещение
     * @returns {Array} [левое смещение, верхнее смещение]
     */
    getOffset: function(targetElement, relativeElement) {
        // функция используется в Safari и Opera
        var elementX = 0,
            elementY = 0,
            relativeElementOffset = [0,0],
            body = this.getBody(),
            position, pStatic, pRelative, inlineElement;

        if (!relativeElement || !Types.element(relativeElement)) {
            relativeElement = document;
        }

        /*
         * если считаем в opera смещение для inline-элемента,
         * то надо вместо целевого элемента подсовывать фейковый span и считать для него offset,
         * т.к. для многострочного элемента offsetLeft будет неверным.
         */ 
        if (y5.is_opera && Elements.css(targetElement, 'display') == 'inline') {
            // если есть margin-left, то фейковый элемент будет смещен влево на это значение
            elementX = Elements.css(targetElement, 'margin-left');
            inlineElement = Elements.create('span');
            this.insertBefore(inlineElement, targetElement);
            targetElement = inlineElement;
        }

        while (targetElement !== null && targetElement !== relativeElement) {
            // offset added
            elementX += targetElement.offsetLeft || 0;
            elementY += targetElement.offsetTop || 0;

            if (!(y5.is_konq || (y5.is_opera && y5.opera_ver > 8.6))) {
                position = Elements.css(targetElement, 'position');
                pStatic = position == 'static';
                pRelative = position == 'relative';

                if (pStatic || (!y5.is_opera && pRelative)) {
                    elementX += Elements.css(targetElement, 'border-left-width');
                    elementY += Elements.css(targetElement, 'border-top-width');

                    if (y5.is_ie && targetElement !== body) {
                        elementX += Elements.css(targetElement, 'margin-left');
                        elementY += Elements.css(targetElement, 'margin-top');
                    }
                }
            }

            // offsetParent указывает на первый внешний блочный элемент с position:relative, от которого отсчитываются координаты текущего элемента.
            targetElement = targetElement.offsetParent;
        }

        if (Types.element(inlineElement)) {
            this.removeNode(inlineElement);
        }

        // если предок не document и не offsetParent, то считаем offset у предка
        if (relativeElement !== document && targetElement !== relativeElement) {
            relativeElementOffset = this.getOffset(relativeElement);
        }

        return [elementX-relativeElementOffset[0], elementY-relativeElementOffset[1]];
    },

    /**
     * Возвращает верхнее смещение одного элемента относительно другого.
     * Если второй аргумент не указан, то смещение считается от document.
 	 * @name offsetTop
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} targetElement Элемент, для которого считается смещение
     * @param {Element} [relativeElement] Элемент, относительно которого считается смещение
     * @returns {Number} Верхнее смещение
     */
    offsetTop: function(targetElement, relativeElement) {
        return this.getOffset(targetElement, relativeElement)[1];
    },

    /**
     * Возвращает левое смещение одного элемента относительно другого.
     * Если второй аргумент не указан, то смещение считается от document.
 	 * @name offsetLeft
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} targetElement Элемент, для которого считается смещение
     * @param {Element} [relativeElement] Элемент, относительно которого считается смещение
     * @returns {Number} Левое смещение
     */
    offsetLeft: function(targetElement, relativeElement) {
        return this.getOffset(targetElement, relativeElement)[0];
    },
    
    getDimensions: function(element) {
        var display = Elements.css(element, 'display');
        // если элемент не скрыт
        if (display != 'none' && display != null) {
            return [element.offsetWidth, element.offsetHeight];
        }
        
        // для элементов c display=none ширина и высота равна 0
        var style = element.style,
            originalVisibility = style.visibility,
            originalPosition = style.position,
            originalDisplay = style.display,
            dimensions;
        
        style.visibility = 'hidden';
        style.position = 'absolute';
        style.display = 'block';
        
        dimensions = [element.offsetWidth, element.offsetHeight];
        
        style.display = originalDisplay;
        style.position = originalPosition;
        style.visibility = originalVisibility;
        
        return dimensions;
    },
    
    getWidth: function (element) {
        return this.getDimensions(element)[0];
    },
    
    getHeight: function (element) {
        return this.getDimensions(element)[1];
    },

    /**
     * Возвращает позицию горизонтальной прокрутки страницы.
 	 * @name getPageScrollX
 	 * @memberOf y5.Dom
 	 * @function
     * @returns {Number} Позиция прокрутки
     */
    getPageScrollX: function() {
        return this.getBody().scrollLeft || this.getHtml().scrollLeft;
    },

    /**
     * Возвращает позицию вертикальной прокрутки страницы.
 	 * @name getPageScrollY
 	 * @memberOf y5.Dom
 	 * @function
     * @returns {Number} Позиция прокрутки
     */
    getPageScrollY: function() {
        return this.getBody().scrollTop || this.getHtml().scrollTop;
    },

    /**
     * Возвращает вычисленные стили элемента.
     * @param {Element} element Элемент
     * @returns {Object} Стили
     * @deprecated y5.Elements.getStyle
     */
    getStyle: function(element) {
        return Elements.getStyle(element);
    },

    /**
     * Возвращает вычисленное значение CSS-свойства элемента.
     * @param {Element} element Элемент
     * @param {String} propname Свойство, например 'margin-left'
     * @returns {String} Значение свойства, например '5px'
     * @deprecated y5.Elements.getPropertyValue
     */
    getPropertyValue: function(element, propname) {
        return Elements.getPropertyValue(element, propname);
    },

    /**
     * Возвращает вычисленное числовое значение CSS-свойства элемента.
     * @param {Element} element Элемент
     * @param {String} propname Свойство, например 'margin-left'
     * @returns {String} Значение свойства, например '5'
     * @deprecated y5.Elements.getPropertyValuePx
     */
    getPropertyValuePx: function(element, propname) {
        return Elements.getPropertyValuePx(element, propname);
    },

    /**
     * Преобразовывает значение свойства элемента из em в px.
 	 * @name em2px
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Number} em Значение в em
     * @param {Element} element Элемент (должен быть видимым).
     * @returns {Number} Результат в px
     * @todo перенести в y5.CSS
     */
    em2px: function(em, element) {
        return unitConvert(em, element, toPxConvert, 'em');
    },

    /**
     * Преобразовывает значение свойства элемента из px в em.
 	 * @name px2em
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Number} px Значение в px
     * @param {Element} element Элемент (должен быть видимым).
     * @returns {Number} Результат в em
     * @todo перенести в y5.CSS
     */
    px2em: function(px, element) {
        return unitConvert(px, element, fromPxConvert, 'em');
    },

    /**
     * Преобразовывает значение свойства элемента из unit (em, %, in,...) в px.
 	 * @name unit2px
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Number} length Значение в unit
     * @param {Element} element Элемент (должен быть видимым).
     * @returns {Number} Результат в px
     * @todo перенести в y5.CSS
     */
    unit2px: function(unit, element, type) {
        return unitConvert(unit, element, toPxConvert, type);
    },

    /**
     * Преобразовывает значение свойства элемента из px в unit (em, %, in,...).
 	 * @name px2unit
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Number} px Значение в px
     * @param {Element} element Элемент (должен быть видимым).
     * @returns {Number} Результат в unit
     * @todo перенести в y5.CSS
     */
    px2unit: function(px, element, type) {
        return unitConvert(px, element, fromPxConvert, type);
    },


/// private
/////////////////////////////////////////////////////////////////////

    /**
     * Возвращает первый элемент заданного типа.
 	 * @name getElementByType
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {String} tagName Имя элемента ('*' - любое имя)
     * @param {String} className Имя класса ('*' - любое имя)
     * @param {Boolean} type Тип элемента: nextSibling, previousSibling, firstChild, parentNode...
     * @returns {Element | null} Результат поиска
     * @private
     */
    getElementByType: function(context, tagName, className, type, selfNode) {
        context = selfNode || context[type];
        while (context) {
            if (this.testElement(context, tagName, className)) {
                return context;
            }
            context = context[type];
        }

        return null;
    },

    /**
     * Возвращает список элементов заданного типа.
 	 * @name getElementsByType
 	 * @memberOf y5.Dom
 	 * @function
     * @param {Element} context Контекст поиска
     * @param {String} tagName Имя элемента ('*' - любое имя)
     * @param {String} className Имя класса ('*' - любое имя)
     * @param {Boolean} type Тип элемента: nextSibling, previousSibling, firstChild, parentNode...
     * @returns {Array} Результат поиска
     * @private
     */
    getElementsByType: function(context, tagName, className, type, selfNode) {
        var result = [];

        context = selfNode || context[type];
        while (context) {
            if (this.testElement(context, tagName, className)) {
                result.push(context);
            }
            context = context[type];
        }

        // для определенных типов меняем последовательность,
        // чтобы узлы в массиве шли в том же порядке, как в документе
        if (type == PreviousSibling || type == ParentNode) {
            return result.reverse();
        }

        return result;
    },

    /**
     * Возвращает элементы по запросу XPath.
 	 * @name getElementsByXPath
 	 * @memberOf y5.Dom
 	 * @function
     * @param {String} expression Выражение XPath
     * @param {Element} [context] Контекст запроса (если не указан, то document)
     * @returns {Array of Elements} Набор узлов документа
     * @private
     */
    getElementsByXPath: function(expression, context) {
        var evaluator = new XPathEvaluator();
        var query = evaluator.evaluate(expression, context, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
        var length = query.snapshotLength;
        var result = new Array(length);
        for (var i = 0; i < length; i++) {
            result[i] = query.snapshotItem(i);
        }
        return result;
    },

    /**
     * Возвращает первый элемент по запросу XPath.
 	 * @name getElementByXPath
 	 * @memberOf y5.Dom
 	 * @function
     * @param {String} expression Выражение XPath
     * @param {Element} [context] Контекст запроса (если не указан, то document)
     * @returns {Array of Elements} Набор узлов документа
     * @private
     */
    getElementByXPath: function(expression, context) {
        var evaluator = new XPathEvaluator();
        var query = evaluator.evaluate(expression, context, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);

        return query != null ? query.singleNodeValue : null;
    }
};

var Dom = y5.Dom;

if (Dom.XPathSupport) {

    function tagNameForXPath(tagName) {
    	switch (Types.type(tagName)) {
    	    case Types.STRING:
    	        return tagName.toLowerCase();

    	    case Types.ARRAY:
                var i = 0,
                    length = tagName.length,
                    exprs = [],
                    name;

                if (length == 1) {
                    return tagName[0].toLowerCase();
                }

                for (; i < length; i++) {
                    name = tagName[i];
                    if (name != Asterix) {
                        exprs.push("name()='" + name.toLowerCase() + "'");
                        exprs.push("name()='" + name.toUpperCase() + "'");
                    } else {
                        return Asterix;
                    }
                }
	
                return Asterix + '[' + exprs.join(' or ') + ']';
    	}
    	
    	return Asterix;
    }

    function classNameForXPath(className) {
        if (className && className != Asterix) {
            return "[contains(concat(' ',@class,' '),' " + className + " ')]";
        }
        return '';
    }

    Dom.getByAxis = function(context, axis, tagName, className, limit) {
        var expression = axis + tagNameForXPath(tagName);

        if (typeof className == 'string') {
            expression += classNameForXPath(className);
            if (limit) {
                expression += '[position()<=' + limit + ']';
            }
            return this.getElementsByXPath(expression, context);
        } else {
            return this.filterElements(this.getElementsByXPath(expression, context), Asterix, className, limit);
        }
    };

    var axis = {
        getDescendants: 'descendant',
        getDescendantsOrSelf: 'descendant-or-self',
        getAncestors: 'ancestor',
        getAncestorsOrSelf: 'ancestor-or-self',
        getChildren: 'child',
        getFollowing: 'following-sibling',
        getPreceding: 'preceding-sibling'
    };

    for (var i in axis) {
        Dom[i] = (function(a) {
            return function(context, tagName, className, limit) {
                return this.getByAxis(context, a + '::', tagName, className, limit);
            };
        })(axis[i]);
    }
}

if (document.getElementsByClassName) {
    Dom.__getElementsByClass = Dom.getElementsByClass;
    Dom.getElementsByClass = function(className, context, limit) {
        if (Types.regexp(className)) {
            return this.__getElementsByClass(className, context, limit);
        }
        return elements2array((context || document).getElementsByClassName(className), limit);
    };
}

if (window.innerHeight) { // all except Explorer

    Dom.viewPort = function() {
        return [window.innerWidth, window.innerHeight];
    };

} else if (document.documentElement && document.documentElement.clientHeight) { // Explorer 6 Strict Mode

    Dom.viewPort = function() {
        var e = document.documentElement;
        return [e.clientWidth, e.clientHeight];
    };

}

// http://msdn2.microsoft.com/en-us/library/ms536433.aspx
// http://developer.mozilla.org/en/docs/DOM:element.getBoundingClientRect
if (document.documentElement && document.documentElement.getBoundingClientRect) { // msie 5+, ff3

    Dom.getOffset = function(targetElement, relativeElement) {
        if (targetElement === document) {
            return [0, 0];
        }

        if (!Types.element(relativeElement)) {
            relativeElement = document;
        }

        var targetElementOffset,
            relativeElementOffset = (relativeElement !== document ?
                relativeElement.getBoundingClientRect() :
                // IE in quirks and standards modes has body and html borders
                {
                    left: document.body.clientLeft + document.documentElement.clientLeft,
                    top: document.body.clientTop + document.documentElement.clientTop
                }
            );
        
        /*
         * для inline элементов, которые располагаются на нескольких строках,
         * надо брать координаты первого текстового прямоугольника, т.к. getBoundingClientRect
         * возвращает координаты объединенной области, а это минимальные x и y
         */ 
        if (Elements.css(targetElement, 'display') == 'inline') {
            var rects = targetElement.getClientRects();
            targetElementOffset = {left: rects[0].left, top: rects[0].top};
            
        } else {
            targetElementOffset = targetElement.getBoundingClientRect()
        }
        
        // FF returns float value => Math.round
        // IE adds border width => -document.documentElement.clientLeft and Top
        return [
            Math.round(targetElementOffset.left + Math.max(document.documentElement.scrollLeft, document.body.scrollLeft) - relativeElementOffset.left),
            Math.round(targetElementOffset.top + Math.max(document.documentElement.scrollTop,  document.body.scrollTop) - relativeElementOffset.top)
        ];
    };

// getBoxObjectFor is deprecated in ff3
} else if (document.getBoxObjectFor) { // Gecko 1.0.1+

    Dom.getOffset = function(targetElement, relativeElement) {
        if (targetElement === document) {
            return [0, 0];
        }

        if (!Types.element(relativeElement)) {
            relativeElement = document;
        }

        var relativeElementOffset = {x:0,y:0},
            targetElementOffset = document.getBoxObjectFor(targetElement);

        if (relativeElement !== document) {
            relativeElementOffset = document.getBoxObjectFor(relativeElement);
        }

        return [targetElementOffset.x-relativeElementOffset.x, targetElementOffset.y-relativeElementOffset.y];
    };

}

if (typeof window.pageXOffset == 'number') {

    Dom.getPageScrollX = function() {
        return window.pageXOffset;
    };

    Dom.getPageScrollY = function() {
        return window.pageYOffset;
    };

}

/// aliases

/**
 * @name y5.$
 * @see y5.Dom.$
 * @memberOf y5
 * @function
 * @param {String} id
 * @returns {Element} элемент DOM
 */
y5.$ = domGetById;

/**
 * Сокращение для document.getElementById.
 * @name y5.Dom.$
 * @memberOf y5.Dom
 * @function
 * @param {String} id
 * @returns {Element} элемент DOM
 */
Dom.$ = domGetById;

/**
 * @name getNextElement
 * @memberOf y5.Dom
 * @function
 * @deprecated y5.Dom.getNext
 */
Dom.getNextElement = Dom.getNext;

/**
 * @name getPreviousElement
 * @memberOf y5.Dom
 * @function
 * @deprecated y5.Dom.getPrev
 */
Dom.getPreviousElement = Dom.getPrev;

/**
 * @name deleteNode
 * @memberOf y5.Dom
 * @function
 * @deprecated y5.Dom.removeNode
 */
Dom.deleteNode = Dom.removeNode;

/**
 * @name getOffsset
 * @memberOf y5.Dom
 * @function
 * @deprecated y5.Dom.getOffset
 */
Dom.getOffsset = Dom.getOffset;

/**
 * @name innerText
 * @memberOf y5.Dom
 * @function
 * @deprecated y5.Dom.textContent
 */
Dom.innerText = Dom.textContent;

y5.loaded('Dom');

});
