(function() {

var Types = y5.Types,
    UNDEF = y5.UNDEF,
    GC    = y5.GC;

// вызов функции с контекстом
function execListenerWithContext(listener, context, param1, param2) {
    return (context ? listener.call(context, param1, param2) : listener(param1, param2));
}

/// Обрабатываем не-DOM типы событий
var typeDOMAttrModified = 'DOMAttrModified',
    typePropertyChange = 'propertychange',
    typeDOMMouseScroll = 'DOMMouseScroll',
    typeMouseWheel = 'mousewheel',
    specificTypes = {};

if (document.attachEvent) { // IE
    specificTypes[typeDOMAttrModified] = typePropertyChange;
} else { // DOM
    specificTypes[typePropertyChange] = typeDOMAttrModified;
}

if (y5.is_ie || y5.is_opera || y5.is_safari) {
    // браузеры с событием колеса мыши 'mouseweel'
    specificTypes[typeDOMMouseScroll] = typeMouseWheel;
} else {
    specificTypes[typeMouseWheel] = typeDOMMouseScroll;
}

function normalizeType(type) {
    return specificTypes[type] || type;
}

/////////////////////////////////////////////////////////////////////

// функция для преобразования свойств Event
var normalizeEventSpecific,

    // коды клавиш мыши
    mouseButtonsListener = {L: [0, 65535], M: [1], R: [2]};

if (y5.is_ie) {

    function IE_preventDefault() {
        this.returnValue = false;
    }

    function IE_stopPropagation() {
        this.cancelBubble = true;
    }

    normalizeEventSpecific = function(e) {
        e.timeStamp = new Date().getTime();
        e.charCode = e.type == 'keypress' ? e.keyCode : 0;
        e.isChar = e.charCode > 0;
        e.target = e.srcElement;
        e.metaKey = e.altKey;
        e.attrName = e.propertyName == 'className' ? 'class' : e.propertyName;
        e.preventDefault = IE_preventDefault;
        e.stopPropagation = IE_stopPropagation;

        var doc = document.documentElement,
            body = document.body;
        e.pageX = e.clientX + (doc.scrollLeft || body.scrollLeft);
        e.pageY = e.clientY + (doc.scrollTop || body.scrollTop);

        switch (e.type) {
            case 'mouseout':
                e.relatedTarget = e.toElement;
                break;

            case 'mouseover':
                e.relatedTarget = e.fromElement;
                break;
        }

        // mouse scroll details
        e.scrollDetail = 0;
        if (e.wheelDelta) {
            e.scrollDetail = -e.wheelDelta / 40;
        }
    };

} else if (y5.is_safari) {

    mouseButtonsListener = {L: [0, 65535, 1 /* safari.ver <= 420 */], M: [2], R: [3]};

    normalizeEventSpecific = function(e) {
        // TODO! раскрыть смысл
        if (!Types.func(e.preventDefault)) {
            e.preventDefault = y5.NULL;
        }

        if (!Types.func(e.stopPropagation)) {
            e.stopPropagation = y5.NULL;
        }

        // Если событие произошло на текстовом узле (TEXT_NODE == 3, CDATA == 4)
        if (e.target && (e.target.nodeType == 3 || e.target.nodeType == 4)) {
            e.target = e.target.parentNode;
        }

        if (e.wheelDelta) {
            e.scrollDetail = -e.wheelDelta / 400;
        }
    };

} else if (y5.is_opera) {

    normalizeEventSpecific = function(e) {
        // mouse scroll details
        e.scrollDetail = 0;

        if (e.wheelDelta) {
            e.scrollDetail = e.wheelDelta / 40;
        }

        if (y5.opera_ver >= 9.2) {
            e.scrollDetail *= -1;
        }
    };

    if (y5.opera_ver < 8) {
        mouseButtonsListener = {L: [1], M: [2], R: [3]};
    }

} else {

    normalizeEventSpecific = function(e) {
        try {
            // mouse scroll details
            e.scrollDetail = e.detail;
        } catch (e) {}
    };
}

if (y5.is_ie || y5.is_konq) {
    mouseButtonsListener = {L: [1], M: [4], R: [2]};
}

function normalizeEvent(e) {
    var button;
    try {
        // ff1 на этот момент говорит [Exception... "Illegal operation on WrappedNative prototype object"  nsresult: "0x8057000c (NS_ERROR_XPC_BAD_OP_ON_WN_PROTO)" . Поэтому оборачиваем в try
        // видимо у MutationEvent нельзя обращаться к каким-то свойствам
        button = e.button;
    } catch (e) {}
    if (typeof button != UNDEF) {
        e.buttonL = mouseButtonsListener.L.indexOf(button) != -1;
        e.buttonM = mouseButtonsListener.M.indexOf(button) != -1;
        e.buttonR = mouseButtonsListener.R.indexOf(button) != -1;
    } else {
        e.buttonL = e.buttonM = e.buttonR = false;
    }

    normalizeEventSpecific(e);

    return e;
}

/////////////////////////////////////////////////////////////////////

/**
 * Создает обработчик DOM-события.
 * @constructor
 * @class Обеспечивает поддержку работы с событиями. Позволяет устанавливать обработчики на различные события браузера и DOM.
 * @name y5.AEventListener
 * @param {String} type Тип события: click, load...
 * @param {Function} listener Callback-функция на возникновение события
 * @param {Element} element Объект, на котором возникает событие
 * @param {Boolean} add Начать "слушать" событие немедленно
 * @param {Object} context Контекст, в котором выполняется callback. По-умолчанию, element
 * @param {Boolean} runOnce Обработать событие один раз и удалить обработчик. По умолчанию, false.
 *
 * @example
 * new y5.AEventListener("click", callback, link, true);
 */
y5.AEventListener = function(type, listener, element, add, context, runOnce) {
    this.type = normalizeType(type);
    this.element = element || document;
    this.blocked = false;
    this.added = false;
    this.runOnce = runOnce || false;
    
    var _this = this;
    this.listener = function(evt) {
        var e = typeof evt != UNDEF ? normalizeEvent(evt) : {};

        if (_this.blocked) {
            e.stopPropagation();
            e.preventDefault();
            return;
        }

        execListenerWithContext(listener, context, e, _this.element);
        if (_this.runOnce) {
            _this.cleanup();
        }
    };

    if (add) {
        this.add();
    }

    GC.collect(this);
};

y5.AEventListener.prototype = {
    /**
     * Включает отслеживание установленного ранее события.
     * @name y5.AEventListener.add
     * @memberOf y5.AEventListener
     * @function
     */
    add: function() {
        if (this.added) return;

        this._add();
        this.added = true;
    },

    /**
     * Добавляет обработчик к элементу
     * @name y5.AEventListener._add
     * @memberOf y5.AEventListener
     * @function
     * @private
     */
    _add: function() {
        this.element.addEventListener(this.type, this.listener, false);
    },

    /**
     * Выключает отслеживание события.
     * @name y5.AEventListener.remove
     * @memberOf y5.AEventListener
     * @function
     */
    remove: function() {
        if (!this.added) return;

        this._remove();
        this.added = false;
    },

    /**
     * Удаляет обработчик с элемента
     * @name y5.AEventListener._remove
     * @memberOf y5.AEventListener
     * @function
     * @private
     */
    _remove: function() {
        this.element.removeEventListener(this.type, this.listener, false);
    },

    /**
     * Блокирует событие. Перестает выполняться установленный обработчик события и отменяется стандартная реакция интерфейсных элементов на данное событие.
     * @name y5.AEventListener.block
     * @memberOf y5.AEventListener
     * @function
     */
    block: function() {
        this.blocked = true;
    },

    /**
     * Разблокирует событие. Установленный обработчик события начинает выполняться снова и восстанавливается стандартная реакция интерфейсных элементов на данное событие.
     * @name y5.AEventListener.unblock
     * @memberOf y5.AEventListener
     * @function
     */
    unblock: function() {
        this.blocked = false;
    },

    /**
     * Удаляет обработчик события, ссылки на элемент и callback-функцию.
     * @name y5.AEventListener.cleanup
     * @memberOf y5.AEventListener
     * @function
     * @private
     */
    cleanup: function() {
        this.remove();
        this.element = null;
        this.listener = null;
    }
};

var AEventListener = y5.AEventListener;

if (document.attachEvent) { // IE

    AEventListener.prototype._add = function() {
        this.element.attachEvent('on' + this.type, this.listener);
    };

    AEventListener.prototype._remove = function() {
        this.element.detachEvent('on' + this.type, this.listener);
    };

}

/// Создание событий
/////////////////////////////////////////////////////////////////////

    // Типы событий (w3)
var eventTypes = {
        Mouse: /^mouse|click/,
        Key: /^key/,
        Mutation: /^DOM/,
        HTML: /./
    },

    // Коды клавиш мыши
    mouseButtonsEvent = [0, 1, 2];

if (y5.is_safari) {
    mouseButtonsEvent = [0, 2, 3];
}

if (y5.is_safari || y5.is_opera) {
    // Создание KeyEvents вызывает исключение
    // Safari 3.0.4 (523.12.9)
    // Opera 9.5b1
    delete eventTypes['Key'];
}

/**
 * Создает экземпляр класса и, если не задан параметр dispatch, инициирует возникновение заданного в параметре type события.
 * @constructor
 * @class Позволяет инициировать возникновение стандартных событий браузера.
 * @name y5.Event
 * @param {String} [type] Тип события: click, mousedown... (если не указан, то используется click)
 * @param {Object} [element] Целевой элемент события (если не указан, то используется document)
 * @param {Boolean} [dispatch] Отправить событие (если не указано, то событие возникает сразу)
 * @param {Object} [params] Параметры события
 *
 * @example
 * new y5.Event("click", link);
 */
y5.Event = function(type, element, dispatch, params) {
    this.type = normalizeType(type || 'click');
    this.element = element || document;
    this.params = params || {};
    this.init();

    if (typeof dispatch == UNDEF) {
        dispatch = true;
    }

    if (dispatch) {
        this.dispatch();
    }

    GC.collect(this);
};

y5.Event.prototype = {
    /**
     * Инициализация.
     * @name y5.Event.init
     * @memberOf y5.Event
     * @function
     * @private
     */
    init: function() {
        for (var type in eventTypes) {
            if (eventTypes[type].test(this.type)) {
                this.eventType = type;
                break;
            }
        }
    },

    /**
     * Инициирует возникновение подготовленного ранее события.
     * @name y5.Event.dispatch
     * @memberOf y5.Event
     * @function
     * @param {Object} params параметры
     *
     * @example
     * var E = new y5.Event("click", link, false);
     * // ...
     * E.dispatch();
     */
    dispatch: function(params, canBubble, cancelable) {
        if (typeof params == UNDEF) {
            params = this.params;
        }

        canBubble = canBubble || true;
        cancelable = cancelable || true;

        // create
        var event = document.createEvent(this.eventType + 'Events');

        // init (по типу события)
        switch (this.eventType) {
            case 'Mouse':
                event.initMouseEvent(
                    this.type,
                    canBubble,
                    cancelable,
                    document.defaultView,
                    params.detail    || 0,
                    params.screenX   || 0,
                    params.screenY   || 0,
                    params.clientX   || 0,
                    params.clientY   || 0,
                    params.ctrlKey   || false,
                    params.altKey    || false,
                    params.shiftKey  || false,
                    params.metaKey   || false,
                    mouseButtonsEvent[params.button || 0 /* default left */],
                    null
                );
                break;

            case 'Key':
                event.initKeyEvent(
                    this.type,
                    canBubble,
                    cancelable,
                    document.defaultView,
                    params.ctrlKey   || false,
                    params.altKey    || false,
                    params.shiftKey  || false,
                    params.metaKey   || false,
                    params.keyCode   || 0,
                    params.charCode  || 0
                );
                break;

            default:
                event.initEvent(this.type, canBubble, cancelable);
                break;
        }

        // dispatch
        return this.element.dispatchEvent(event);
    },

    /**
     * Удаляет ссылку на элемент
     * @name y5.Event.cleanup
     * @memberOf y5.Event
     * @function
     * @private
     */
    cleanup: function() {
        this.element = null;
    }
};

var Event = y5.Event;

if (document.createEventObject) { // IE

    mouseButtonsEvent = [1, 4, 2];

    function setEvent(event, params) {
        for (var name in params) {
            try {
                var value;
                switch (name) {
                    case 'button':
                        value = mouseButtonsEvent[params.button || 0 /* default left */];
                        break;

                    default:
                        value = params[name];
                        break;
                }
                event[name] = value;
            } catch(e) {}
        }
        return event;
    }

    Event.prototype.init = y5.NULL;

    Event.prototype.dispatch = function(params) {
        if (typeof params == UNDEF) {
            params = this.params;
        }

        var event = setEvent(document.createEventObject(), params);
        return this.element.fireEvent('on' + this.type, event);
    };

} else if (!document.createEvent) {

    Event.prototype.init = y5.NULL;

    Event.prototype.dispatch = function(params) {
        try {
            return this.element[this.type](params);
        } catch (e) {
            y5.Console.warn('Browser is too old', ['Event']);
            return false;
        }
    };
}

/**
 * @memberOf y5.Event
 * @name y5.Event.dispatchEvent
 * @function
 * @deprecated y5.Event.dispatch
 */
Event.prototype.dispatchEvent = Event.prototype.dispatch;


/// Observer & Notifier
/////////////////////////////////////////////////////////////////////
(function() {

var ObserverManager = {
    // список постоянных событий
    notifiers: {},
    observers: {},

    dispatchNotify: function(id, params, permanent) {
        var result = true,
            observers = this.observers[id];

        if (observers) {
            // надо делать копию массива обработчиков, чтобы при удалении не ехали индексы и не было пропусков.
            var cloneObservers = [].concat(observers),
                i = 0,
                length = cloneObservers.length;
                
            // проходимся по массиву и вызываем обработчики
            for (; i<length; i++) {
                if (!cloneObservers[i].added) continue;
                result &= this.runListener(cloneObservers[i], params);
                // если обработчик одноразовый, то вызываем cleanup
                // непосредственно из массива удалять обработчик по его индексу (i) нельзя, т.к. если до этого уже был кто-то удален, он не совпадет  
                if (cloneObservers[i].runOnce) {
                    cloneObservers[i].cleanup();
                }
            }
        }

        if (permanent) {
            var notifiers = this.notifiers;
            if (!notifiers[id]) {
                notifiers[id] = [];
            }
            notifiers[id].push(params);
        }

        return result;
    },

    runListener: function(observer, params) {
        var result = true;

        result &= execListenerWithContext(observer.listener, observer.context, params);
        y5.Console.log('Observer listener: ', observer, ['Notifier']);

        return result;
    },

    addObserver: function(observer) {
        var id = observer.id,
            observers = this.observers;

        if (!observers[id]) {
            observers[id] = [];
        }
        observers[id].push(observer);

        // запускаем выполнение для "постоянных" событий
        this.fireNotify(observer);
    },

    removeObserver: function(observer) {
        var observers = this.observers[observer.id],
            index;

        if (observers) {
            for (var i=0, length = observers.length; i < length; i++) {
                if (observers[i] === observer) {
                    observers.splice(i, 1);
                    break;
                }
            }
        }
    },
    
    fireNotify: function(observer) {
        var notifiers = this.notifiers[observer.id];

        if (notifiers) {
            for (var i = 0, l = notifiers.length; i < l; i++) {
                this.runListener(observer, notifiers[i]);
            }
        }
    },

    generateId: function(type, element) {
        var id = element;
        if (!element || typeof element == 'object') {
            id = y5.Utils.getUniqueId(element || y5);
        }
        return type + '_' + id;
    }
};

/**
 * Создает объект-слушатель, реагирующий на пользовательское событие заданного типа.
 * @class Класс для реализации совместно с классом y5.Notifier механизма пользовательских событий. Позволяет расширить стандартный набор событий браузера новыми, проблемно-ориентированными типами. Класс y5.Observer отвечает за создание слушателей — объектов, отслеживающих возникновение пользовательских событий.
 * @constructor
 * @name y5.Observer
 * @param {String} type Тип события
 * @param {Function} listener Callback-функция
 * @param {Object} object Объект, на котором возникает событие
 * @param {Boolean} add Начать слушать немедленно
 * @param {Object} context Контекст выполнения функции listener
 * @param {Boolean} runOnce Обработать событие один раз и удалить обработчик. По умолчанию, false.
 *
 * @example
 * new y5.Observer("customEvent", callback, object, true);
 */
y5.Observer = function(type, listener, element, add, context, runOnce) {
    this.id = ObserverManager.generateId(type, element);
    this.added = false;
    this.listener = listener;
    this.context = context;
    this.runOnce = runOnce || false;
    
    if (add) {
        this.add();
    }

    GC.collect(this);
};

y5.Observer.prototype = {
    /**
     * Включает (активирует) объект-слушатель.
     * @name y5.Observer.add
     * @memberOf y5.Observer
     * @function
     */
    add: function() {
        if (!this.added) {
            ObserverManager.addObserver(this);
            this.added = true;

            y5.Console.log('Observer added: ', this, ['Observer']);
        }
    },

    /**
     * Выключает (деактивирует) объект-слушатель.
     * @name y5.Observer.remove
     * @memberOf y5.Observer
     * @function
     */
    remove: function() {
        if (this.added) {
            ObserverManager.removeObserver(this);
            this.added = false;

            y5.Console.log('Observer removed: ', this, ['Observer']);
        }
    },

    /**
     * Удаляет обработчик события, ссылки на элемент и callback-функцию.
     * @name y5.Observer.cleanup
     * @memberOf y5.Observer
     * @function
     * @private
     */
    cleanup: function() {
        this.remove();
        this.context = null;
        this.listener = null;
    }
};

var ObserverProto = y5.Observer.prototype;

/**
 * @memberOf y5.Observer
 * @name y5.Observer.start
 * @function
 * @deprecated y5.Observer.add
 */
ObserverProto.start = ObserverProto.add;

/**
 * @memberOf y5.Observer
 * @name y5.Observer.stop
 * @function
 * @deprecated y5.Observer.remove
 */
ObserverProto.stop = ObserverProto.remove;

/**
 * Подготовливает пользовательское событие и, если параметр dispatch равен true, инициирует его возникновение.
 * @class Класс для реализации совместно с классом y5.Observer механизма пользовательских событий. Позволяет расширить стандартный набор событий браузера новыми, проблемно-ориентированными типами. Класс y5.Notifier отвечает за инициализацию событий и отправку сообщений объектам-слушателям, созданным при помощи класса y5.Observer.
 * @constructor
 * @name y5.Notifier
 * @param {String} type Тип события
 * @param {Object} object Объект, на котором возникает событие
 * @param {Boolean} [dispatch] Послать сообщение немедленно (true - по умолчанию)
 * @param {Object} [params] Параметры сообщения
 *
 * @example
 * new y5.Notifier("customEvent", object, true, {foo: 'bar'});
 */
y5.Notifier = function(type, element, dispatch, params) {
    this.id = ObserverManager.generateId(type, element);
    this.params = params;

    y5.Console.log('Notifier new: ', this, ['Notifier']);

    if (typeof dispatch == UNDEF) {
        dispatch = true;
    }

    if (dispatch) {
        this.dispatch();
    }
};

y5.Notifier.prototype = {
    /**
     * Инициирует возникновение события - отправляет сообщение. Возвращает true, если общий результат выполнения функций объекта-слушателя равен true.
     * @name y5.Notifier.dispatch
     * @memberOf y5.Notifier
     * @function
     * @param {Object} [params] Параметры, передаваемые функции слушателя
     * @param {Boolean} [permanent] "Постоянное" событие
     * @returns {Boolean} Общий результат выполнения функций слушателя
     *
     * @example
     * var notify = new y5.Notifier("customEvent", object, false, {foo: 'bar'});
     *
     * // передать слушателю новые параметры
     * notify.dispatch({foo: 'yadda'});
     *
     * // передать слушателю параметры заданные при создании
     * notify.dispatch();
     */
    dispatch: function(params, permanent) {
        if (typeof params == UNDEF) {
            params = this.params;
        }

        y5.Console.group('Notifier dispatch: ', [this, params], ['Notifier']);
        var result = ObserverManager.dispatchNotify(this.id, params, permanent);
        y5.Console.groupEnd();

        return result;
    }
};

/**
 * Инициирует возникновение пользовательского события. Возвращает true, если общий результат выполнения функций объекта-слушателя равен true.
 * @class Класс для создания пользовательского события. Упрощенный вариант y5.Notifiler.
 * @name y5.Notify
 * @param {String} type Тип события
 * @param {Object} object Объект, на котором возникает событие
 * @param {Object} [params] Параметры, передаваемые функции слушателя
 * @param {Boolean} [permanent] "Постоянное" событие
 * @returns {Boolean} Общий результат выполнения функций слушателя
 *
 * @example
 * y5.Notify("customEvent", object, {foo: 'bar'});
 */
y5.Notify = function(type, element, params, permanent) {
    y5.Console.group('Notifier dispatch: ', [this, params], ['Notifier']);
    var id = ObserverManager.generateId(type, element);
    var result = ObserverManager.dispatchNotify(id, params, permanent);
    y5.Console.groupEnd();

    return result;
};

})();

/// Events Factory
/////////////////////////////////////////////////////////////////////

/**
 * Создает один объект-слушатель.
 * @function
 * @private
 */
function observeEvent (type, listener, element, add, context, runOnce) {
    var obj = isCustom(type) ? y5.Observer : AEventListener;
    return new obj(type, listener, element, add, context, runOnce);
}

function isCustom (type) {
    return type.indexOf(':') != -1;
}

/**
 * @class Набор функций для инициализации стандартных событий браузера, изменения их свойств, а также для создания слушателей — объектов, отслеживающих возникновение данных событий.
 * @static
 * @name y5.Events
 */
y5.Events = {
    /**
     * Создает один или несколько объектов-слушателей.
     * @name y5.Events.observe
     * @memberOf y5.Events
     * @function
     * @param {String | Array} types Тип события: click, load...
     * @param {Function} listener Callback-функция на возникновение события
     * @param {Element | Object} element Объект, на котором возникает событие
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @param {Boolean} [runOnce] Обработать событие один раз и удалить обработчик. По умолчанию, false.
     * @returns {y5.AEventListener | y5.Observer | Array}
     *
     * @example
     * y5.Events.observe("click", callback, element, true);
     */
    observe: function(types, listener, element, add, context, runOnce) {
        if (!element) {
            return {add: y5.NULL, remove: y5.NULL};
        }

        switch (Types.type(types)) {
            case Types.ARRAY:
                var length = types.length,
                    events = new Array(length),
                    i=0;

                for (; i < length; i++) {
                    events[i] = observeEvent(types[i], listener, element, add, context, runOnce);
                }
                
                return events;

            case Types.STRING:
                return observeEvent(types, listener, element, add, context, runOnce);
        }

        return null;
    },
    
    /**
     * Создает один или несколько одноразовых объектов-слушателей, обработчики удаляются после наступления события.
     * @name y5.Events.observeOnce
     * @memberOf y5.Events
     * @function
     * @param {String | Array} types Тип события: click, load...
     * @param {Function} listener Callback-функция на возникновение события
     * @param {Element | Object} element Объект, на котором возникает событие
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @returns {y5.AEventListener | y5.Observer | Array}
     *
     * @example
     * y5.Events.observe("click", callback, element, true);
     */
    observeOnce: function (types, listener, element, add, context) {
        return this.observe(types, listener, element, add, context, true);
    },

    /**
     * Создает событие и, если не задан параметр dispatch, инициирует его  возникновение.
     * @name y5.Events.notify
     * @memberOf y5.Events
     * @function
     * @param {String} [type] Тип события: click, mousedown... (если не указан, то используется click)
     * @param {Object} [element] Целевой элемент события (если не указан, то используется document)
     * @param {Boolean} [dispatch] Отправить событие (если не указано, то событие возникает сразу)
     * @param {Object} [event] Параметры события
     * @param {Boolean} [permanent] "Постоянное" событие
     * @returns {y5.Event | y5.Notifier}
     *
     * @example
     * // создать событие "mousedown" на элементе
     * y5.Events.notify("mousedown", element);
     *
     * // создать событие "click" на документе
     * y5.Events.notify();
     *
     * // создать событие "mousedown" на элементе
     * var evt = y5.Events.notify("mousedown", element, false);
     * // ...
     * // выполнить его позже
     * // evt.dispatch()
     */
    notify: function(type, element, dispatch, event, permanent) {
        var obj = isCustom(type) ? y5.Notifier : y5.Event;
        return new obj(type, element, dispatch, event, permanent);
    },

    /**
     * Создает объект-слушатель, реагирующий на изменение определенного свойства элемента (например, на изменение содержимого поля ввода).
     * @name y5.Events.observeProperty
     * @memberOf y5.Events
     * @function
     * @param {String} property Свойство
     * @param {Function} listener Callback-функция
     * @param {Element} element Элемент
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @param {Boolean} [runOnce] Обработать событие один раз и удалить обработчик. По умолчанию, false.
     * @returns {y5.AEventListener}
     * @note Не работает в Safari 3.0.4, Firefox 1.0
     *
     * @example
     * y5.Events.observeProperty("value", callback, element, true);
     */
    observeProperty: function(property, listener, element, add, context, runOnce) {
        // Внимание! Не используем контекст для listenerAttr,
        // только для listener
        function listenerAttr(evt) {
            if (evt.attrName == property) {
                execListenerWithContext(listener, context, evt, element);
            }
        }
        return new AEventListener(typeDOMAttrModified, listenerAttr, element, add, null, runOnce);
    },
    
    /**
     * Одноразовый слушатель изменения свойств элемента. После наступления события обработчик удаляется.
     * @name y5.Events.observePropertyOnce
     * @memberOf y5.Events
     * @function
     * @param {String} property Свойство
     * @param {Function} listener Callback-функция
     * @param {Element} element Элемент
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @returns {y5.AEventListener}
     * @note Не работает в Safari 3.0.4, Firefox 1.0
     *
     * @example
     * y5.Events.observeProperty("value", callback, element, true);
     */
    observePropertyOnce: function(property, listener, element, add, context) {
        // Внимание! Не используем контекст для listenerAttr,
        // только для listener
        function listenerAttr(evt) {
            if (evt.attrName == property) {
                execListenerWithContext(listener, context, evt, element);
            }
        }
        return new AEventListener(typeDOMAttrModified, listenerAttr, element, add, null, true);
    }
};

var Events = y5.Events;

// aliases

/**
 * Создает один или несколько объектов-слушателей.
 * @name y5.on
 * @see y5.Events.observe
 * @memberOf y5
 * @function
 */
y5.on = Events.observe;

/**
 * Создает событие и инициирует его возникновение.
 * @name y5.fire
 * @see y5.Events.notify
 * @memberOf y5
 * @function
 */
y5.fire = Events.notify;

// deprecated
    
/**
 * @memberOf y5.Events
 * @name y5.Events.create
 * @function
 * @deprecated y5.Events.observe
 */
Events.create = Events.observe;

/**
 * @memberOf y5.Events
 * @name y5.Events.make
 * @function
 * @deprecated y5.Events.notify
 */
Events.make = Events.notify;

/**
 * @memberOf y5.Events
 * @name y5.Events.PropertyListener
 * @function
 * @deprecated y5.Events.observeProperty
 */
Events.PropertyListener = Events.observeProperty;


/// dom:loaded notify
/////////////////////////////////////////////////////////////////////
(function() {

var timer, type = 'load';

function notifyDomLoaded() {
    if (y5.domloaded) {
        return;
    }

    // устанавливаем глобальный флаг
    y5.domloaded = true;

    if (timer) {
        window.clearInterval(timer);
    }

    // обнуляем эту функцию, чтобы вызов был только один раз
    notifyDomLoaded = y5.VOID;

    // посылаем сообщение и делаем его постоянным
    y5.Notify('dom:loaded', y5, null, true);
}

if (document.addEventListener) {

    if (y5.is_safari || y5.is_khtml) {
        var loadedRegex = /loaded|complete/;

        timer = window.setInterval(function() {
            if (loadedRegex.test(document.readyState)) {
                notifyDomLoaded();
            }
        }, 0);
    } else if (y5.is_opera && y5.opera_ver < 9) {
        ;
    } else {
        type = 'DOMContentLoaded';
    }

} else {
    var doc = document;

    function isSet(property) {
        return typeof doc[property] != 'undefined';
    }

    function poll() {
        if (doc.body !== null && doc.getElementsByTagName) {
            if (isSet('readyState') && (/loaded|complete/).test(doc.readyState)) {
                notifyDomLoaded();
            }
            if (isSet('fileSize')) {
                try {
                    doc.documentElement.doScroll('left');
                    notifyDomLoaded();
                } catch (e) {}
            }
        }
    }

    timer = window.setInterval(poll, 10);
}

y5.Events.observe(type, notifyDomLoaded, window, true);

})();

/// Init GC
/////////////////////////////////////////////////////////////////////

// В старых gecko-браузерах вызывает крах
if (!(y5.is_gecko && y5.gecko_ver < 1.8 || y5.is_opera)) {
    // Register the cleanup handler on page unload
    new AEventListener('unload', GC.cleanup, window, true, GC);
}

})();