/**
 * @class Модуль для реализации механизма &laquo;сборки мусора&raquo; &mdash; освобождения памяти от отработавших объектов, которые больше не будут востребованы приложением.
 * @name y5.GC
 * @static
 */
y5.GC = {
    /**
     * Массив объектов предназначенныз для удаления
     * @type Array
     * @name y5.GC.data
     * @memberOf y5.GC
     * @private
     */
    data: [],

    /**
     * Регистрирует объект в сборщике мусора. Возвращает зарегистрированный объект.
     * @name y5.GC.collect
     * @memberOf y5.GC
     * @function
     * @param {Object} obj объект
     * @returns {Object} сам объект
     */
    collect: function(obj) {
        this.data.push(obj);
        return obj;
    },

    /**
     * Удаляет объект из коллекции сборщика мусора.
     * @name y5.GC.remove
     * @memberOf y5.GC
     * @function
     * @param {Object} obj объект
     */
    remove: function(obj) {
        var i = this.data.indexOf(obj);
        if (i != -1) {
            this.destruct(i);
            this.data.splice(i, 1);
        }
    },

    /**
     * Вызывает деструктор сохраненного объекта.
     * @name y5.GC.destruct
     * @memberOf y5.GC
     * @function
     * @private
     */
    destruct: function(i) {
        var obj = this.data[i];
        if (obj) {
            if (typeof obj.cleanup == 'function') {
                obj.cleanup();
            } else if (typeof obj.destruct == 'function') {
                obj.destruct();
            }
        }
        this.data[i] = null;
    },

    /**
     * Удаляет все сохраненные объекты.
     * @name y5.GC.cleanup
     * @memberOf y5.GC
     * @function
     * @private
     */
    cleanup: function() {
        for (var i = this.data.length - 1; i >= 0; i--) {
            this.destruct(i);
        }
        this.data.length = 0;

        if (y5.is_ie && CollectGarbage) {
            CollectGarbage();
        }
    }
};
