y5.require('Request', function() {

var UNDEF = y5.UNDEF;

/**
 * Создает экземпляр класса для выполнения Ajax-запросов с помощью объекта XMLHttpRequest.
 * @class Класс для выполнения Ajax-запроса с помощью объекта XMLHttpRequest. Использование XMLHttpRequest позволяет загружать внешние документы и использовать функции DOM для доступа к их содержимому. Одним из ограничений загрузки данных при помощи XMLHttpRequest является невозможность выполнения запросов между разными доменами. Для междоменных запросов следует использовать y5.Request.Script или y5.Request.Iframe.
 * При использовании XMLHttpRequest на сервер отправляются дополнительные заголовки:
 * X-Requested-With: XMLHttpRequest
 * Accept: text/javascript,text/html,application/xml,text/xml,text/plain,&#42;/&#42;
 * @constructor
 * @name y5.Request.XML
 * @extends y5.Request
 */
function RequestXML(url, params) {
    this.Request(url, params);
}

RequestXML.prototype = {
    /**
     * Параметры по умолчанию.
     * @memberOf y5.Request.XML
     * @name y5.Request.XML.params
     * @type Object
     * @private
     * @example
     * // параметры по умолчанию
     * {
     *     async: true,
     *     contentType: 'application/x-www-form-urlencoded',
     *     encoding: 'UTF-8',
     *     headers: {
     *         'X-Requested-With': 'XMLHttpRequest',
     *         'Accept': 'text/javascript,text/html,application/xml,text/xml,text/plain,&#42;/&#42;'
     *     }
     * }
     */
    defaultParams: {
        async: true,

        contentType: 'application/x-www-form-urlencoded',

        encoding: 'UTF-8',

        headers: {
            'X-Requested-With': 'XMLHttpRequest',
            'Accept': 'text/javascript,text/html,application/xml,text/xml,text/plain,*/*'
        }
    },

    abort: function() {
        try {
            this.req.abort();
        } catch (e) {}

        this.dispatch('abort');
    },

    _send: function(query) {
        var _this = this,
            params = this.params,
            method = params.method,
            async = params.async,
            get = method == 'get',
            post = method == 'post',
            url = this.url;

        if (typeof XMLHttpRequest != UNDEF) {
            this.req = new XMLHttpRequest();
        } else if (typeof ActiveXObject != UNDEF) {
            this.req = new ActiveXObject('Microsoft.XMLHTTP');
            /*
            var objects = ['MSXML2.XMLHTTP.3.0', 'Microsoft.XMLHTTP'];
            for (var i = 0, l = objects.length; i < l; i++) {
                if ((this.req = new ActiveXObject(objects[i]))) {
                    break;
                }
            }
            */
        } else {
            return false;
        }

        if (get && query) {
            url = url.clone();
            url.query(query);
            query = null;
        }

        if (post && !query) {
            url = url.clone();
            query = url.query();
            url.clearQuery();
        }

        // toUpperCase для Opera 8
        this.req.open(method.toUpperCase(), url, async);

        if (async) {
            this.req.onreadystatechange = function() {
                _this.onStateChange();
            };
        }

        var headers = y5.Utils.objectCopy({}, params.headers);
        if (post) {
            var encoding = params.encoding;
            headers['Content-Type'] = params.contentType + (encoding ? '; charset=' + encoding : '');
            if (y5.is_gecko && y5.gecko_ver < 1.8) {
                headers['Connection'] = 'close'; // Mozilla Bug #246651
            }
        }
        for (var name in headers) {
            this.req.setRequestHeader(name, headers[name]);
        }

        this.req.send(query);

        if (!async) {
            this.onStateChange();
        }
    }
};

if (y5.is_ie7down) {

    RequestXML.prototype.onStateChange = function() {

        var req = this.req;

        if (this.req.readyState == 4) {
            this.req = {
                status: req.status,
                readyState: req.readyState,
                responseText: req.responseText,
                responseXML: req.responseXML
            };

            this.req.getResponseHeader = function(name) {
                return req.getResponseHeader(name);
            };

            this.req.abort = function() {
                return req.abort();
            };
        }
        this.Request.prototype.onStateChange.apply(this);
    };
}

y5.Request.ext('XML', RequestXML);

});