(function() {

var UNDEF = y5.UNDEF,
    Types = y5.Types,
    urlRegexp = /^((((\w+):)\/\/)?(([\w\-\.]+\.\w+|localhost)(\:(\d+))?))?(\/?[^\?#]*)?(\?([^#]*))?(#(.*))?$/,
    validURLRegexp = /^(([\w]+:)?\/\/)?(([\d\w]|%[a-fA-f\d]{2,2})+(:([\d\w]|%[a-fA-f\d]{2,2})+)?@)?([\d\w][-\d\w]{0,253}[\d\w]\.)+[\w]{2,4}(:[\d]+)?(\/([-+_~.\d\w]|%[a-fA-f\d]{2,2})*)*(\?(&?([-+_~.\d\w]|%[a-fA-f\d]{2,2})=?)*)?(#([-+_~.\d\w]|%[a-fA-f\d]{2,2})*)?$/,
    winHex = 'E9F6F3EAE5EDE3F8F9E7F5FAF4FBE2E0EFF0EEEBE4E6FDFFF7F1ECE8F2FCE1FEB8C9D6D3CAC5CDC3D8D9C7D5DAD4DBC2C0CFD0CECBC4C6DDDFD7D1CCC8D2DCC1DEA8'.match(/../g),
    utfHex = '%D0%B9%D1%86%D1%83%D0%BA%D0%B5%D0%BD%D0%B3%D1%88%D1%89%D0%B7%D1%85%D1%8A%D1%84%D1%8B%D0%B2%D0%B0%D0%BF%D1%80%D0%BE%D0%BB%D0%B4%D0%B6%D1%8D%D1%8F%D1%87%D1%81%D0%BC%D0%B8%D1%82%D1%8C%D0%B1%D1%8E%D1%91%D0%99%D0%A6%D0%A3%D0%9A%D0%95%D0%9D%D0%93%D0%A8%D0%A9%D0%97%D0%A5%D0%AA%D0%A4%D0%AB%D0%92%D0%90%D0%9F%D0%A0%D0%9E%D0%9B%D0%94%D0%96%D0%AD%D0%AF%D0%A7%D0%A1%D0%9C%D0%98%D0%A2%D0%AC%D0%91%D0%AE%D0%81'.match(/.{6}/g),
    win2utfTable = {},
    encode = encodeURIComponent,
    fileExt = ['.xml', '.html', '.jpg', '.gif', '.png', '.xhtml', '.php', '.xsl', '.py', '.pl'],
    i = 0, l = winHex.length,
    // Символы ; , / ? : @ & = + $ не кодируются и не раскодируются через encodeURI,
    // в pathname их надо обрабатывать отдельно
    reservedCharactersRegexp = /%3B|%2C|%2F|%3F|%3A|%40|%26|%3D|%2B|%24|%23/gi;

for (; i < l; i++) {
    win2utfTable[winHex[i]] = utfHex[i];
}

function win2utf(token, code) {
    return win2utfTable[code] || token;
}

function decodeWin(value) {
    return value.replace(/%([A-Fa-f0-9]{2})/g, win2utf);
}

/**
 * Декодирует строку.
 * @param {String} string Строка
 * @returns {String} Результат перекодировки
 * @private
 */
function decode(value) {
    var val = value.replace(/\+/g, '%20');

    try {
        var str = decodeURIComponent(val);
        if (str == undefined) {
            throw 'malformed URI sequence';
        }
        return str;
    } catch (e) {
        try {
            return decodeURIComponent(decodeWin(val));
        } catch (e) {
            return unescape(val);
        }
    }
}

function decodeURI_win (value) {
    try {
        var str = decodeURI(value);
        if (str == undefined) {
            throw 'malformed URI sequence';
        }
        return str;
    } catch (e) {
        try {
            return decodeURI(decodeWin(value));
        } catch (e) {
            return unescape(value);
        }
    }
}

/**
 * Добавляет параметр к запросу.
 * @param {Object} query Объект запроса
 * @param {String} string Имя параметра
 * @param {String} string Значение параметра
 * @private
 */
function addParam(Query, name, value) {
    // make array
    if (Types.undef(Query[name])) {
        Query[name] = [];
    }
    if (Types.array(value)) {
        Query[name] = Query[name].concat(value);
    } else {
        Query[name].push(value);
    }
}

function getParamQuery(Query, name) {
    var list = Query[name],
        len = list.length,
        result = new Array(len),
        enc_name = encode(name),
        i = 0;

    for (; i < len; i++) {
        result[i] = enc_name + '=' + encode(list[i]);
    }

    return result;
}

/**
 * Разбирает параметры запроса.
 * @param {String} queryString параметры запроса
 * @returns {Object} результат в виде объекта
 * @private
 */
function parseQuery (queryString) {
    var query = {},
        parts = queryString.replace(/\+/g, '%20').split('&'),
        pair, name, value,
        i = 0,
        l = parts.length;

    for (; i < l; i++) {
        pair = parts[i].split('=');
        name = pair.shift();
        if (name) {
            value = pair.length > 1 ? pair.join('=') : (pair[0] || '');
            addParam(query, decode(name), decode(value));
        }
    }

    return query;
}

/**
 * Создает экземпляр класса для работы с URL и его параметрами.
 * @constructor
 * @name y5.URL
 * @class Класс для работы с URL и его частями. Содержит набор методов для упрощения операций, связанных с обработкой параметров, переданных в URL, и использованием полученных данных внутри приложения.
 * @param {String} href URL
 *
 * @example
 * var url = new y5.URL("http://www.yandex.ru/");
 * url.host(); // -> "www.yandex.ru"
 * url.port(); // -> "http"
 *
 * url.path("/yandsearch").query({text: 'test'});
 * url.go(); // -> "http://www.yandex.ru/yandsearch?text=test"
 *
 * // пустой URL
 * var url = new y5.URL("");
 * url.path("www.yandex.ru");
 * url.toString(); // -> "http://www.yandex.ru" (http по умолчанию)
 *
 * // по умолчанию
 * var url = new y5.URL();
 * url.toString() == window.location.href; // -> true
 */
y5.URL = function(href) {
    href = Types.def(href) ? href.toString() : window.location.href;
    // поиск запроса в url
    var match = href.match(urlRegexp);


    if (!match) {
        throw new y5.Exception('This is not an url', 'constructor', 'URL');
    }
    this.Host  = match[6] || '';
    this.Path = match[9] || '';

    if (match[6]) {
        // чтобы отличить yandex.ru от index.html, проверяем хост на распространенные расширения файлов
        var ext = match[6].substring(match[6].lastIndexOf('.')),
            extIndex = fileExt.indexOf(ext);
        // если расширение совпало, значит хост - путь до файла
        if (extIndex != -1) {
            this.Path = match[6];
            this.Host = '';    
        }
    }
    
    this.Href  = match[0];
    this.Proto = match[4] || '';
    this.Port  = match[8] || 0;
    this.Path  = decodeURI_win(this.Path);
    this.Query = parseQuery(match[11] || '');
    this.Hash  = decode(match[13] || '');
};

y5.URL.prototype = {
    /**
     * Выполняет переход по URL согласно текущему состоянию объекта y5.URL.
     * @name y5.URL.go
     * @memberOf y5.URL
     * @function
     */
    go: function() {
        window.location.href = this.toString();
    },

    /**
     * Формирует URL со всеми параметрами из содержимого объекта. Возвращает получившийся URL в виде строки.
     * @name y5.URL.toString
     * @memberOf y5.URL
     * @function
     * @returns {String} URL
     */
    toString: function() {
        var result = '';

        // proto & hostname
        if (this.Proto || this.Host) {
            if (this.Host) {
                result += (this.Proto || 'http') + '://' + this.Host;
                if (this.Port) {
                    result += ':'+this.Port;
                }
            }
        }

        // path
        if (this.Path) {
            if (this.Host && this.Path.indexOf('/') != 0) {
                result += '/';
            }
            /*
            pathname ведет себя следующим образом:
             - для урл сначала делается decodeURI, потом encodeURI
               /"quote"/ -> /%22quote%22/
               /%22quote%22/ -> /%22quote%22/
             - но не преобразовываются как сами зарезервированные символы - ; , / ? : @ & = + $, так и их их url-кодированные значения,
              это приводит к тому, что переданное значение www.yandex.ru/url/f%3Dx%2F2/index.xml преобразуется в www.yandex.ru/url/f=x/2/index.xml, чего допустить нельзя
             - пути f=2 и f%3D2 равнозначны, а f%3Dx%2F2 и f=x/2 - нет

             Поэтому разбиваем pathname некодируемыми символами и кодируем получившиеся части отдельно
             т.к. /f%3Dx%2F2/comments должно остаться непреобразованным
             */

            // тут можно использовать this.Path.split(reservedCharactersRegexp),
            // но IE не возвращает сепаратор в результирующий массив
            // см. http://msdn.microsoft.com/en-us/library/microsoft.jscript.stringprototype.split.aspx

            // обнуляем регулярку
            reservedCharactersRegexp.lastIndex = 0;
            var match, lastFindedIndex = 0, chunks = [];
            while (match = reservedCharactersRegexp.exec(this.Path)) {
                if (reservedCharactersRegexp.lastIndex > lastFindedIndex) {
                    if (match[0]) {
                        chunks.push(this.Path.slice(lastFindedIndex, match.index));
                        chunks.push(match[0]);
                        lastFindedIndex = match.index + match[0].length;
                    }
                }
            }
            if (lastFindedIndex !== this.Path.length) {
                chunks.push(this.Path.slice(lastFindedIndex));
            }

            chunks[0] = encodeURI(chunks[0]);

            // собираем обратно строку
            result += chunks.reduce(function(result, current, index){
                if (!(index % 2)) {
                    current = encodeURI(current);
                }
                return result + current;
            });
        }

        // query
        var query = this.query();
        if (query) {
            result += '?' + query;
        }

        // hash
        if (this.Hash) {
            result += '#' + encode(this.Hash);
        }

        return result;
    },

    /**
     * Создает новый объект y5.URL из данного. Возвращает копию исходного объекта.
     * @name y5.URL.clone
     * @memberOf y5.URL
     * @function
     * @returns {y5.URL} Копия объекта
     */
    clone: function() {
        return new y5.URL(this.toString());
    },

/// getters/setters

    /**
     * Устанавливает или получает текущее значение протокола URL. Если вызывается без параметра, то возвращает текущее значение протокола. Если вызывается с параметром, то заменяет текущее значение протокола на заданное в параметре и возвращает исходный объект с измененным значением протокола.
     * @name y5.URL.proto
     * @memberOf y5.URL
     * @function
     * @param {String} [string] Новый протокол
     * @returns {String | this} Протокол или сам объект
     * @example
     * var url = new y5.URL("http://www.yandex.ru/");
     * url.proto(); // -> "http"
     *
     * url.proto("FOO");
     * url.toString(); // -> "FOO://www.yandex.ru/"
     */
    proto: function() {
        /* if (typeof value != UNDEF) {
            this.Proto = value;
            return this;
        }

        return this.Proto; */
    },

    /**
     * Устанавливает или получает текущее значение хоста URL. Если вызывается без параметра, то возвращает текущее значение хоста. Если вызывается с параметром, то заменяет текущее значение хоста на заданное в параметре и возвращает исходный объект с измененным значением хоста.
     * @name y5.URL.host
     * @memberOf y5.URL
     * @function
     * @param {String} [string] Новый хост
     * @returns {String | this} Хост или сам объект
     * @example
     * var url = new y5.URL("http://www.yandex.ru/");
     * url.host(); // -> "www.yandex.ru"
     *
     * url.host("ya.ru");
     * url.toString(); // -> "http://ya.ru/"
     */
    host: function() {
        /* if (typeof value != UNDEF) {
            this.Host = value;
            return this;
        }

        return this.Host; */
    },

    /**
     * Устанавливает или получает текущее значение порта URL. Если вызывается без параметра, то возвращает текущее значение порта. Если вызывается с параметром, то заменяет текущее значение порта на заданное в параметре и возвращает исходный объект с измененным значением порта.
     * @name y5.URL.port
     * @memberOf y5.URL
     * @function
     * @param {Number} [string] Новый порт
     * @returns {Number | this} Порт или сам объект
     * @example
     * var url = new y5.URL("http://www.yandex.ru/");
     * url.port(); // -> 0
     *
     * url.port(8080);
     * url.toString(); // -> "http://www.yandex.ru:8080/"
     */
    port: function() {
        /* if (typeof value != UNDEF) {
            this.Port = value;
            return this;
        }

        return this.Port; */
    },

    /**
     * Устанавливает или получает текущее значение пути URL (содержимого URL без хоста и набора параметров). Если вызывается без параметра, то возвращает текущее значение пути. Если вызывается с параметром, то заменяет текущее значение пути на заданное в параметре и возвращает исходный объект с измененным значением пути.
     * @name y5.URL.path
     * @memberOf y5.URL
     * @function
     * @param {String} [string] Новый путь
     * @returns {String | this} Путь или сам объект
     * @example
     * var url = new y5.URL("http://www.yandex.ru/");
     * url.path(); // -> "/"
     *
     * url.path("/yandsearch");
     * url.toString(); // -> "http://www.yandex.ru/yandsearch"
     */
    path: function() {
        /* if (typeof value != UNDEF) {
            this.Path = value;
            return this;
        }

        return this.Path; */
    },

    /**
     * Устанавливает или получает текущее значение якоря URL. Если вызывается без параметра, то возвращает текущее значение якоря. Если вызывается с параметром, то устанавливает заданный в параметре якорь и возвращает исходный объект с добавленным якорем.
     * @name y5.URL.hash
     * @memberOf y5.URL
     * @function
     * @param {String} [string] Новый якорь
     * @returns {String | this} Якорь или сам объект
     * @example
     * var url = new y5.URL("http://www.yandex.ru/#top");
     * url.hash(); // -> "top"
     *
     * url.hash("bottom");
     * url.toString(); // -> "http://www.yandex.ru/#bottom"
     */
    hash: function() {
        /* if (typeof value != UNDEF) {
            this.Hash = value;
            return this;
        }

        return this.Hash; */
    },

    /**
     * Устанавливает или получает текущее значение строки запроса URL. Если вызывается без параметра, то возвращает текущее значение запроса. Если вызывается с параметром, то заменяет текущее значение запроса на заданное в параметре и возвращает исходный объект с измененным значением запроса.
     * @name y5.URL.query
     * @memberOf y5.URL
     * @function
     * @param {Object | String} [query] Новые параметры запроса
     * @returns {String | this} Строка запроса или сам объект
     * @example
     * var url = new y5.URL("http://www.yandex.ru/yandsearch?text=test");
     * url.query(); // -> "text=test"
     *
     * url.query({foo: "bar"});
     * url.toString(); // -> "http://www.yandex.ru/yandsearch?foo=bar"
     *
     * url.query("foo=bar");
     * url.toString(); // -> "http://www.yandex.ru/yandsearch?foo=bar"
     */
    query: function(value) {
        if (Types.def(value)) {
            this.clearQuery();

            if (Types.string(value)) {
                this.Query = parseQuery(value);
            } else {
                this.replaceParams(value);
            }

            return this;
        }

        var result = [];
        
        for (var name in this.Query) {
            result = result.concat(getParamQuery(this.Query, name));
        }

        return result.join('&');
    },

    /**
     * Разбирает URL на составляющие и возвращает список имен параметров запроса в виде массива. Если URL не содержит параметров, то возвращается пустой массив.
     * @name y5.URL.queryKeys
     * @memberOf y5.URL
     * @function
     * @returns {Array} Список ключей
     * @example
     * var url = new y5.URL("http://www.yandex.ru/yandsearch?text=test&amp;foo=bar");
     * url.queryKeys(); // -> ['text', 'foo']
     */
    queryKeys: function() {
        var result = [];
        for (var name in this.Query) {
            result.push(name);
        }
        return result.sort();
    },

/// Методы работы с параметрами

    /**
     * Добавляет параметр с именем name и значением value к URL. Возвращает исходный объект с добавленными данными.
     * @name y5.URL.addParam
     * @memberOf y5.URL
     * @function
     * @param {String} name Название параметра
     * @param {String} value Значение параметра
     * @returns {this}
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test");
     * url.addParam("stype", "www");
     * url.toString(); // -> "http://ya.ru/?text=test&amp;stype=www"
     */
    addParam: function(name, value) {
        addParam(this.Query, name, value);

        return this;
    },

    /**
     * Добавляет набор параметров, переданных в виде хеша к URL. Возвращает исходный объект с добавленными данными.
     * @name y5.URL.addParams
     * @memberOf y5.URL
     * @function
     * @param {Object} params Хэш
     * @returns {this}
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test");
     * url.addParams({stype: "www"});
     * url.toString(); // -> "http://ya.ru/?text=test&amp;stype=www"
     */
    addParams: function(params) {
        for (var name in params) {
            addParam(this.Query, name, params[name]);
        }

        return this;
    },

    /**
     * Удаляет набор параметров, переданных в виде массива имен из URL. Возвращает измененный исходный объект.
     * @name y5.URL.removeParams
     * @memberOf y5.URL
     * @function
     * @param {Array} params Удаляемый параметр запроса
     * @returns {this}
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test&amp;stype=www");
     * url.removeParam(["stype"]);
     * url.toString(); // -> "http://ya.ru/?text=test"
     */
    removeParams: function(params) {
        for (var i = 0, l = params.length; i < l; i++) {
            delete this.Query[params[i]];
        }

        return this;
    },

    /**
     * Заменяет текущий набор параметров URL на набор параметров, переданных в виде объекта params. Возвращает измененный исходный объект.
     * @name y5.URL.replaceParams
     * @memberOf y5.URL
     * @function
     * @param {Object} params Заменяемый параметр запроса
     * @returns {this}
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test&amp;stype=www");
     * url.replaceParams({stype: "images"});
     * url.toString(); // -> "http://ya.ru/?text=test&amp;stype=images"
     */
    replaceParams: function(params) {
        var list = [];
        for (var name in params) {
            list.push(name);
        }
        this.removeParams(list);
        this.addParams(params);

        return this;
    },

    /**
     * Удаляет все параметры из URL. Возвращает измененный исходный объект.
     * @name y5.URL.clearQuery
     * @memberOf y5.URL
     * @function
     * @returns {this}
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test&amp;stype=www");
     * url.clearQuery();
     * url.toString(); // -> "http://ya.ru/"
     */
    clearQuery: function() {
        this.Query = {};
        return this;
    },

    /**
     * Находит параметр с заданным именем и возвращает его значение. Если параметра с заданным именем в URL найти не удалось, то возвращается null. Если в URL содержится несколько параметров с заданным именем, то возвращается значение первого из них.
     * @name y5.URL.getParam
     * @memberOf y5.URL
     * @function
     * @param {String} name Имя параметра
     * @returns {String | null} Значение параметра запроса
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test&amp;text=www");
     *
     * // параметр присутствует в запросе
     * url.getParam("text"); // -> "test"
     *
     * // параметр отсутствует в запросе
     * url.getParam("foo"); // -> null
     */
    getParam: function(name) {
        var params = this.Query[name];
        return params ? params[0] : null;
    },

    /**
     * Находит все параметры с заданным именем и возвращает массив их значений. Если ни одного параметра с заданным именем найти не удалось, то возвращается пустой массив.
     * @name y5.URL.getParams
     * @memberOf y5.URL
     * @function
     * @param {String} name Имя параметра
     * @returns {Array | []} Список значений параметра
     * @example
     * var url = new y5.URL("http://ya.ru/?text=test&amp;text=www");
     *
     * // параметры присутствуют в запросе
     * url.getParams("text"); // -> ["test", "www"]
     *
     * // параметры отсутствуют в запросе
     * url.getParams("foo"); // -> []
     */
    getParams: function(name) {
        return this.Query[name] || [];
    }
};

var URL = y5.URL,
    URLProto = URL.prototype;

// Создаем getters/setters
var funcs = {proto: 'Proto', host: 'Host', port: 'Port', path: 'Path', hash: 'Hash'};
for (i in funcs) {
    URLProto[i] = (function(property) {
        return function(value) {
            if (Types.def(value)) {
                this[property] = value;
                return this;
            }
            return this[property];
        };
    })(funcs[i]);
}

/// aliases

/**
 * @memberOf y5.URL
 * @name y5.URL.get
 * @see y5.URL.toString
 * @function
 */
URLProto.get = URLProto.toString;

/**
 * Сокращенная функция для работы с URL.
 * @name y5.Url
 * @memberOf y5.URL
 * @param {String} href URL
 *
 * @example
 * y5.Url("http://www.yandex.ru/").path("/yandsearch").addParams({text: "test"}).go();
 * // -> "http://www.yandex.ru/yandsearch?text=test"
 */
y5.Url = function(href) {
    return new URL(href);
};

/**
 * Проверяем, что данный url валиден
 * @param {String} url URL
 * @returns {Boolean} Результат проверки
 * @name y5.URL.isValid
 * @memberOf y5.URL
 * @function
 * @static
 */
URL.isValid = function (url) {
    return validURLRegexp.test(url);    
}

y5.loaded('URL');

})();