/*
   Определение undefined для старых браузеров
*/
try {
    undefined
} catch (e) {
    window.undefined = void 0;
}

/*
    Если head не определен в теле документа,
    во всех браузерах, кроме Opera он создается автоматически,
    а Opera нужно помочь.
*/
try {
    if (!document.getElementsByTagName('head')[0]) {
        document.getElementsByTagName('html')[0].appendChild(document.createElement('head'));
    }
} catch (e) {}


/// Функции call, apply
/////////////////////////////////////////////////////////////////////

(function() {

/**
 * Методы функций для поддержки старых браузеров.
 * Включает набор функций, отсутствующих в старых версиях браузеров (например, IE 5), но содержащихся в стандарте ECMAScript.
 * @class
 * @external
 * @name Function
 */
var F = Function.prototype;

if (!F.apply) {

    var ac = 0;

    /**
     * Применяет метод одного объекта в контексте другого. Возвращает результат выполнения функции.
     * @name apply
     * @function
     * @memberOf Function
     * @jsver 1.3
     * @param {Object} context Контекст выполнения функции
     * @param {Array} [args] Массив, элементы которого передаются функции как аргументы
     * @returns {Object} Результат выполнения функции
     *
     * @example
     * var foo = new Foo();
     * function bar() { ... }
     * bar.apply(foo, [1, 2, 3]);
     */
    F.apply = function(c, a) {
        var n = '__y5_apply__' + (ac++) + '__', r;

        c = c || window;
        c[n] = this;

        switch ((a || []).length) {
            case 0: r = c[n](); break;
            case 1: r = c[n](a[0]); break;
            case 2: r = c[n](a[0], a[1]); break;
            case 3: r = c[n](a[0], a[1], a[2]); break;
            default:
                var af = [];
                if (a) {
                    var l = a.length;
                    af = new Array(l);
                    for (var i = 0; i < l; i++) {
                        af[i] = 'a[' + i + ']';
                    }
                }

                r = eval('c.' + n + '(' + af.join(',') + ')');
        }

        if (typeof c.valueOf == 'function') {
            delete c[n];
        } else {
            c[n] = undefined;
        }

        return r;
    };

}

if (!F.call) {

    /**
     * Вызывает метод одного объекта в контексте другого. Возвращает результат выполнения функции.
     * @name call
     * @function
     * @memberOf Function
     * @jsver 1.3
     * @param {Object} context Контекст выполнения функции
     * @param {Object} [arg1] Аргумент вызываемой функции
     * @param {Object} [arg2]
     * @param {Object} [...]
     * @param {Object} [argN]
     * @returns {Object} Результат выполнения функции
     *
     * @example
     * var foo = new Foo();
     * function bar() { ... }
     * bar.call(foo, 1, 2, 3);
     */
    F.call = function(context) {
        return this.apply(context, Array.prototype.slice.apply(arguments, [1]));
    };

}

})();

(function() {

/// Функции для работы с массивами
/////////////////////////////////////////////////////////////////////

/**
 * Набор функций для работы с массивами.
 * Включает реализацию функций, которые отсутствуют в некоторых старых браузерах, но содержатся в стандарте ECMAScript.
 * @class
 * @external
 * @name Array
 */
var A = Array.prototype, ArrayFuncs = {

    /**
     * Добавляет элемент в конец массива.
     * Возвращает длину полученного массива.
     * Функция изменяет исходный массив.
     * @name push
     * @function
     * @memberOf Array
     * @jsver 1.2
     * @param {Object} element1 Элемент, добавляемый в конец массива
     * @param {Object} [element2]
     * @param {Object} [elementN]
     * @returns {Number} Длина полученного массива
     *
     * @example
     * var a = ["Mars", "Pluto", "Mercury"];
     * a.push("Venus");
     * // a == ["Mars", "Pluto", "Mercury", "Venus"]
     * // a.length == 4
     */
    push: function() {
        var args = arguments;
        for (var i = 0, l = args.length; i < l; i++) {
            this[this.length] = args[i];
        }

        return this.length;
    },

    /**
     * Удаляет последний элемент массива.
     * Возвращает удаляемый элемент.
     * Функция изменяет исходный массив.
     * @name pop
     * @function
     * @memberOf Array
     * @jsver 1.2
     * @returns {Object} Удаленный элемент массива
     *
     * @example
     * var a = [1, 2, 3, 4];
     * a.pop();
     * // -> 4
     * // a == [1, 2, 3]
     *
     * var i = [];
     * typeof i.pop();
     * // -> "undefined"
     * // i == []
     */
    pop: function() {
        var element, length = this.length;

        if (length != 0) {
            element = this[length - 1];
            this.length--;
        }

        return element;
    },

    /**
     * Добавляет элемент в начало массива. Возвращает длину полученного массива.
     * Функция изменяет исходный массив.
     * @name unshift
     * @function
     * @memberOf Array
     * @jsver 1.2
     * @param {Object} element1 Элемент, добавляемый в начало массива
     * @param {Object} [element2]
     * @param {Object} [elementN]
     * @returns {Number} Длина полученного массива
     *
     * @example
     * var a = ["Mars", "Pluto", "Mercury"];
     * a.unshift("Venus");
     * // a == ["Venus", "Mars", "Pluto", "Mercury"]
     * // a.length == 4
     */
    unshift: function() {
        var i, l,
            a  = arguments,
            al = a.length,
            tl = this.length,
            nl = al + tl;

        this.length = nl;

        for (i = tl - 1, l = 0; i >= l; i--) {
            this[i + al] = this[i];
        }

        for (i = 0, l = al; i < l; i++) {
            this[i] = a[i];
        }

        return this.length;
    },

    /**
     * Изменяет содержимое массива. Позволяет удалить часть элементов массива и одновременно добавить новые. Возвращает массив удаленных элементов.
     * Функция изменяет исходный массив.
     * @name splice
     * @function
     * @memberOf Array
     * @jsver 1.2
     * @param {Number} start Индекс, с которого изменяется массив
     * @param {Number} [deleteCount] Количество удаляемых элементов (если 0 или не указан, то элементы не удаляются)
     * @param {Object} [element1] Элемент, добавляемый в массив (если элемент не определен, то функция просто удаляет элементы из массива)
     * @param {Object} [element2]
     * @param {Object} [...]
     * @param {Object} [elementN]
     * @returns {Array} Удаленные элементы массива
     *
     * @example
     * var a = [1, 2, 3, 4, 5];
     * a.splice(1, 3);
     * // удаленные элементы -> [2, 3, 4]
     * // a == [1, 5]
     *
     * var i = [1, 2, 3, 4, 5];
     * i.splice(1, 2, 6, "foo");
     * // удаленные элементы -> [2, 3]
     * // i == [1, 6, "foo", 4, 5]
     */
    splice: function(start, deleteCount) {
        var i,
            args = arguments,
            l = this.length,
            removed = [];

        start = start || 0;
        deleteCount = deleteCount || 0;

        start = start < 0 ? Math.max(l + start, 0) : Math.min(start, l);
        deleteCount = Math.min(Math.max(deleteCount, 0), l - start);

        if (deleteCount) {
            removed = this.slice(start, start + deleteCount);

            l -= deleteCount;
            for (i = start; i < l; i++) {
                this[i] = this[i + deleteCount];
            }
            this.length = l;
        }

        if (args.length > 2) {
            var il;
            var arr = this.slice(start);
            this.length = start;

            for (i = 2, il = args.length; i < il; i++) {
                this.push(args[i]);
            }
            for (i = 0, il = arr.length; i < il; i++) {
                this.push(arr[i]);
            }
        }

        return removed;
    },

    /**
     * Удаляет первый элемент массива. Возвращает удаляемый элемент.
     * Функция изменяет исходный массив.
     * @name shift
     * @function
     * @memberOf Array
     * @jsver 1.2
     * @returns {Object} Удаленный элемент массива
     *
     * @example
     * var a = [1, 2, 3, 4];
     * a.shift();
     * // -> 1
     * // a == [2, 3, 4]
     *
     * var i = [];
     * typeof i.shift();
     * // -> "undefined"
     * // i == []
     */
    shift: function() {
        var element;

        if (this.length !== 0) {
            element = this[0];
            this.splice(0, 1);
        }

        return element;
    },

    /**
     * Поиск элемента массива.
     * Ищет заданный элемент с начала массива и возвращает его индекс или -1, если необходимого элемента в массиве нет.
     * @name indexOf
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Object} searchElement Искомый элемент
     * @param {Number} [fromIndex] Индекс, с которого ищется элемент (по умолчанию - с первого элемента массива)
     * @returns {Number} Индекс искомого элемента
     *
     * @example
     * var a = [2, 5, 9];
     * var i = a.indexOf(2);
     * // i == 0
     * i = a.indexOf(7);
     * // i == -1
     */
    indexOf: function(searchElement, fromIndex) {
        fromIndex = fromIndex || 0;

        for (var length = this.length; fromIndex < length; fromIndex++) {
            if (this[fromIndex] === searchElement) {
                return fromIndex;
            }
        }

        return -1;
    },

    /**
     * Обратный поиск элемента массива.
     * Ищет заданный элемент с конца массива и возвращает его индекс или -1, если необходимого элемента в массиве нет.
     * @name lastIndexOf
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Object} searchElement Искомый элемент
     * @param {Number} [fromIndex] Индекс, с которого ищется элемент (по умолчанию - с последнего элемента массива)
     * @returns {Number} Индекс искомого элемента
     *
     * @example
     * var array = [2, 5, 9, 2];
     * var index = array.lastIndexOf(2);
     * // index == 3
     * index = array.lastIndexOf(7);
     * // index == -1
     * index = array.lastIndexOf(2, 3);
     * // index == 3
     * index = array.lastIndexOf(2, 2);
     * // index == 0
     * index = array.lastIndexOf(2, -2);
     * // index == 0
     * index = array.lastIndexOf(2, -1);
     * // index == 3
     */
    lastIndexOf: function(searchElement, fromIndex) {
        var length = this.length;

        fromIndex = fromIndex || length - 1;

        if (fromIndex < 0) {
            fromIndex += length;
        }

        for (; fromIndex >= 0; fromIndex--) {
            if (this[fromIndex] == searchElement) {
                return fromIndex;
            }
        }

        return -1;
    },

    /**
     * Тестирование каждого элемента массива при помощи функции проверки.
     * Результатом тестирования массива является true, если все функции проверки вернули true. Иначе результатом тестирования массива будет false.
     * @name every
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Function} callback Функция для проверки элемента массива (должна возвращать true или false)
     * @param {Object} [thisObject] Контекст выполнения функции проверки
     * @returns {Boolean} true - если все функции проверки вернули true, иначе - false
     *
     * @example
     * function isBigEnough(element, index, array) {
     *     return (element >= 10);
     * }
     *
     * var passed = [12, 5, 8, 130, 44].every(isBigEnough);
     * // passed == false
     * passed = [12, 54, 18, 130, 44].every(isBigEnough);
     * // passed == true
     */
    every: function(callback, thisObject) {
        thisObject = thisObject || window;

        var index = 0, length = this.length;

        for (; index < length; index++) {
            if (!callback.apply(thisObject, [this[index], index, this])) {
                break;
            }
        }

        return (index == length);
    },

    /**
     * Фильтрация элементов массива.
     * Возвращает новый массив, который состоит из элементов, прошедших функцию проверки, т.е. удовлетворяющих определенным условиям.
     * Исходный массив не изменяется.
     * @name filter
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Function} callback Функция для проверки элемента массива (должна возвращать true или false)
     * @param {Object} [thisObject] Контекст выполнения функции проверки
     * @returns {Array} Массив
     *
     * @example
     * function isBigEnough(element, index, array) {
     *     return (element >= 10);
     * }
     * var filtered = [12, 5, 8, 130, 44].filter(isBigEnough);
     * // filtered == [12, 130, 44]
     */
    filter: function(callback, thisObject) {
        thisObject = thisObject || window;

        var length = this.length, count = 0, filtered = new Array(length);

        for (var index = 0; index < length; index++) {
            if (callback.apply(thisObject, [this[index], index, this])) {
                filtered[count++] = this[index];
            }
        }

        filtered.length = count;

        return filtered;
    },

    /**
     * Обработка каждого элемента массива заданной функцией.
     * @name forEach
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Function} callback Функция, выполняемая для каждого элемента
     * @param {Object} [thisObject] Контекст выполнения функции
     *
     * @example
     * function printElt(element, index, array) {
     *     print("[" + index + "] == " + element);
     * }
     * [2, 5, 9].forEach(printElt);
     * // Печатает:
     * // [0] == 2
     * // [1] == 5
     * // [2] == 9
     */
    forEach: function(callback, thisObject) {
        thisObject = thisObject || window;

        for (var index = 0, length = this.length; index < length; index++) {
            callback.apply(thisObject, [this[index], index, this]);
        }
    },

    /**
     * Обработка каждого элемента массива заданной функцией.
     * Создает новый массив с результатами выполнения функции для каждого элемента массива.
     * Исходный массив не изменяется.
     * @name map
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Function} callback Функция, выполняемая для каждого элемента
     * @param {Object} [thisObject] Контекст выполнения функции
     * @returns {Array} Массив элементов
     *
     * @example
     * var numbers = [1, 4, 9];
     * var roots = numbers.map(Math.sqrt);
     * // roots == [1, 2, 3]
     * // numbers == [1, 4, 9]
     */
    map: function(callback, thisObject) {
        thisObject = thisObject || window;

        var index = 0, length = this.length, map = new Array(length);

        for (; index < length; index++) {
            map[index] = callback.apply(thisObject, [this[index], index, this]);
        }

        return map;
    },

    /**
     * Тестирование каждого элемента массива при помощи функции проверки. Результатом тестирования массива является true, если хотя бы один элемент прошел успешную проверку. Иначе результатом тестирования массива будет false.
     * @name some
     * @function
     * @memberOf Array
     * @jsver 1.6
     * @param {Function} callback Функция, выполняемая для каждого элемента
     * @param {Object} [thisObject] Контекст выполнения функции
     *
     * @example
     * function isBigEnough(element, index, array) {
     *     return (element >= 10);
     * }
     * var passed = [2, 5, 8, 1, 4].some(isBigEnough);
     * // passed == false
     * passed = [12, 5, 8, 1, 4].some(isBigEnough);
     * // passed == true
     */
    some: function(callback, thisObject) {
        thisObject = thisObject || window;

        var index = 0, length = this.length;
        for (; index < length; index++) {
            if (callback.apply(thisObject, [this[index], index, this])) {
                break;
            }
        }

        return (index != length);
    },

    /**
     * Обработка слева направо соседних элементов массива заданной функцией. Функция применяется одновременно к двум элементам массива и превращает их в одно значение.
     * Функция изменяет исходный массив.
     * @name reduce
     * @function
     * @memberOf Array
     * @jsver 1.8
     * @param {Function} callback Функция, выполняемая для каждого элемента
     * @param {Object} [initial] Начальное значение
     *
     * @example
     * [0, 1, 2, 3, 4].reduce(function(previousValue, currentValue, index, array) {
     *     return previousValue + currentValue;
     * });
     * // 10
     */
    reduce: function(callback /*, initial*/) {
        var len = this.length;
        if (typeof callback != 'function') {
            throw new TypeError();
        }

        // no value to return if no initial value and an empty array
        if (len == 0 && arguments.length == 1) {
            throw new TypeError();
        }

        var i = 0, rv;
        if (arguments.length >= 2) {
            rv = arguments[1];
        } else {
            do {
                // if (i in this) { // IE 5 compatibility
                if (typeof this[i] != 'undefined') {
                    rv = this[i++];
                    break;
                }

                // if array contains no values, no initial value to return
                if (++i >= len) {
                    throw new TypeError();
                }
            } while (true);
        }

        for (; i < len; i++) {
            // if (i in this) { // IE 5 compatibility
            if (typeof this[i] != 'undefined') {
                rv = callback.call(null, rv, this[i], i, this);
            }
        }

        return rv;
    },

    /**
     * Обработка cправа налево соседних элементов массива заданной функцией. Функция применяется одновременно к двум элементам массива и превращает их в одно значение.
     * Функция изменяет исходный массив.
     * @name reduceRight
     * @function
     * @memberOf Array
     * @jsver 1.8
     * @param {Function} callback Функция, выполняемая для каждого элемента
     * @param {Object} [initial] Начальное значение
     *
     * @example
     * [0, 1, 2, 3, 4].reduceRight(function(previousValue, currentValue, index, array) {
     *     return previousValue + currentValue;
     * });
     * // 10
     */
    reduceRight: function(callback /*, initial*/) {
        var len = this.length;
        if (typeof callback != 'function') {
            throw new TypeError();
        }

        // no value to return if no initial value and an empty array
        if (len == 0 && arguments.length == 1) {
            throw new TypeError();
        }

        var i = len - 1, rv;
        if (arguments.length >= 2) {
            rv = arguments[1];
        } else {
            do {
                // if (i in this) { // IE 5 compatibility
                if (typeof this[i] != 'undefined') {
                    rv = this[i--];
                    break;
                }

                // if array contains no values, no initial value to return
                if (--i >= len) {
                    throw new TypeError();
                }
            } while (true);
        }

        for (; i >= 0; i--) {
            // if (i in this) { // IE 5 compatibility
            if (typeof this[i] != 'undefined') {
                rv = callback.call(null, rv, this[i], i, this);
            }
        }

        return rv;
    }
};

for (var name in ArrayFuncs) {
    if (!A[name]) {
        A[name] = ArrayFuncs[name];
    }
}

var StringPrototype = String.prototype,
    Space = '';

/// Исправляем поведение indexOf в IE 5
if (Space.indexOf(Space) != 0) {

    StringPrototype.indexOfBug = StringPrototype.indexOf;

    StringPrototype.indexOf = function(needle) {
        if (this.toString() == Space && needle === Space) {
            return 0;
        }

        return this.indexOfBug(needle);
    };

    StringPrototype.lastIndexOfBug = StringPrototype.lastIndexOf;

    StringPrototype.lastIndexOf = function(needle) {
        var pos = this.lastIndexOfBug(needle);
        if (needle === Space) {
            pos++;
        }

        return pos;
    };

}

/// Исправление String.replace (Safari1.x/IE5.0).
// IE7/IE8.js - copyright 2004-2008, Dean Edwards
if (Space.replace(/^/, String)) {
    var GLOBAL = /(g|gi)$/,
        _replace = StringPrototype.replace;

    StringPrototype.replace = function(expression, replacement) {
        if (typeof replacement == 'function') { // Safari doesn't like functions
            var regexp, global, match, string = this, result = Space;

            if (expression && expression.constructor == RegExp) {
                regexp = expression;
                global = regexp.global;
                if (global == null) {
                    global = GLOBAL.test(regexp);
                }
                // we have to convert global RexpExps for exec() to work consistently
                if (global) {
                    regexp = new RegExp(regexp.source); // non-global
                }
            } else {
                regexp = new RegExp(rescape(expression));
            }

            while (string && (match = regexp.exec(string))) {
                result += string.slice(0, match.index) + replacement.apply(this, match);
                string = string.slice(match.index + match[0].length);
                if (!global) {
                    break;
                }
            }
            return result + string;
        }
        return _replace.apply(this, arguments);
    };
}

var fromCharCode = String.fromCharCode;

if (!StringPrototype.charCodeAt) {

    StringPrototype.charCodeAt = function(index) {
        var k = 0,
            enc = escape(this).match(/(%[\da-fA-F]{2}|%u[\da-fA-F]{4}|.)/g);

        for (var i = 0, l = enc.length; i < l; i++) {
            var symb = enc[i];
            if (!symb) {
                continue;
            }

            if (symb.indexOf('%u') == 0) {
                symb = parseInt(symb.replace('%u', Space), 16);
            } else if (symb.indexOf('%') == 0) {
                symb = parseInt(symb.replace('%', Space), 16);
            } else {
                var begin = 0, end = 256, cur = 0, curChar = Space;

                while (end - begin > 1) {
                    cur = (end + begin) >> 1; // div 2
                    curChar = fromCharCode(cur);
                    if (curChar > symb) {
                        end = cur;
                    } else if (curChar < symb) {
                        begin = cur;
                    } else {
                        symb = cur;
                        break;
                    }
                }
            }

            if (k == index) {
                return symb;
            }
            k++;
        }

        return NaN;
    };
}

// ie 5.0
if (!window.encodeURIComponent) {

    var hexDigit = '0123456789ABCDEF'.split('');

    function dec2hex (n) {
        return hexDigit[n >> 4] + hexDigit[n & 15];
    }

    function utf8_and_URI_encode (string, encodeURI) {
        string = string.toString().replace(/\r\n/g, "\n");

        var text = '',
            c,
            n = 0,
            strlen = string.length,
            // символы, которые не преобразуются encodeURIComponent
            nonEscapeCharacters = "!'()*-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz~";

        if (encodeURI) {
            // encodeURI не преобразует еще 12 символов
            nonEscapeCharacters += '#$&+,-/:;=?@';
        }

        for (; n < strlen; n++) {
            c = string.charCodeAt(n);
            if (c < 128 ) {
                if (nonEscapeCharacters.indexOf(string.charAt(n)) != -1) {
                    text += fromCharCode(c);
                } else {
                    text += '%'+dec2hex(c);
                }

            // utf-8 encode
            } else if ((c > 127) && (c < 2048)) {
                text += escape(fromCharCode((c >> 6) | 192) + fromCharCode((c & 63) | 128));

            } else {
                text += escape(fromCharCode((c >> 12) | 224) + fromCharCode(((c >> 6) & 63) | 128) + fromCharCode((c & 63) | 128));
            }
        }

        return text;
    };

    window.encodeURIComponent = utf8_and_URI_encode;

    window.encodeURI = function (string) {
        return utf8_and_URI_encode(string, true);
    }
}

if (!window.decodeURIComponent) {
    var mask21 = 6,  // parseInt('110', 2)
        mask22 = 2,  // parseInt('10', 2)
        mask31 = 14, // parseInt('1110', 2)
        throwMsg = 'malformed URI sequence';

    function utf8_and_URI_decode (text) {
        text = unescape(text.toString());
        var string = '',
            strlen = text.length,
            i = 0,
            s,
            c = 0,
            c1 = 0,
            c2 = 0;

        while (i < strlen) {
            c = text.charCodeAt(i);
            if (c < 128) {
                s = fromCharCode(c);
                i++;
            } else if ((c > 191) && (c < 224)) {
                c1 = text.charCodeAt(i + 1);
                if (!((c >> 5) == mask21 && (c1 >> 6) == mask22)) {
                    throw throwMsg;
                }
                s = fromCharCode(((c & 31) << 6) | (c1 & 63));
                i += 2;
            } else {
                c1 = text.charCodeAt(i + 1);
                c2 = text.charCodeAt(i + 2);
                if (!((c >> 4) == mask31 && (c1 >> 6) == mask22 && (c2 >> 6) == mask22)) {
                    throw throwMsg;
                }
                s = fromCharCode(((c & 15) << 12) | ((c1 & 63) << 6) | (c2 & 63));
                i += 3;
            }
            string += s;
        }

        return string;
    }

    window.decodeURIComponent = window.decodeURI = utf8_and_URI_decode;
}


/// Node
/////////////////////////////////////////////////////////////////////

if (!window.Node) {
    window.Node = {};
}

if (!Node.ELEMENT_NODE) {

// DOM level 2 ECMAScript Language Binding
/*
    var types = {
        ELEMENT_NODE: 1,
        ATTRIBUTE_NODE: 2,
        TEXT_NODE: 3,
        CDATA_SECTION_NODE: 4,
        ENTITY_REFERENCE_NODE: 5,
        ENTITY_NODE: 6,
        PROCESSING_INSTRUCTION_NODE: 7,
        COMMENT_NODE: 8,
        DOCUMENT_NODE: 9,
        DOCUMENT_TYPE_NODE: 10,
        DOCUMENT_FRAGMENT_NODE: 11,
        NOTATION_NODE: 12
    };

    for (var type in types) {
        Node[type] = types[type];
    }

    delete types;
*/

    (
     'ELEMENT,ATTRIBUTE,TEXT,CDATA_SECTION,ENTITY_REFERENCE,ENTITY,' +
     'PROCESSING_INSTRUCTION,COMMENT,DOCUMENT,DOCUMENT_TYPE,DOCUMENT_FRAGMENT,NOTATION'
    ).split(',').forEach(function(type, num) { Node[type + '_NODE'] = num + 1 });

}

})();/**
 * Ядро фреймворка.
 * Содержит определения часто используемых переменных (версия браузера, тип операционной системы и т.п.) и набор функций для загрузки модулей. 
 * 
 * @class
 * @static
 * @name y5
 *
 * @example
 * Для загрузки модулей библиотек можно использовать удобный механизм динамической загрузки.
 * Для этого используется метод y5.require.
 *
 * y5.require('Dom', function() {
 *     // здесь можно использовать функции y5.Dom
 * });
 *
 * y5.require('URL', 'Classes', function() {
 *     // здесь можно использовать функции y5.URL и y5.Classes
 * });
 */
var y5 = (function() {

    var
        // Список загруженных модулей
        // y5:Arrays, y5:Dom, ...
        Modules = {},
        
        /**
         * Массив заблокированных модулей
         * @name BlockModules
         * @memberOf y5
         * @type Array
         * @private
         */
        BlockModules = [],

        // Список требуемых модулей
        ModulesRequired = {},

        // Правило для вычисления названия библиотеки и имени модуля
        // y5:Arrays или {y5}.Arrays (устарело)
        ModuleRegexp = /^(\{([^\}]+)\}\.|([^:]+):)?(.+)$/,

        // Список библиотек
        // y5, Friends, ...
        // Формат: {y5:
        //   {
        //     path: [путь к библиотеке],
        //     charset: [кодировка файлов модулей],
        //     query: [параметры URL, добавляемые к запросу модуля]
        //   }
        // }
        Namespaces = {},

        // Имя библиотеки по умолчанию
        NamespaceDefault = 'y5',

        // Очередь модулей
        // [y5:Arrays, y5:Dom, ...]
        Query = [],

        // Список скриптов страницы
        Scripts = document.getElementsByTagName('script'),

        moduleLoadedNotifier,
        moduleRequiredNotifier,
        namespaceAddedNotifier;

    function runCallback(callback) {
        if (typeof callback == 'function') {
            callback();
        }
    }

    function isModuleLoaded(module) {
        return !!Modules[module];
    }

    function isModuleNotLoaded(module) {
        return !Modules[module];
    }

    function isModulesLoaded(modules) {
        return modules.every(isModuleLoaded);
    }
    
    function isModuleBlocked (module) {
        for (var i=0,j=BlockModules.length; i<j; i++) {
            if (module.indexOf(BlockModules[i]) == 0) {
                return true;
            }
        }
        return false;
    }

    function addModuleLoaded(module) {
        Modules[module] = 1;

        if (!moduleLoadedNotifier) {
            moduleLoadedNotifier = y5.Events.notify('y5:moduleLoaded', y5, false);
        }
        moduleLoadedNotifier.dispatch(module);
    }

    function isModuleRequired(module) {
        return !!ModulesRequired[module];
    }

    function addModuleRequired(module) {
        ModulesRequired[module] = 1;

        if (!moduleRequiredNotifier) {
            moduleRequiredNotifier = y5.Events.notify('y5:moduleRequired', y5, false);
        }
        moduleRequiredNotifier.dispatch(module);
    }

    function removeModuleRequired(module) {
        delete ModulesRequired[module];
    }

    function addNamespace(namespace, data) {
        Namespaces[namespace] = data;
        if (!namespaceAddedNotifier) {
            namespaceAddedNotifier = y5.Events.notify('y5:namespaceAdded', y5, false);
        }
        namespaceAddedNotifier.dispatch(data);
        callQuery();
    }

    function getNamespaceData(namespace) {
        return Namespaces[namespace];
    }

    function getModuleData(module) {
        var data = module.match(ModuleRegexp);
        // TODO: exception
        return [
            data[3] || data[2] || NamespaceDefault,
            data[4]
        ];
    }

    function getModuleNameCanonical(module) {
        var data = getModuleData(module);
        return data[0] + ':' + data[1];
    }

    function getModuleURL(module, type) {
        var data = getModuleData(module);
        var namespace = getNamespaceData(data[0]);

        if (!namespace) {
            return false;
        }

        return [
            (
                namespace.path +
                data[1].replace(/\./g, '/') +
                '.' + (type || 'js') +
                namespace.query
            ),
            namespace.charset
        ];
    }

    /**
     * Загружает модули по необходимости.
     * @private
     */
    function loadModules(modules) {
        var i = 0,
            l = modules.length,
            module;
        for (; i < l; i++) {
            module = modules[i];
            if (isModuleLoaded(module) || isModuleRequired(module) || isModuleBlocked(module)) {
                continue;
            }

            var data = getModuleURL(module);
            if (data) {
                addModuleRequired(module);
                y5.Loader.loadScript(data[0], data[1]);
            }
        }
    }

    /**
     * Помещает в очередь список требуемых модулей и callback-функцию.
     * @private
     */
    function pushQuery(modules, callback) {
        modules = modules.filter(isModuleNotLoaded);

        if (modules.length == 0) {
            runCallback(callback);
            return false;
        }

        Query.push({
            modules: modules,
            callback: callback
        });

        return modules;
    }

    /**
     * Запускает callback-функции из очереди для загруженных модулей.
     * @private
     */
    function callQuery() {
        for (var i = 0; i < Query.length; i++) {
            var item = Query[i];
            if (isModulesLoaded(item.modules)) {
                Query.splice(i, 1);
                runCallback(item.callback);
                i--;
            } else {
                loadModules(item.modules);
            }
        }
    }

    /**
     * Возвращает данные скрипта по имени файла.
     * @param {String} file Имя файла
     * @param {String} [charset] Кодировка файла (если указано, то переопределяет кодировку указанную в скрипте)
     * @returns {Object} Объект вида {path: 'http://www.yandex.ru/foo/', query: '?build=1', charset: 'utf-8'}
     * @private
     */
    function getData(file, charset) {
        for (var i = 0, l = Scripts.length; i < l; i++) {
            var script = Scripts[i],
                src = script.getAttribute('src');

            if (src && src.lastIndexOf(file) >= 0) {
                var base = {
                    path: src.substring(0, src.lastIndexOf('/') + 1),
                    charset: charset || script.getAttribute('charset') || 'utf-8'
                };

                var query = src.lastIndexOf('?');
                if (query >= 0) {
                    base.query = src.substring(query, src.length);
                }

                return base;
            }
        }
        return null;
    }

    function registerNamespaceByData(namespace, data, charset) {
        if (typeof data == 'string') {
            data = {path: data, charset: charset};
        }

        // дополняем путь слэшем, если забыт
        if (data.path.lastIndexOf('/') != data.path.length - 1) {
            data.path += '/';
        }

        // устанавливаем кодировку по умолчанию
        if (!data.charset) {
            data.charset = 'utf-8';
        }

        // устанавливаем параметры запроса по умолчанию
        if (!data.query) {
            data.query = '';
        }

        addNamespace(namespace, data);
    }

    function getNamespaceItem(namespace, name) {
        try {
            return Namespaces[namespace][name];
        } catch (e) {
            return null;
        }
    }

    var obj = {
        /**
         * Текущая версия фреймворка.
         * @name y5.version
         * @memberOf y5
         * @type Number
         */
        version: 1.5,

        /**
         * Логическая переменная. Устанавливается в true, когда происходит событие dom:loaded.
         * @name y5.domloaded
         * @memberOf y5
         * @type Boolean
         */
        domloaded: false,

        /**
         * Загружает требуемые модули и выполняет callback-функцию после загрузки модулей.
         * @name y5.require
         * @memberOf y5
         * @function
         * @param {String | Array} modules "foo.bar"
         * @param {Function} callback Функция, выполняемая в момент загрузки
         */
        require: function(/* module1, module2, ..., moduleN [ , callback ] */) {
            var args = arguments,
                modules = [],
                callback = y5.VOID,
                Types = y5.Types;

            for (var i = 0, l = args.length; i < l; i++) {
                var arg = args[i];
                switch (Types.type(arg)) {
                    case Types.ARRAY:
                        modules = modules.concat(arg);
                        break;

                    case Types.STRING:
                        modules.push(arg);
                        break;

                    case Types.FUNCTION:
                        callback = arg;
                        break;
                }
            }

            modules = modules.map(getModuleNameCanonical);
            modules = pushQuery(modules, callback);

            if (modules) {
                loadModules(modules);
            }
        },

        /**
         * Функция вызывается модулем после окончания загрузки всех зависимых модулей.
         * @name y5.loaded
         * @memberOf y5
         * @function
         * @param {String} module Имя модуля (например, "foo.bar")
         */
        loaded: function(module) {
            module = getModuleNameCanonical(module);
            removeModuleRequired(module);
            addModuleLoaded(module);
            callQuery();
        },

        /**
         * Регистрирует новую библиотеку модулей по имени файла.
         * @name y5.registerNamespace
         * @memberOf y5
         * @function
         * @param {String} namespace Имя, под которым регистрируется библиотека
         * @param {String} filename Строка, которая входит в имя файла библиотеки
         * @param {String} [charset] Кодировка файлов библиотеки
         */
        registerNamespace: function(namespace, file, charset) {
            var timer = null,
                counter = 0;

            function get() {
                if (counter < 1000) {
                    var data = getData(file, charset);
                    if (data) {
                        window.clearTimeout(timer);
                        registerNamespaceByData(namespace, data);
                        return true;
                    }
                    counter++;
                }
                return false;
            }

            if (!get()) {
                timer = window.setInterval(get, 1);
            }
        },

        /**
         * Регистрирует новую библиотеку модулей по URL или объекту с данными о библиотеке.
         * @name y5.registerNamespaceByData
         * @memberOf y5
         * @function
         * @param {String} namespace Имя, под которым регистрируется библиотека
         * @param {Object | String} data Данные вида: {path: 'http://www.yandex.ru/foo/', query: '?build=1', charset: 'utf-8'}
         * или путь к библиотеке.
         * @param {String} [charset] Кодировка файлов библиотеки
         */
        registerNamespaceByData: registerNamespaceByData,

        /**
         * Возвращает путь к файлам библиотеки.
         * @name y5.namespacePath
         * @memberOf y5
         * @function
         * @param {String} namespace Имя библиотеки
         * @returns {String | null} Путь
         *
         * @example
         * y5.namespacePath("Foo");
         * // -> "http://www.yandex.ru/foo/"
         */
        namespacePath: function(namespace) {
            return getNamespaceItem(namespace, 'path');
        },

        /**
         * Возвращает кодировку файлов библиотеки.
         * @name y5.namespaceCharset
         * @memberOf y5
         * @function
         * @param {String} namespace Имя библиотеки
         * @returns {String | null} Кодировка
         *
         * @example
         * y5.namespaceCharset("Foo");
         * // -> "utf-8"
         */
        namespaceCharset: function(namespace) {
            return getNamespaceItem(namespace, 'charset');
        },

        /**
         * Возвращает URL модуля.
         * @name y5.moduleURL
         * @memberOf y5
         * @function
         * @param {String} module Имя модуля (например, "foo.bar")
         * @param {String} ext Расширение имени файла модуля (по умолчанию - "js")
         * @returns {String} URL модуля
         *
         * @example
         * y5.moduleURL("Dom");
         * // -> [base URL]/Dom.js
         *
         * y5.moduleURL("Widget.Window", "css");
         * // -> [base URL]/Widget/Window.css
         */
        moduleURL: function(module, ext) {
            return getModuleURL(module, ext)[0];
        },

        /**
         * Возвращает каноническое (полное) имя модуля.
         * @name y5.moduleName
         * @memberOf y5
         * @function
         * @param {String} module Имя модуля (например, "foo.bar")
         * @returns {String} Имя модуля
         *
         * @example
         * y5.moduleName("foo.bar");
         * // -> "y5:foo.bar"
         */
        moduleName: getModuleNameCanonical,

        /**
         * Возвращает название библиотеки модуля.
         * @name y5.moduleNamespace
         * @memberOf y5
         * @function
         * @param {String} module Имя модуля (например, "foo.bar")
         * @returns {String} Namespace модуля
         *
         * @example
         * y5.moduleNamespace("foo.bar");
         * // -> "y5"
         */
        moduleNamespace: function(module) {
            return getModuleData(module)[0];
        },

        /**
         * Возвращает объект по имени модуля.
         * @name y5.moduleObject
         * @memberOf y5
         * @function
         * @param {String} module Имя модуля (например, "foo.bar")
         * @returns {Object | null} Объект
         *
         * @example
         * y5.moduleNamespace("foo.bar");
         * // -> y5.foo.bar
         */
        moduleObject: function(module) {
            var obj = window;
            var classes = getModuleNameCanonical(module).split(/[:\.]/g);

            for (var i = 0, l = classes.length; i < l; i++) {
                obj = obj[classes[i]];
                if (typeof obj == y5.UNDEF) {
                    return null;
                }
            }

            return obj;
        },
        
        /**
         * Блокирует загрузку дочерних модулей для указанного модуля или библиотеки.
         * @name y5.blockLoad
         * @memberOf y5
         * @function 
         * @param {String} module Имя модуля (например, "foo.bar") или библиотеки
         * @param {Boolean} withModule Блокировать также загрузку указанного модуля
         * @returns  {Boolean}
         */
        blockLoad: function (module, withModule) {
            // не неймспейс
            if (module.indexOf(':') != (module.length-1)) {
                module = getModuleNameCanonical(module);
                // если блокируем только дочерние модули, то добавляем в конец имени точку
                if (!withModule) {
                    module += '.';
                }
            }
            
            // если модуля нет в списке 
            if (module && BlockModules.indexOf(module) == -1) {
                BlockModules.push(module);
                return true;
            }
            return false;
        },
        
        /**
         * Разблокирует загрузку дочерних модулей для указанного модуля или библиотеки.
         * @name y5.unblockLoad
         * @memberOf y5
         * @function 
         * @param {String} module Имя модуля (например, "foo.bar") или библиотеки
         * @returns {Boolean}
         */
        unblockLoad: function (module) {
            // не неймспейс
            if (module.indexOf(':') != (module.length-1)) {
                module = getModuleNameCanonical(module);
            }
            
            if (module) {
                var index = BlockModules.indexOf(module);
                if (index == -1) {
                    index = BlockModules.indexOf(module + '.');
                }
                if (index != -1) {
                    BlockModules.splice(index, 1);
                    return true;
                }
            }
            return false;
        }
    };

    /**
     * Возвращает данные скрипта по имени файла.
     * @param {String} file Имя файла
     * @param {String} [charset] Кодировка файла (если указана, то переопределяет кодировку, указанную в скрипте)
     * @returns {Object} Объект вида {path: 'http://www.yandex.ru/foo/', query: '?build=1', charset: 'utf-8'}
     * @name getBase
     * @memberOf y5
     * @function
     * @deprecated
     */
    obj.getBase = getData;
    obj.getBaseAndSetAlias = obj.registerNamespace;
    obj.setAlias = registerNamespaceByData;
    obj.constructURL = obj.moduleURL;
    obj.getAlias = obj.moduleNamespace;
    obj.charsets = {};
    obj.setAliasCharset = function() {};

    return obj;

})();

/// Vars
/////////////////////////////////////////////////////////////////////

/**
 * @class Содержит глобальные переменные.
 * @static
 * @name y5.Vars
 */
y5.Vars = {
    DEBUG: false,

    /**
     * Строка 'undefined'.
     * @name y5.Vars.UNDEF
     * @memberOf y5.Vars
     * @constant
     * @type String
     */

    /**
     * @name y5.UNDEF
     * @link y5.Vars.UNDEF
     * @memberOf y5
     * @type String
     */
    UNDEF: 'undefined',

    /**
     * Всегда возвращает значение false.
     * @name y5.Vars.FALSE
     * @memberOf y5.Vars
     * @function
     * @type Function
     * @returns {Boolean} false
     */

    /**
     * @name y5.FALSE
     * @link y5.Vars.FALSE
     * @memberOf y5
     * @type Function
     */
    FALSE: function() { return false; },

    /**
     * Всегда возвращает значение true.
     * @name y5.Vars.TRUE
     * @memberOf y5.Vars
     * @function
     * @type Function
     * @returns {Boolean} true
     */

    /**
     * @name y5.TRUE
     * @link y5.Vars.TRUE
     * @memberOf y5
     * @type Function
     */
    TRUE: function() { return true; },

    /**
     * Всегда возвращает значение null.
     * @name y5.Vars.NULL
     * @memberOf y5.Vars
     * @function
     * @type Function
     * @returns {Object} null
     */

    /**
     * @name y5.NULL
     * @link y5.Vars.NULL
     * @memberOf y5
     * @type Function
     */
    NULL: function() { return null; },

    /**
     * Всегда возвращает значение undefined.
     * @name y5.Vars.VOID
     * @memberOf y5.Vars
     * @function
     * @type Function
     * @returns {undefined}
     */

    /**
     * @name y5.VOID
     * @link y5.Vars.VOID
     * @memberOf y5
     * @type Function
     */
    VOID: function() { }

};

/// Browser
/////////////////////////////////////////////////////////////////////

y5.Browser = {
    get: function(nav) {
        var agt = nav.userAgent.toLowerCase(),
            result = {};

        function getVer(is_this, regex) {
            if (is_this) {
                var f = agt.match(regex);
                return f ? parseFloat(f[1]) : 0;
            }
            return 0;
        }

        function contains(needle) {
            return agt.indexOf(needle) != -1;
        }

        /**
         * Определяет операционную систему пользователя. Возвращает true, если операционная система пользователя - MS Windows.
         * @name y5.Vars.is_win
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_win
         * @link y5.Vars.is_win
         * @memberOf y5
         * @type Boolean
         */
        result.is_win       = contains('windows');

        /**
         * Определяет операционную систему пользователя. Возвращает true, если операционная система пользователя - Mac OS.
         * @name y5.Vars.is_mac
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_mac
         * @link y5.Vars.is_mac
         * @memberOf y5
         * @type Boolean
         */
        result.is_mac       = contains('mac');
        
        /**
         * Определяет операционную систему пользователя. Возвращает true, если операционная система пользователя - GNU/Linux.
         * @name y5.Vars.is_linux
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_linux
         * @link y5.Vars.is_linux
         * @memberOf y5
         * @type Boolean
         */
        result.is_linux       = contains('linux');

        /**
         * Определяет тип браузера. Возвращает true для браузера Safari.
         * @name y5.Vars.is_safari
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_safari
         * @link y5.Vars.is_safari
         * @memberOf y5
         * @type Boolean
         */
        result.is_safari    = contains('safari');

        /**
         * Определяет тип браузера. Возвращает true для браузера Safari на iPhone и iPod Touch.
         * @name y5.Vars.is_iphone
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_iphone
         * @link y5.Vars.is_iphone
         * @memberOf y5
         * @type Boolean
         */
        result.is_iphone    = result.is_safari && contains('iphone');

        /**
         * Определяет тип браузера. Возвращает true для браузера Opera.
         * @name y5.Vars.is_opera
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_opera
         * @link y5.Vars.is_opera
         * @memberOf y5
         * @type Boolean
         */
        result.is_opera     = contains('opera');

        /**
         * Определяет тип браузера. Возвращает true для браузера Konqueror.
         * @name y5.Vars.is_konq
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_konq
         * @link y5.Vars.is_konq
         * @memberOf y5
         * @type Boolean
         */
        result.is_konq      = contains('konqueror');

        /**
         * Определяет тип браузера. Возвращает true для браузеров семейства MS Internet Explorer.
         * @name y5.Vars.is_ie
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie
         * @link y5.Vars.is_ie
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie        = !result.is_opera && contains('msie');

        /**
         * Определяет тип движка браузера. Возвращает true для браузеров, основанных на KHTML.
         * @name y5.Vars.is_khtml
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_khtml
         * @link y5.Vars.is_khtml
         * @memberOf y5
         * @type Boolean
         */
        result.is_khtml     = !result.is_safari && contains('khtml');

        /**
         * Определяет тип движка браузера. Возвращает true, если в браузере используется движок Gecko.
         * @name y5.Vars.is_gecko
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_gecko
         * @link y5.Vars.is_gecko
         * @memberOf y5
         * @type Boolean
         */
        result.is_gecko     = contains('gecko/');

        /**
         * Определяет версию браузера семейства MS Internet Explorer. Возвращает номер версии.
         * @name y5.Vars.ie_ver
         * @memberOf y5.Vars
         * @constant
         * @type Number
         */

        /**
         * @name y5.ie_ver
         * @link y5.Vars.ie_ver
         * @memberOf y5
         * @type Number
         */
        result.ie_ver       = getVer(result.is_ie, /msie (\d+\.\d)/);

        /**
         * Определяет версию движка рендеринга страниц для браузеров, основанных на коде проекта Gecko. Возвращает номер версии.
         * @name y5.Vars.gecko_ver
         * @memberOf y5.Vars
         * @constant
         * @type Number
         */

        /**
         * @name y5.gecko_ver
         * @link y5.Vars.gecko_ver
         * @memberOf y5
         * @type Number
         */
        result.gecko_ver    = getVer(result.is_gecko, /rv:(\d+\.\d)/);

        /**
         * Определяет версию браузера Opera. Возвращает номер версии.
         * @name y5.Vars.opera_ver
         * @memberOf y5.Vars
         * @constant
         * @type Number
         */

        /**
         * @name y5.opera_ver
         * @link y5.Vars.opera_ver
         * @memberOf y5
         * @type Number
         */
        result.opera_ver    = getVer(result.is_opera, /opera[\/ ](\d+\.\d)/);

        /**
         * Определяет версию браузера Safari. Возвращает номер версии.
         * @name y5.Vars.safari_ver
         * @memberOf y5.Vars
         * @constant
         * @type Number
         */

        /**
         * @name y5.safari_ver
         * @link y5.Vars.safari_ver
         * @memberOf y5
         * @type Number
         */
        result.safari_ver   = getVer(result.is_safari, /safari\/(\d+)/);

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 5.0.
         * @name y5.Vars.is_ie5
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie5
         * @link y5.Vars.is_ie5
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie5       = result.ie_ver == 5.0;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 5.5.
         * @name y5.Vars.is_ie55
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie55
         * @link y5.Vars.is_ie55
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie55      = result.ie_ver == 5.5;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 5.0 и последующих версий.
         * @name y5.Vars.is_ie5up
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie5up
         * @link y5.Vars.is_ie5up
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie5up     = result.ie_ver > 4.9;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 5.5 и последующих версий.
         * @name y5.Vars.is_ie55up
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie55up
         * @link y5.Vars.is_ie55up
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie55up    = result.ie_ver > 5.4;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 6.0 и последующих версий.
         * @name y5.Vars.is_ie6up
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie6up
         * @link y5.Vars.is_ie6up
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie6up     = result.ie_ver > 5.9;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 7.0 и последующих версий.
         * @name y5.Vars.is_ie7up
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie7up
         * @link y5.Vars.is_ie7up
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie7up     = result.ie_ver > 6.9;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии ниже 6.0 (не включая 6.0).
         * @name y5.Vars.is_ie6down
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie6down
         * @link y5.Vars.is_ie6down
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie6down   = result.is_ie && result.ie_ver < 6.0;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии ниже 7.0 (не включая 7.0).
         * @name y5.Vars.is_ie7down
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie7down
         * @link y5.Vars.is_ie7down
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie7down   = result.is_ie && result.ie_ver < 7.0;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии ниже 8.0 (не включая 8.0).
         * @name y5.Vars.is_ie8down
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie7down
         * @link y5.Vars.is_ie8down
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie8down = result.is_ie && result.ie_ver < 8.0;

        /**
         * Определяет версию браузера. Возвращает true для браузеров MS Internet Explorer версии 8.
         * @name y5.Vars.is_ie8
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.is_ie55
         * @link y5.Vars.is_ie8
         * @memberOf y5
         * @type Boolean
         */
        result.is_ie8 = result.is_ie && result.ie_ver == 8;


        /**
         * Определяет доступность в браузере cookie. Устанавливается в true, если доступ к cookie возможен,  или в false, если cookie запрещены.
         * @name y5.Vars.cookieEnabled
         * @memberOf y5.Vars
         * @constant
         * @type Boolean
         */

        /**
         * @name y5.cookieEnabled
         * @link y5.Vars.cookieEnabled
         * @memberOf y5
         * @type Boolean
         */
        result.cookieEnabled = nav.cookieEnabled;

        return result;
    }
};

/// Копируем переменные браузера в Vars, затем в y5
(function () {
    var i;

    // Определение браузера
    var browser = y5.Browser.get(window.navigator);
    for (i in browser) {
        y5.Vars[i] = browser[i];
    }

    // Копируем переменные в y5
    for (i in y5.Vars) {
        y5[i] = y5.Vars[i];
    }

})();

/// Loader
/////////////////////////////////////////////////////////////////////

/**
 * @class Набор функций для загрузки дополнительных JavaScript-файлов и CSS-стилей на страницу.
 * @static
 * @name y5.Loader
 */
y5.Loader = (function() {

    function setElementAttributes(element, attributes) {
        for (var type in attributes) {
            var value = attributes[type];
            if (value) {
                element.setAttribute(type, value);
            }
        }
    }

    function insertElement(container, element) {
        container.insertBefore(element, container.firstChild);
    }

    var loadObject;
    if (y5.is_opera && y5.opera_ver < 8) {

        loadObject = function(name, attributes) {
            if (!document.body) {
                return null;
            }

            var span = document.createElement('span');
            span.style.display = 'none';
            span.innerHTML = '<' + name + '><\/' + name + '>';

            var element = span.getElementsByTagName(name).item(0);
            setElementAttributes(element, attributes);
            insertElement(document.body, span);

            return element;
        }

    } else {

        var head = document.getElementsByTagName('head')[0];

        loadObject = function(name, attributes) {
            var element = document.createElement(name);
            setElementAttributes(element, attributes);
            insertElement(head, element);

            return element;
        };

    }

    function createScript(attributes, callback) {
        var tries = 10, timer = null;

        function tryCreateScript() {
            if (!--tries) {
                window.clearTimeout(timer);
                return false;
            }

            var script = loadObject('script', attributes);

            if (script) {
                if (typeof callback == 'function') {
                    callback(script);
                }
                window.clearTimeout(timer);
                return true;
            }

            return false;
        }

        if (!tryCreateScript()) {
            timer = window.setInterval(tryCreateScript, 10);
        }
    }

    return {
        /**
         * Загружает файл, содержащий JavaScript
         * @name y5.Loader.loadScript
         * @memberOf y5.Loader
         * @function
         * @param {String} src Путь к скрипту
         * @param {String} [charset] Кодировка скрипта
         * @param {String} [callback] Функция, выполняемая по окончании загрузки
         * @param {String} [id] ID создаваемого элемента DOM
         */
        loadScript: function(src, charset, callback, id) {
            createScript({src: src, charset: charset, type: 'text/javascript', id: id}, callback);
        },

        /**
         * Загружает объект.
         * @name y5.Loader.loadObject
         * @memberOf y5.Loader
         * @param {String} name Имя элемента
         * @param {String} attributes Список атрибутов
         * @private
         *
         * @example
         * y5.Loader.loadObject('script', {src: 'script.js'});
         * // -> &lt;script src="script.js">&lt;/script>
         */
        loadObject: loadObject
    };

})();

/// Scripts
/////////////////////////////////////////////////////////////////////

/**
 * Создает элемент script.
 * @class
 * @static
 * @deprecated y5.Loader
 */
y5.Scripts = {
    /**
     * Создает элемент script.
     * @memberOf y5.Scripts
     * @deprecated y5.Loader.loadScript
     */
    createScript: function(src, charset, callback) {
        y5.Loader.loadScript(src, charset, callback);
    }
};

/// Console
/////////////////////////////////////////////////////////////////////

(function() {

var VOID = y5.VOID;

y5.Console = {
    log:      VOID,
    info:     VOID,
    warn:     VOID,
    error:    VOID,
    trace:    VOID,
    dir:      VOID,
    dirxml:   VOID,
    group:    VOID,
    groupEnd: VOID
};

})();

/// Exception
/////////////////////////////////////////////////////////////////////

/**
 * Создает экземпляр исключения
 * @class Класс Exception
 * @name y5.Exception
 * @param {String} message Сообщение
 * @param {String} method Название метода
 * @param {String} module Название модуля
 * @example
 * try {
 *     // JavaScript code
 * } catch (e) {
 *     throw new y5.Exception(e, 'myMethod', 'myModule');
 * }
 */
y5.Exception = function(message, method, module) {
    if (!y5.DEBUG) {
        return true;
    }

    var ret = 'y5.' + module + '.' + method + ': ' + message;
    var e = new Error(ret);

    if (e.stack) {
        e.message += "\nStack:\t" + e.stack.replace(/\n/ig, '\n\t');
    }

    return e;
};

y5.Exception.prototype = new Error();
(function() {

var UNDEF = y5.UNDEF;

function checkDomNode(obj, type) {
    return (obj && obj.nodeType && obj.nodeType == type) || false;
}

/**
 * @class Модуль для работы с типами объектов. Содержит определения констант, соответствующих разным типам объектов, и набор функций для выяснения типа объекта.
 * @name y5.Types
 * @static
 */
y5.Types = {
    /// consts

    // standard

    /**
     * Константа, обозначающая тип «неопределенный». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.UNDEF
     * @memberOf y5.Types
     * @constant
     * @link y5.Types.UNDEFINED
     * @type Number
     */
    UNDEF:          1 << 0,

    /**
     * Константа, обозначающая тип «неопределенный». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.UNDEFINED
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    UNDEFINED:      1 << 0,

    /**
     * Константа, обозначающая тип «объект». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.OBJECT
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    OBJECT:         1 << 1,

    /**
     * Константа, обозначающая тип «функция». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.FUNCTION
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    FUNCTION:       1 << 2,

    /**
     * Константа, обозначающая тип «число». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.NUMBER
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    NUMBER:         1 << 3,

    /**
     * Константа, обозначающая тип «строка». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.STRING
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    STRING:         1 << 4,

    /**
     * Константа, обозначающая «логический» тип. Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.BOOLEAN
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    BOOLEAN:        1 << 5,

    // user
    /**
     * Константа, обозначающая тип «дата». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.DATE
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    DATE:           1 << 10,

    /**
     * Константа, обозначающая тип «регулярное выражение». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.REGEXP
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    REGEXP:         1 << 11,

    /**
     * Константа, обозначающая тип «массив». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.ARRAY
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    ARRAY:          1 << 12,

    /**
     * Тип null.
     * @name y5.Types.NULL
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    NULL:           1 << 13,

    /**
     * Константа, обозначающая тип «событие». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.EVENT
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    EVENT:          1 << 14,

    /**
     * Константа, обозначающая тип «узел документа». Возвращает цифровой код, соответствующий данному типу.
     * @name y5.Types.NODE
     * @memberOf y5.Types
     * @constant
     * @type Number
     */
    NODE:           1 << 15,

    TYPES: {
        'undefined': 1 << 0,
        'object':    1 << 1,
        'function':  1 << 2,
        'number':    1 << 3,
        'string':    1 << 4,
        'boolean':   1 << 5
    },

    /// methods

    /**
     * Возвращает числовой код типа объекта – одну из констант y5.Types.
     * @name y5.Types.type
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект
     * @returns {Number} Константа y5.Types
     *
     * @example
     * y5.Types.type(1);
     * // -> y5.Types.NUMBER
     *
     * y5.Types.type(new Date);
     * // -> y5.Types.DATE
     *
     * y5.Types.type([1, 2, 3]);
     * // -> y5.Types.ARRAY
     *
     * y5.Types.type(document);
     * // -> y5.Types.NODE
     */
    type: function(obj) {
        var type = this.TYPES[typeof obj];

        // NULL
        if (obj === null) {
            return this.NULL;
        }

        // DOM NODE
        if (type == this.OBJECT) {
            if (obj.nodeName || this.document(obj) /* IE 5.0 */) {
                return this.NODE;
            }
        }

        // ARRAY, REGEXP, DATE
        if (type == this.OBJECT || type == this.FUNCTION) {
            switch(obj.constructor) {
                case Array:
                    return this.ARRAY;

                case RegExp:
                    return this.REGEXP;

                case Date:
                    return this.DATE;
            }
        }

        // EVENT
        if (this.event(obj)) {
            return this.EVENT;
        }

        return type;
    },

    /**
     * Проверяет, является ли тип объекта одним из типов, заданных параметром mask. В случае успеха проверки возвращает true.
     * @name y5.Types.test
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @param {Number} mask Маска для проверки (задается в виде TYPE1 | TYPE2 | ...)
     * @returns {Boolean} Результат проверки
     *
     * @example
     * // проверить, что тип объекта Date или RegExp
     * y5.Types.test(obj, y5.Types.DATE | y5.Types.REGEXP);
     */
    test: function(obj, mask) {
        return !!(this.type(obj) & mask);
    },

    // standard

    /**
     * Проверяет, является ли объект определенным. В случае успеха проверки возвращает true.
     * @name y5.Types.def
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    def: function(obj) {
        return typeof obj != UNDEF;
    },

    /**
     * Проверяет, является ли объект неопределенным. В случае успеха проверки возвращает true.
     * @name y5.Types.undef
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    undef: function(obj) {
        return typeof obj == UNDEF;
    },

    /**
     * Проверяет, принадлежит ли объект к типу Object. В случае успеха проверки возвращает true.
     * @name y5.Types.object
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    object: function(obj) {
        return typeof obj == 'object';
    },

    /**
     * Проверяет, является ли объект функцией. В случае успеха проверки возвращает true.
     * @name y5.Types.func
     * @memberOf y5.Types
     * @function
     * @param {Object} object Рбъект для проверки
     * @returns {Boolean} Результат проверки
     */
    func: function(obj) {
        return typeof obj == 'function';
    },

    /**
     * Проверяет, является ли объект числом. В случае успеха проверки возвращает true.
     * @name y5.Types.number
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    number: function(obj) {
        return typeof obj == 'number';
    },

    /**
     * Проверяет, является ли объект строкой. В случае успеха проверки возвращает true.
     * @name y5.Types.string
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    string: function(obj) {
        return typeof obj == 'string';
    },

    /**
     * Проверяет, принадлежит ли объект к типу Boolean. В случае успеха проверки возвращает true.
     * @name y5.Types.bool
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    bool: function(obj) {
        return typeof obj == 'boolean';
    },

    /**
     * Проверяет, является ли объект null. В случае успеха проверки возвращает true.
     * @name y5.Types.nul
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    nul: function(obj) {
        return obj === null;
    },

    // user

    /**
     * Проверяет, является ли объект массивом. В случае успеха проверки возвращает true.
     * @name y5.Types.array
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    array: function(obj) {
        return obj instanceof Array;
    },

    /**
     * Проверяет, является ли объект регулярным выражением. В случае успеха проверки возвращает true.
     * @name y5.Types.regexp
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    regexp: function(obj) {
        return obj instanceof RegExp;
    },

    /**
     * Проверяет, является ли объект датой. В случае успеха проверки возвращает true.
     * @name y5.Types.date
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    date: function(obj) {
        return obj instanceof Date;
    },

    /**
     * Проверяет, является ли объект событием. В случае успеха проверки возвращает true.
     * @name y5.Types.event
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    event: function(obj) {
        // Правильно: obj instanceof Event
        // Для IE
        return obj && typeof obj.type != UNDEF && typeof (obj.stopPropagation || obj.cancelBubble) != UNDEF;
    },

    // dom

    /**
     * Проверяет, является ли объект элементом DOM. В случае успеха проверки возвращает true.
     * @name y5.Types.element
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    element: function(obj) {
        return checkDomNode(obj, Node.ELEMENT_NODE);
    },

    /**
     * Проверяет, является ли объект атрибутом DOM. В случае успеха проверки возвращает true.
     * @name y5.Types.attribute
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    attribute: function(obj) {
        return checkDomNode(obj, Node.ATTRIBUTE_NODE);
    },

    /**
     * Проверяет, является ли объект текстовым узлом DOM. В случае успеха проверки возвращает true.
     * @name y5.Types.text
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    text: function(obj) {
        return checkDomNode(obj, Node.TEXT_NODE);
    },

    /**
     * Проверяет, является ли объект документом. В случае успеха проверки возвращает true.
     * @name y5.Types.document
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    document: function(obj) {
        // для старых браузеров IE 5.0
        return (obj && typeof obj.documentElement != UNDEF) || false;

        // правильно
        // return checkDomNode(obj, Node.DOCUMENT_NODE);
    },

    /**
     * Проверяет, является ли объект узлом комментария. В случае успеха проверки возвращает true.
     * @name y5.Types.comment
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    comment: function(obj) {
        return checkDomNode(obj, Node.COMMENT_NODE);
    },

    /**
     * Проверяет, является ли объект узлом DOM. В случае успеха проверки возвращает true.
     * @name y5.Types.node
     * @memberOf y5.Types
     * @function
     * @param {Object} object Объект для проверки
     * @returns {Boolean} Результат проверки
     */
    node: function(obj) {
        return (obj && typeof obj.nodeType != UNDEF);
    }
};

})();/**
 * @class Модуль для реализации механизма &laquo;сборки мусора&raquo; &mdash; освобождения памяти от отработавших объектов, которые больше не будут востребованы приложением.
 * @name y5.GC
 * @static
 */
y5.GC = {
    /**
     * Массив объектов предназначенныз для удаления
     * @type Array
     * @name y5.GC.data
     * @memberOf y5.GC
     * @private
     */
    data: [],

    /**
     * Регистрирует объект в сборщике мусора. Возвращает зарегистрированный объект.
     * @name y5.GC.collect
     * @memberOf y5.GC
     * @function
     * @param {Object} obj объект
     * @returns {Object} сам объект
     */
    collect: function(obj) {
        this.data.push(obj);
        return obj;
    },

    /**
     * Удаляет объект из коллекции сборщика мусора.
     * @name y5.GC.remove
     * @memberOf y5.GC
     * @function
     * @param {Object} obj объект
     */
    remove: function(obj) {
        var i = this.data.indexOf(obj);
        if (i != -1) {
            this.destruct(i);
            this.data.splice(i, 1);
        }
    },

    /**
     * Вызывает деструктор сохраненного объекта.
     * @name y5.GC.destruct
     * @memberOf y5.GC
     * @function
     * @private
     */
    destruct: function(i) {
        var obj = this.data[i];
        if (obj) {
            if (typeof obj.cleanup == 'function') {
                obj.cleanup();
            } else if (typeof obj.destruct == 'function') {
                obj.destruct();
            }
        }
        this.data[i] = null;
    },

    /**
     * Удаляет все сохраненные объекты.
     * @name y5.GC.cleanup
     * @memberOf y5.GC
     * @function
     * @private
     */
    cleanup: function() {
        for (var i = this.data.length - 1; i >= 0; i--) {
            this.destruct(i);
        }
        this.data.length = 0;

        if (y5.is_ie && CollectGarbage) {
            CollectGarbage();
        }
    }
};
(function() {

var Types = y5.Types,
    UNDEF = y5.UNDEF,
    GC    = y5.GC;

// вызов функции с контекстом
function execListenerWithContext(listener, context, param1, param2) {
    return (context ? listener.call(context, param1, param2) : listener(param1, param2));
}

/// Обрабатываем не-DOM типы событий
var typeDOMAttrModified = 'DOMAttrModified',
    typePropertyChange = 'propertychange',
    typeDOMMouseScroll = 'DOMMouseScroll',
    typeMouseWheel = 'mousewheel',
    specificTypes = {};

if (document.attachEvent) { // IE
    specificTypes[typeDOMAttrModified] = typePropertyChange;
} else { // DOM
    specificTypes[typePropertyChange] = typeDOMAttrModified;
}

if (y5.is_ie || y5.is_opera || y5.is_safari) {
    // браузеры с событием колеса мыши 'mouseweel'
    specificTypes[typeDOMMouseScroll] = typeMouseWheel;
} else {
    specificTypes[typeMouseWheel] = typeDOMMouseScroll;
}

function normalizeType(type) {
    return specificTypes[type] || type;
}

/////////////////////////////////////////////////////////////////////

// функция для преобразования свойств Event
var normalizeEventSpecific,

    // коды клавиш мыши
    mouseButtonsListener = {L: [0, 65535], M: [1], R: [2]};

if (y5.is_ie) {

    function IE_preventDefault() {
        this.returnValue = false;
    }

    function IE_stopPropagation() {
        this.cancelBubble = true;
    }

    normalizeEventSpecific = function(e) {
        e.timeStamp = new Date().getTime();
        e.charCode = e.type == 'keypress' ? e.keyCode : 0;
        e.isChar = e.charCode > 0;
        e.target = e.srcElement;
        e.metaKey = e.altKey;
        e.attrName = e.propertyName == 'className' ? 'class' : e.propertyName;
        e.preventDefault = IE_preventDefault;
        e.stopPropagation = IE_stopPropagation;

        var doc = document.documentElement,
            body = document.body;
        e.pageX = e.clientX + (doc.scrollLeft || body.scrollLeft);
        e.pageY = e.clientY + (doc.scrollTop || body.scrollTop);

        switch (e.type) {
            case 'mouseout':
                e.relatedTarget = e.toElement;
                break;

            case 'mouseover':
                e.relatedTarget = e.fromElement;
                break;
        }

        // mouse scroll details
        e.scrollDetail = 0;
        if (e.wheelDelta) {
            e.scrollDetail = -e.wheelDelta / 40;
        }
    };

} else if (y5.is_safari) {

    mouseButtonsListener = {L: [0, 65535, 1 /* safari.ver <= 420 */], M: [2], R: [3]};

    normalizeEventSpecific = function(e) {
        // TODO! раскрыть смысл
        if (!Types.func(e.preventDefault)) {
            e.preventDefault = y5.NULL;
        }

        if (!Types.func(e.stopPropagation)) {
            e.stopPropagation = y5.NULL;
        }

        // Если событие произошло на текстовом узле (TEXT_NODE == 3, CDATA == 4)
        if (e.target && (e.target.nodeType == 3 || e.target.nodeType == 4)) {
            e.target = e.target.parentNode;
        }

        if (e.wheelDelta) {
            e.scrollDetail = -e.wheelDelta / 400;
        }
    };

} else if (y5.is_opera) {

    normalizeEventSpecific = function(e) {
        // mouse scroll details
        e.scrollDetail = 0;

        if (e.wheelDelta) {
            e.scrollDetail = e.wheelDelta / 40;
        }

        if (y5.opera_ver >= 9.2) {
            e.scrollDetail *= -1;
        }
    };

    if (y5.opera_ver < 8) {
        mouseButtonsListener = {L: [1], M: [2], R: [3]};
    }

} else {

    normalizeEventSpecific = function(e) {
        try {
            // mouse scroll details
            e.scrollDetail = e.detail;
        } catch (e) {}
    };
}

if (y5.is_ie || y5.is_konq) {
    mouseButtonsListener = {L: [1], M: [4], R: [2]};
}

function normalizeEvent(e) {
    var button;
    try {
        // ff1 на этот момент говорит [Exception... "Illegal operation on WrappedNative prototype object"  nsresult: "0x8057000c (NS_ERROR_XPC_BAD_OP_ON_WN_PROTO)" . Поэтому оборачиваем в try
        // видимо у MutationEvent нельзя обращаться к каким-то свойствам
        button = e.button;
    } catch (e) {}
    if (typeof button != UNDEF) {
        e.buttonL = mouseButtonsListener.L.indexOf(button) != -1;
        e.buttonM = mouseButtonsListener.M.indexOf(button) != -1;
        e.buttonR = mouseButtonsListener.R.indexOf(button) != -1;
    } else {
        e.buttonL = e.buttonM = e.buttonR = false;
    }

    normalizeEventSpecific(e);

    return e;
}

/////////////////////////////////////////////////////////////////////

/**
 * Создает обработчик DOM-события.
 * @constructor
 * @class Обеспечивает поддержку работы с событиями. Позволяет устанавливать обработчики на различные события браузера и DOM.
 * @name y5.AEventListener
 * @param {String} type Тип события: click, load...
 * @param {Function} listener Callback-функция на возникновение события
 * @param {Element} element Объект, на котором возникает событие
 * @param {Boolean} add Начать "слушать" событие немедленно
 * @param {Object} context Контекст, в котором выполняется callback. По-умолчанию, element
 * @param {Boolean} runOnce Обработать событие один раз и удалить обработчик. По умолчанию, false.
 *
 * @example
 * new y5.AEventListener("click", callback, link, true);
 */
y5.AEventListener = function(type, listener, element, add, context, runOnce) {
    this.type = normalizeType(type);
    this.element = element || document;
    this.blocked = false;
    this.added = false;
    this.runOnce = runOnce || false;
    
    var _this = this;
    this.listener = function(evt) {
        var e = typeof evt != UNDEF ? normalizeEvent(evt) : {};

        if (_this.blocked) {
            e.stopPropagation();
            e.preventDefault();
            return;
        }

        execListenerWithContext(listener, context, e, _this.element);
        if (_this.runOnce) {
            _this.cleanup();
        }
    };

    if (add) {
        this.add();
    }

    GC.collect(this);
};

y5.AEventListener.prototype = {
    /**
     * Включает отслеживание установленного ранее события.
     * @name y5.AEventListener.add
     * @memberOf y5.AEventListener
     * @function
     */
    add: function() {
        if (this.added) return;

        this._add();
        this.added = true;
    },

    /**
     * Добавляет обработчик к элементу
     * @name y5.AEventListener._add
     * @memberOf y5.AEventListener
     * @function
     * @private
     */
    _add: function() {
        this.element.addEventListener(this.type, this.listener, false);
    },

    /**
     * Выключает отслеживание события.
     * @name y5.AEventListener.remove
     * @memberOf y5.AEventListener
     * @function
     */
    remove: function() {
        if (!this.added) return;

        this._remove();
        this.added = false;
    },

    /**
     * Удаляет обработчик с элемента
     * @name y5.AEventListener._remove
     * @memberOf y5.AEventListener
     * @function
     * @private
     */
    _remove: function() {
        this.element.removeEventListener(this.type, this.listener, false);
    },

    /**
     * Блокирует событие. Перестает выполняться установленный обработчик события и отменяется стандартная реакция интерфейсных элементов на данное событие.
     * @name y5.AEventListener.block
     * @memberOf y5.AEventListener
     * @function
     */
    block: function() {
        this.blocked = true;
    },

    /**
     * Разблокирует событие. Установленный обработчик события начинает выполняться снова и восстанавливается стандартная реакция интерфейсных элементов на данное событие.
     * @name y5.AEventListener.unblock
     * @memberOf y5.AEventListener
     * @function
     */
    unblock: function() {
        this.blocked = false;
    },

    /**
     * Удаляет обработчик события, ссылки на элемент и callback-функцию.
     * @name y5.AEventListener.cleanup
     * @memberOf y5.AEventListener
     * @function
     * @private
     */
    cleanup: function() {
        this.remove();
        this.element = null;
        this.listener = null;
    }
};

var AEventListener = y5.AEventListener;

if (document.attachEvent) { // IE

    AEventListener.prototype._add = function() {
        this.element.attachEvent('on' + this.type, this.listener);
    };

    AEventListener.prototype._remove = function() {
        this.element.detachEvent('on' + this.type, this.listener);
    };

}

/// Создание событий
/////////////////////////////////////////////////////////////////////

    // Типы событий (w3)
var eventTypes = {
        Mouse: /^mouse|click/,
        Key: /^key/,
        Mutation: /^DOM/,
        HTML: /./
    },

    // Коды клавиш мыши
    mouseButtonsEvent = [0, 1, 2];

if (y5.is_safari) {
    mouseButtonsEvent = [0, 2, 3];
}

if (y5.is_safari || y5.is_opera) {
    // Создание KeyEvents вызывает исключение
    // Safari 3.0.4 (523.12.9)
    // Opera 9.5b1
    delete eventTypes['Key'];
}

/**
 * Создает экземпляр класса и, если не задан параметр dispatch, инициирует возникновение заданного в параметре type события.
 * @constructor
 * @class Позволяет инициировать возникновение стандартных событий браузера.
 * @name y5.Event
 * @param {String} [type] Тип события: click, mousedown... (если не указан, то используется click)
 * @param {Object} [element] Целевой элемент события (если не указан, то используется document)
 * @param {Boolean} [dispatch] Отправить событие (если не указано, то событие возникает сразу)
 * @param {Object} [params] Параметры события
 *
 * @example
 * new y5.Event("click", link);
 */
y5.Event = function(type, element, dispatch, params) {
    this.type = normalizeType(type || 'click');
    this.element = element || document;
    this.params = params || {};
    this.init();

    if (typeof dispatch == UNDEF) {
        dispatch = true;
    }

    if (dispatch) {
        this.dispatch();
    }

    GC.collect(this);
};

y5.Event.prototype = {
    /**
     * Инициализация.
     * @name y5.Event.init
     * @memberOf y5.Event
     * @function
     * @private
     */
    init: function() {
        for (var type in eventTypes) {
            if (eventTypes[type].test(this.type)) {
                this.eventType = type;
                break;
            }
        }
    },

    /**
     * Инициирует возникновение подготовленного ранее события.
     * @name y5.Event.dispatch
     * @memberOf y5.Event
     * @function
     * @param {Object} params параметры
     *
     * @example
     * var E = new y5.Event("click", link, false);
     * // ...
     * E.dispatch();
     */
    dispatch: function(params, canBubble, cancelable) {
        if (typeof params == UNDEF) {
            params = this.params;
        }

        canBubble = canBubble || true;
        cancelable = cancelable || true;

        // create
        var event = document.createEvent(this.eventType + 'Events');

        // init (по типу события)
        switch (this.eventType) {
            case 'Mouse':
                event.initMouseEvent(
                    this.type,
                    canBubble,
                    cancelable,
                    document.defaultView,
                    params.detail    || 0,
                    params.screenX   || 0,
                    params.screenY   || 0,
                    params.clientX   || 0,
                    params.clientY   || 0,
                    params.ctrlKey   || false,
                    params.altKey    || false,
                    params.shiftKey  || false,
                    params.metaKey   || false,
                    mouseButtonsEvent[params.button || 0 /* default left */],
                    null
                );
                break;

            case 'Key':
                event.initKeyEvent(
                    this.type,
                    canBubble,
                    cancelable,
                    document.defaultView,
                    params.ctrlKey   || false,
                    params.altKey    || false,
                    params.shiftKey  || false,
                    params.metaKey   || false,
                    params.keyCode   || 0,
                    params.charCode  || 0
                );
                break;

            default:
                event.initEvent(this.type, canBubble, cancelable);
                break;
        }

        // dispatch
        return this.element.dispatchEvent(event);
    },

    /**
     * Удаляет ссылку на элемент
     * @name y5.Event.cleanup
     * @memberOf y5.Event
     * @function
     * @private
     */
    cleanup: function() {
        this.element = null;
    }
};

var Event = y5.Event;

if (document.createEventObject) { // IE

    mouseButtonsEvent = [1, 4, 2];

    function setEvent(event, params) {
        for (var name in params) {
            try {
                var value;
                switch (name) {
                    case 'button':
                        value = mouseButtonsEvent[params.button || 0 /* default left */];
                        break;

                    default:
                        value = params[name];
                        break;
                }
                event[name] = value;
            } catch(e) {}
        }
        return event;
    }

    Event.prototype.init = y5.NULL;

    Event.prototype.dispatch = function(params) {
        if (typeof params == UNDEF) {
            params = this.params;
        }

        var event = setEvent(document.createEventObject(), params);
        return this.element.fireEvent('on' + this.type, event);
    };

} else if (!document.createEvent) {

    Event.prototype.init = y5.NULL;

    Event.prototype.dispatch = function(params) {
        try {
            return this.element[this.type](params);
        } catch (e) {
            y5.Console.warn('Browser is too old', ['Event']);
            return false;
        }
    };
}

/**
 * @memberOf y5.Event
 * @name y5.Event.dispatchEvent
 * @function
 * @deprecated y5.Event.dispatch
 */
Event.prototype.dispatchEvent = Event.prototype.dispatch;


/// Observer & Notifier
/////////////////////////////////////////////////////////////////////
(function() {

var ObserverManager = {
    // список постоянных событий
    notifiers: {},
    observers: {},

    dispatchNotify: function(id, params, permanent) {
        var result = true,
            observers = this.observers[id];

        if (observers) {
            // надо делать копию массива обработчиков, чтобы при удалении не ехали индексы и не было пропусков.
            var cloneObservers = [].concat(observers),
                i = 0,
                length = cloneObservers.length;
                
            // проходимся по массиву и вызываем обработчики
            for (; i<length; i++) {
                if (!cloneObservers[i].added) continue;
                result &= this.runListener(cloneObservers[i], params);
                // если обработчик одноразовый, то вызываем cleanup
                // непосредственно из массива удалять обработчик по его индексу (i) нельзя, т.к. если до этого уже был кто-то удален, он не совпадет  
                if (cloneObservers[i].runOnce) {
                    cloneObservers[i].cleanup();
                }
            }
        }

        if (permanent) {
            var notifiers = this.notifiers;
            if (!notifiers[id]) {
                notifiers[id] = [];
            }
            notifiers[id].push(params);
        }

        return result;
    },

    runListener: function(observer, params) {
        var result = true;

        result &= execListenerWithContext(observer.listener, observer.context, params);
        y5.Console.log('Observer listener: ', observer, ['Notifier']);

        return result;
    },

    addObserver: function(observer) {
        var id = observer.id,
            observers = this.observers;

        if (!observers[id]) {
            observers[id] = [];
        }
        observers[id].push(observer);

        // запускаем выполнение для "постоянных" событий
        this.fireNotify(observer);
    },

    removeObserver: function(observer) {
        var observers = this.observers[observer.id],
            index;

        if (observers) {
            for (var i=0, length = observers.length; i < length; i++) {
                if (observers[i] === observer) {
                    observers.splice(i, 1);
                    break;
                }
            }
        }
    },
    
    fireNotify: function(observer) {
        var notifiers = this.notifiers[observer.id];

        if (notifiers) {
            for (var i = 0, l = notifiers.length; i < l; i++) {
                this.runListener(observer, notifiers[i]);
            }
        }
    },

    generateId: function(type, element) {
        var id = element;
        if (!element || typeof element == 'object') {
            id = y5.Utils.getUniqueId(element || y5);
        }
        return type + '_' + id;
    }
};

/**
 * Создает объект-слушатель, реагирующий на пользовательское событие заданного типа.
 * @class Класс для реализации совместно с классом y5.Notifier механизма пользовательских событий. Позволяет расширить стандартный набор событий браузера новыми, проблемно-ориентированными типами. Класс y5.Observer отвечает за создание слушателей — объектов, отслеживающих возникновение пользовательских событий.
 * @constructor
 * @name y5.Observer
 * @param {String} type Тип события
 * @param {Function} listener Callback-функция
 * @param {Object} object Объект, на котором возникает событие
 * @param {Boolean} add Начать слушать немедленно
 * @param {Object} context Контекст выполнения функции listener
 * @param {Boolean} runOnce Обработать событие один раз и удалить обработчик. По умолчанию, false.
 *
 * @example
 * new y5.Observer("customEvent", callback, object, true);
 */
y5.Observer = function(type, listener, element, add, context, runOnce) {
    this.id = ObserverManager.generateId(type, element);
    this.added = false;
    this.listener = listener;
    this.context = context;
    this.runOnce = runOnce || false;
    
    if (add) {
        this.add();
    }

    GC.collect(this);
};

y5.Observer.prototype = {
    /**
     * Включает (активирует) объект-слушатель.
     * @name y5.Observer.add
     * @memberOf y5.Observer
     * @function
     */
    add: function() {
        if (!this.added) {
            ObserverManager.addObserver(this);
            this.added = true;

            y5.Console.log('Observer added: ', this, ['Observer']);
        }
    },

    /**
     * Выключает (деактивирует) объект-слушатель.
     * @name y5.Observer.remove
     * @memberOf y5.Observer
     * @function
     */
    remove: function() {
        if (this.added) {
            ObserverManager.removeObserver(this);
            this.added = false;

            y5.Console.log('Observer removed: ', this, ['Observer']);
        }
    },

    /**
     * Удаляет обработчик события, ссылки на элемент и callback-функцию.
     * @name y5.Observer.cleanup
     * @memberOf y5.Observer
     * @function
     * @private
     */
    cleanup: function() {
        this.remove();
        this.context = null;
        this.listener = null;
    }
};

var ObserverProto = y5.Observer.prototype;

/**
 * @memberOf y5.Observer
 * @name y5.Observer.start
 * @function
 * @deprecated y5.Observer.add
 */
ObserverProto.start = ObserverProto.add;

/**
 * @memberOf y5.Observer
 * @name y5.Observer.stop
 * @function
 * @deprecated y5.Observer.remove
 */
ObserverProto.stop = ObserverProto.remove;

/**
 * Подготовливает пользовательское событие и, если параметр dispatch равен true, инициирует его возникновение.
 * @class Класс для реализации совместно с классом y5.Observer механизма пользовательских событий. Позволяет расширить стандартный набор событий браузера новыми, проблемно-ориентированными типами. Класс y5.Notifier отвечает за инициализацию событий и отправку сообщений объектам-слушателям, созданным при помощи класса y5.Observer.
 * @constructor
 * @name y5.Notifier
 * @param {String} type Тип события
 * @param {Object} object Объект, на котором возникает событие
 * @param {Boolean} [dispatch] Послать сообщение немедленно (true - по умолчанию)
 * @param {Object} [params] Параметры сообщения
 *
 * @example
 * new y5.Notifier("customEvent", object, true, {foo: 'bar'});
 */
y5.Notifier = function(type, element, dispatch, params) {
    this.id = ObserverManager.generateId(type, element);
    this.params = params;

    y5.Console.log('Notifier new: ', this, ['Notifier']);

    if (typeof dispatch == UNDEF) {
        dispatch = true;
    }

    if (dispatch) {
        this.dispatch();
    }
};

y5.Notifier.prototype = {
    /**
     * Инициирует возникновение события - отправляет сообщение. Возвращает true, если общий результат выполнения функций объекта-слушателя равен true.
     * @name y5.Notifier.dispatch
     * @memberOf y5.Notifier
     * @function
     * @param {Object} [params] Параметры, передаваемые функции слушателя
     * @param {Boolean} [permanent] "Постоянное" событие
     * @returns {Boolean} Общий результат выполнения функций слушателя
     *
     * @example
     * var notify = new y5.Notifier("customEvent", object, false, {foo: 'bar'});
     *
     * // передать слушателю новые параметры
     * notify.dispatch({foo: 'yadda'});
     *
     * // передать слушателю параметры заданные при создании
     * notify.dispatch();
     */
    dispatch: function(params, permanent) {
        if (typeof params == UNDEF) {
            params = this.params;
        }

        y5.Console.group('Notifier dispatch: ', [this, params], ['Notifier']);
        var result = ObserverManager.dispatchNotify(this.id, params, permanent);
        y5.Console.groupEnd();

        return result;
    }
};

/**
 * Инициирует возникновение пользовательского события. Возвращает true, если общий результат выполнения функций объекта-слушателя равен true.
 * @class Класс для создания пользовательского события. Упрощенный вариант y5.Notifiler.
 * @name y5.Notify
 * @param {String} type Тип события
 * @param {Object} object Объект, на котором возникает событие
 * @param {Object} [params] Параметры, передаваемые функции слушателя
 * @param {Boolean} [permanent] "Постоянное" событие
 * @returns {Boolean} Общий результат выполнения функций слушателя
 *
 * @example
 * y5.Notify("customEvent", object, {foo: 'bar'});
 */
y5.Notify = function(type, element, params, permanent) {
    y5.Console.group('Notifier dispatch: ', [this, params], ['Notifier']);
    var id = ObserverManager.generateId(type, element);
    var result = ObserverManager.dispatchNotify(id, params, permanent);
    y5.Console.groupEnd();

    return result;
};

})();

/// Events Factory
/////////////////////////////////////////////////////////////////////

/**
 * Создает один объект-слушатель.
 * @function
 * @private
 */
function observeEvent (type, listener, element, add, context, runOnce) {
    var obj = isCustom(type) ? y5.Observer : AEventListener;
    return new obj(type, listener, element, add, context, runOnce);
}

function isCustom (type) {
    return type.indexOf(':') != -1;
}

/**
 * @class Набор функций для инициализации стандартных событий браузера, изменения их свойств, а также для создания слушателей — объектов, отслеживающих возникновение данных событий.
 * @static
 * @name y5.Events
 */
y5.Events = {
    /**
     * Создает один или несколько объектов-слушателей.
     * @name y5.Events.observe
     * @memberOf y5.Events
     * @function
     * @param {String | Array} types Тип события: click, load...
     * @param {Function} listener Callback-функция на возникновение события
     * @param {Element | Object} element Объект, на котором возникает событие
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @param {Boolean} [runOnce] Обработать событие один раз и удалить обработчик. По умолчанию, false.
     * @returns {y5.AEventListener | y5.Observer | Array}
     *
     * @example
     * y5.Events.observe("click", callback, element, true);
     */
    observe: function(types, listener, element, add, context, runOnce) {
        if (!element) {
            return {add: y5.NULL, remove: y5.NULL};
        }

        switch (Types.type(types)) {
            case Types.ARRAY:
                var length = types.length,
                    events = new Array(length),
                    i=0;

                for (; i < length; i++) {
                    events[i] = observeEvent(types[i], listener, element, add, context, runOnce);
                }
                
                return events;

            case Types.STRING:
                return observeEvent(types, listener, element, add, context, runOnce);
        }

        return null;
    },
    
    /**
     * Создает один или несколько одноразовых объектов-слушателей, обработчики удаляются после наступления события.
     * @name y5.Events.observeOnce
     * @memberOf y5.Events
     * @function
     * @param {String | Array} types Тип события: click, load...
     * @param {Function} listener Callback-функция на возникновение события
     * @param {Element | Object} element Объект, на котором возникает событие
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @returns {y5.AEventListener | y5.Observer | Array}
     *
     * @example
     * y5.Events.observe("click", callback, element, true);
     */
    observeOnce: function (types, listener, element, add, context) {
        return this.observe(types, listener, element, add, context, true);
    },

    /**
     * Создает событие и, если не задан параметр dispatch, инициирует его  возникновение.
     * @name y5.Events.notify
     * @memberOf y5.Events
     * @function
     * @param {String} [type] Тип события: click, mousedown... (если не указан, то используется click)
     * @param {Object} [element] Целевой элемент события (если не указан, то используется document)
     * @param {Boolean} [dispatch] Отправить событие (если не указано, то событие возникает сразу)
     * @param {Object} [event] Параметры события
     * @param {Boolean} [permanent] "Постоянное" событие
     * @returns {y5.Event | y5.Notifier}
     *
     * @example
     * // создать событие "mousedown" на элементе
     * y5.Events.notify("mousedown", element);
     *
     * // создать событие "click" на документе
     * y5.Events.notify();
     *
     * // создать событие "mousedown" на элементе
     * var evt = y5.Events.notify("mousedown", element, false);
     * // ...
     * // выполнить его позже
     * // evt.dispatch()
     */
    notify: function(type, element, dispatch, event, permanent) {
        var obj = isCustom(type) ? y5.Notifier : y5.Event;
        return new obj(type, element, dispatch, event, permanent);
    },

    /**
     * Создает объект-слушатель, реагирующий на изменение определенного свойства элемента (например, на изменение содержимого поля ввода).
     * @name y5.Events.observeProperty
     * @memberOf y5.Events
     * @function
     * @param {String} property Свойство
     * @param {Function} listener Callback-функция
     * @param {Element} element Элемент
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @param {Boolean} [runOnce] Обработать событие один раз и удалить обработчик. По умолчанию, false.
     * @returns {y5.AEventListener}
     * @note Не работает в Safari 3.0.4, Firefox 1.0
     *
     * @example
     * y5.Events.observeProperty("value", callback, element, true);
     */
    observeProperty: function(property, listener, element, add, context, runOnce) {
        // Внимание! Не используем контекст для listenerAttr,
        // только для listener
        function listenerAttr(evt) {
            if (evt.attrName == property) {
                execListenerWithContext(listener, context, evt, element);
            }
        }
        return new AEventListener(typeDOMAttrModified, listenerAttr, element, add, null, runOnce);
    },
    
    /**
     * Одноразовый слушатель изменения свойств элемента. После наступления события обработчик удаляется.
     * @name y5.Events.observePropertyOnce
     * @memberOf y5.Events
     * @function
     * @param {String} property Свойство
     * @param {Function} listener Callback-функция
     * @param {Element} element Элемент
     * @param {Boolean} [add] Начать слушать немедленно (false - по умолчанию)
     * @param {Object} [context] Контекст выполнения функции listener
     * @returns {y5.AEventListener}
     * @note Не работает в Safari 3.0.4, Firefox 1.0
     *
     * @example
     * y5.Events.observeProperty("value", callback, element, true);
     */
    observePropertyOnce: function(property, listener, element, add, context) {
        // Внимание! Не используем контекст для listenerAttr,
        // только для listener
        function listenerAttr(evt) {
            if (evt.attrName == property) {
                execListenerWithContext(listener, context, evt, element);
            }
        }
        return new AEventListener(typeDOMAttrModified, listenerAttr, element, add, null, true);
    }
};

var Events = y5.Events;

// aliases

/**
 * Создает один или несколько объектов-слушателей.
 * @name y5.on
 * @see y5.Events.observe
 * @memberOf y5
 * @function
 */
y5.on = Events.observe;

/**
 * Создает событие и инициирует его возникновение.
 * @name y5.fire
 * @see y5.Events.notify
 * @memberOf y5
 * @function
 */
y5.fire = Events.notify;

// deprecated
    
/**
 * @memberOf y5.Events
 * @name y5.Events.create
 * @function
 * @deprecated y5.Events.observe
 */
Events.create = Events.observe;

/**
 * @memberOf y5.Events
 * @name y5.Events.make
 * @function
 * @deprecated y5.Events.notify
 */
Events.make = Events.notify;

/**
 * @memberOf y5.Events
 * @name y5.Events.PropertyListener
 * @function
 * @deprecated y5.Events.observeProperty
 */
Events.PropertyListener = Events.observeProperty;


/// dom:loaded notify
/////////////////////////////////////////////////////////////////////
(function() {

var timer, type = 'load';

function notifyDomLoaded() {
    if (y5.domloaded) {
        return;
    }

    // устанавливаем глобальный флаг
    y5.domloaded = true;

    if (timer) {
        window.clearInterval(timer);
    }

    // обнуляем эту функцию, чтобы вызов был только один раз
    notifyDomLoaded = y5.VOID;

    // посылаем сообщение и делаем его постоянным
    y5.Notify('dom:loaded', y5, null, true);
}

if (document.addEventListener) {

    if (y5.is_safari || y5.is_khtml) {
        var loadedRegex = /loaded|complete/;

        timer = window.setInterval(function() {
            if (loadedRegex.test(document.readyState)) {
                notifyDomLoaded();
            }
        }, 0);
    } else if (y5.is_opera && y5.opera_ver < 9) {
        ;
    } else {
        type = 'DOMContentLoaded';
    }

} else {
    var doc = document;

    function isSet(property) {
        return typeof doc[property] != 'undefined';
    }

    function poll() {
        if (doc.body !== null && doc.getElementsByTagName) {
            if (isSet('readyState') && (/loaded|complete/).test(doc.readyState)) {
                notifyDomLoaded();
            }
            if (isSet('fileSize')) {
                try {
                    doc.documentElement.doScroll('left');
                    notifyDomLoaded();
                } catch (e) {}
            }
        }
    }

    timer = window.setInterval(poll, 10);
}

y5.Events.observe(type, notifyDomLoaded, window, true);

})();

/// Init GC
/////////////////////////////////////////////////////////////////////

// В старых gecko-браузерах вызывает крах
if (!(y5.is_gecko && y5.gecko_ver < 1.8 || y5.is_opera)) {
    // Register the cleanup handler on page unload
    new AEventListener('unload', GC.cleanup, window, true, GC);
}

})();(function() {

var Types = y5.Types,
    NOT_OBJECT = Types.NODE | Types.BOOLEAN | Types.NUMBER | Types.STRING | Types.EVENT | Types.REGEXP | Types.FUNCTION | Types.NULL;

/**
 * @class Набор полезных функций-утилит общего назначения.
 * @static
 * @name y5.Utils
 */
y5.Utils = {
    /**
     * Счетчик генерируемых ID.
     * @name y5.Utils.counterId
     * @memberOf y5.Utils
     * @type Number
     * @private
     */
    counterId: 0,

    /**
     * Генерирует случайный ID. Возвращает результат в виде строки символов.
     * @name y5.Utils.generateId
     * @memberOf y5.Utils
     * @function
     * @param {String} [prefix] Префикс (по умолчанию = '')
     * @returns {String} Случайный ID
     */
    generateId: function(prefix) {
        return (prefix || '') + ((new Date()).getTime() + Math.round(Math.random() * 10000));
    },

    /**
     * Генерирует уникальный ID. Возвращает результат в виде строки символов.
     * @name y5.Utils.generateUniqueId
     * @memberOf y5.Utils
     * @function
     * @returns {String} Уникальный ID
     *
     * @example
     * y5.Utils.generateUniqueId();
     * // -> 'y5__id45'
     */
    generateUniqueId: function() {
        return 'y5__id' + (++this.counterId);
    },

    /**
     * Генерирует уникальный ID и присваивает его свойству uniqueID переданного объекта.
     * Если свойство uniqueID объекта существует, то значение этого свойства не изменяется. Возвращает значение uniqueID в виде строки символов.
     * @name y5.Utils.getUniqueId
     * @memberOf y5.Utils
     * @function
     * @param {Object} object Объект
     * @returns {String} Уникальный ID
     */
    getUniqueId: function(object) {
        if (object === document) {
            return this.documentID;
        }
        return object.uniqueID || this.setUniqueId(object);
    },

    /**
     * Присваивает свойству uniqueID объекта переданное значение. Если параметр uniqueID не определен, то генерируется новый уникальный идентификатор и полученное значение присваивается свойству uniqueID объекта. Возвращает значение свойства uniqueID в виде строки символов.
     * @name y5.Utils.setUniqueId
     * @memberOf y5.Utils
     * @function
     * @param {Object} object Объект
     * @param {String} [uniqueID] Уникальный ID (если неопределен, то генерируется новый)
     * @returns {String} Уникальный ID
     *
     * @example
     * y5.Utils.setUniqueId(object, 'foo');
     * // -> 'foo'
     *
     * y5.Utils.setUniqueId(object);
     * // -> 'y5__id46'
     */
    setUniqueId: function(object, uniqueID) {
        if (typeof uniqueID == y5.UNDEF) {
            uniqueID = this.generateUniqueId();
        }
        return (object.uniqueID = uniqueID);
    },

    /**
     * Сравнивает два объекта. Возвращает true, если оба объекта на самом деле - это один и тот же объект.
     * @name y5.Utils.isEqual
     * @memberOf y5.Utils
     * @function
     * @param {Object} first Объект
     * @param {Object} second Объект
     * @returns {Boolean} Результат сравнения
     */
    isEqual: function(first, second) {
        return this.getUniqueId(first) == this.getUniqueId(second);
    },

    /**
     * Решает проблему браузера IE, связанную с некорректной отрисовкой тега select и объекта Flash, в результате которой эти визуальные элементы перекрывают любые другие элементы страницы, даже находящиеся на  слоях выше select или Flash. Проблема решается путем создания элемента iframe и «подкладывания» его под элемент (обычно div), который необходимо корректно отобразить поверх select или Flash.
     * @name y5.Utils.fakeFrame
     * @memberOf y5.Utils
     * @type Object
     */
    fakeFrame: {
        frame: null,
        transparentFrame: null,

        init: function (obj) {
            if (!y5.is_ie7down) {
                return;
            }
            this.frame = y5.Dom.$('fakeFrame');
            if (!this.frame) {
                this.create(obj);
            }

            return [this.frame, this.transparentFrame];
        },

        create: function (obj) {
            this.transparentFrame = document.createElement('<iframe id="fakeTransparentFrame" src="about:blank" frameborder="0" tabindex="-1" style="filter:Alpha(opacity=1); position: absolute;">');
            this.frame = document.createElement('<iframe id="fakeFrame" src="about:blank" frameborder="0" tabindex="-1" allowtransparency="true" style="FILTER: chroma(color=#FFFFFF); position: absolute;">');
            this.transparentFrame.style.zIndex = y5.Elements.css(obj, 'z-index') - 2;
            this.frame.style.zIndex = y5.Elements.css(obj, 'z-index') - 1;
            this.frame.style.display = this.transparentFrame.style.display = 'none';
            y5.Dom.getBody().appendChild(this.transparentFrame);
            y5.Dom.getBody().appendChild(this.frame);
        },

        adjust: function (obj) {
            if (!this.frame) {
                return;
            }

            if (obj.offsetHeight > 0) {
                if (this.transparentFrame.style.zIndex == -2) {
                    this.transparentFrame.style.zIndex = y5.Elements.css(obj, 'z-index') - 2;
                    this.frame.style.zIndex = y5.Elements.css(obj, 'z-index') - 1;
                }
                this.frame.style.width = this.transparentFrame.style.width = obj.offsetWidth + 'px';
                this.frame.style.height = this.transparentFrame.style.height = obj.offsetHeight + 'px';
                this.frame.style.left = this.transparentFrame.style.left = obj.offsetLeft + 'px';
                this.frame.style.top = this.transparentFrame.style.top = obj.offsetTop + 'px';
            }
            this.frame.style.display = this.transparentFrame.style.display = obj.style.display;
        }
    },

    /**
     * Массив символов шестнадцатиричной системы счисления.
     * @private
     * @name y5.Utils.hexDigit
     * @memberOf y5.Utils
     * @type Array
     */
    hexDigit: '0123456789ABCDEF'.split(''),

    /**
     * Преобразует десятичное число в двузначное шестнадцатеричное. Возвращает полученное значение в виде строки.
     * @name y5.Utils.dec2hex
     * @memberOf y5.Utils
     * @function
     * @param {Number} number Число
     * @returns {String} 16-тиричное число
     *
     * @example
     * y5.Utils.dec2hex(10);
     * // -> '0A'
     */
    dec2hex: function(n) {
        return this.hexDigit[n >> 4] + this.hexDigit[n & 15];
    },

    /**
     * Преобразует шестнадцатеричное число в десятичное. Возвращает результат преобразования.
     * @name y5.Utils.hex2dec
     * @memberOf y5.Utils
     * @function
     * @param {String} number 16-тиричное число
     * @returns {Number} Десятичное число
     *
     * @example
     * y5.Utils.hex2dec('A');
     * // -> 10
     */
    hex2dec: function(n) {
        return parseInt(n, 16);
    },

    /**
     * Поочередно копирует свойства нескольких объектов в новый. Возвращает объект, получившийся в результате слияния исходных.
     * @name y5.Utils.objectCopy
     * @memberOf y5.Utils
     * @function
     * @param {Object} object1 Первый объект
     * @param {Object} object2 Второй объект
     * @param {Object} objectN n-объект
     * @returns {Object} Объект, получившийся в результате слияния объектов
     *
     * @example
     * y5.Utils.objectCopy({a: 1, b: 3}, {b: 2, c: 3});
     * // -> {a: 1, b: 2, c: 3}
     */
    objectCopy: function(destination) {
        var key,
            result = {},
            argsCnt = arguments.length;

        for (key in destination) {
            result[key] = destination[key];
        }

        if (argsCnt == 1) {
            return result;

        } else if (argsCnt == 2) {
            var source = arguments[1],
                src,
                data;
            for (key in source) {
                src = source[key];
                data = {};

                if (Types.test(src, NOT_OBJECT)) {
                    data = src;
                // Date
                } else if (Types.date(src)) {
                    data = new Date(src);
                // Array
                } else if (Types.array(src)) {
                    data = [].concat(src);
                // Object
                } else {
                    if (Types.def(result[key])) {
                        data = result[key];
                    }
                    data = this.objectCopy(data, src);
                }

                result[key] = data;
            }

        } else {
            key=1;
            for (; key<argsCnt; key++) {
                result = this.objectCopy(result, arguments[key]);
            }
        }
        return result;
    },

    /**
     * Расширяет функции прототипа первого объекта функциями прототипа второго объекта.
     * @name y5.Utils.objectExtends
     * @memberOf y5.Utils
     * @function
     * @param {Object} destination Расширяемый объект
     * @param {Object} source Объект с функциями расширения
     * @param {Object} sourceName Имя объекта расширения
     */
    objectExtends: function(destination, source, source_name) {
        source_name = source_name || source.toString().match(/function\s*([^\(]+)\(/)[1];

        var key,
            destProto = destination.prototype,
            sourceProto = source.prototype;

        destProto[source_name] = source;

        for (key in sourceProto) {
            // если такого свойства нет в destination, то копируем его из source
            if (Types.undef(destProto[key])) {
                destProto[key] = sourceProto[key];

            // если свойство в destination и в source - объекты, то объединяем их  через objectCopy    
            } else if (Types.object(destProto[key]) && Types.object(sourceProto[key])) {
                destProto[key] = this.objectCopy(sourceProto[key], destProto[key]);
            }
            /*
             // если свойство есть, то ничего не делаем
             } else {}
            */
        }
    },

    /**
     * Запускает выполнение метода указанного объекта с периодичностью, заданной в параметре timeout. Возвращает уникальный ID таймера.
     * @name y5.Utils.setTimeout
     * @memberOf y5.Utils
     * @function
     * @param {Function} method Метод
     * @param {Number} timeout Таймаут (мсек)
     * @param {Object} [context] Объект, в контексте которого запускается функция (по умолчанию null)
     * @param {Object} [arg1] Аргумент вызываемой функции
     * @param {Object} [arg2]
     * @param {Object} [...]
     * @param {Object} [argN]
     * @returns {ID} ID таймера
     */
    setTimeout: function(func, timeout, context /* args */) {
        var args = [];

        for (var i = 3, l = arguments.length; i < l; i++) {
            args.push(arguments[i]);
        }

        function call() {
            func.apply(context, args);
        }

        return window.setTimeout(call, timeout);
    },

    /**
     * Форматирует число в строку с разделителем
     * @name y5.Utils.formatNumber
     * @memberOf y5.Utils
     * @function
     * @param {Number|String} num Число
     * @param {String} [exponent_delimiter] Разделитель порядков. По умолчанию: пробел
     * @param {String} [float_delimiter] Разделитель дробной части. По умолчанию: точка
     * @returns {String} Отформатированное число
     * @example
     * y5.Utils.formatNumber('10000', ' ');
     * // -> '10 000'
     *
     * y5.Utils.formatNumber('-10000.02', ',', '.');
     * // -> '-10,000.02'
     *
     */
    formatNumber: function (num, exponent_delimiter, float_delimiter) {
        num = parseFloat(num.toString(), 10);
        if (isNaN(num)) {
            return;
        }

        exponent_delimiter = Types.string(exponent_delimiter) ? exponent_delimiter : ' ';
        float_delimiter = float_delimiter ? float_delimiter : '.';

        // определяем знак
        var sgn = num < 0 ? '-' : '',
        // отделяем дробную часть
            strNum = num.toString(),
            pointIndex = strNum.indexOf('.'),
            floatPart = 0;
        if (pointIndex != -1) {
            floatPart = strNum.substr(pointIndex + 1);
        }

        // берем число без знака и отсекаем дробную часть
        num = Math.floor(Math.abs(num)).toString();

        var startIndex = num.length % 3,
            result = num.substr(0, startIndex),
            cnt = Math.floor(num.length / 3),
            i = 0;
        for (; i < cnt; i++) {
            result += exponent_delimiter + num.substr(3 * i + startIndex, 3);
        }

        // если количество цифр кратно 3, то убираем первый exponent_delimiter
        if (startIndex == 0) {
            result = result.substr(1);
        }

        if (floatPart) {
            result += float_delimiter + floatPart;
        }

        return sgn + result;
    }
};

y5.Utils.documentID = y5.Utils.generateId('y5__');

/**
 * @name y5.Utils.getUniqueID
 * @memberOf y5.Utils
 * @function
 * @deprecated y5.Utils.getUniqueId
 */
y5.Utils.getUniqueID = y5.Utils.getUniqueId;

y5.loaded('Utils');

})();/// Init
/////////////////////////////////////////////////////////////////////

// Устанавливаем namespace y5
y5.registerNamespace('y5', 'y5.js');
y5.loaded('Types');
y5.loaded('Events');

if (/y5debug/.test(location.search + document.cookie)) {
    y5.require('Debug');
}